#pragma once


#define NO_CHANGE           -1

//
// Class which handles a list of tracks and their relationships (ie., when one
// ends, which is the next to go to)
//
class CAnimMgr
{
public:

    // Constructors / destructors
    CAnimMgr() :
        m_dwNumTracks(0), m_dwCurrentTrackIndex(-1), m_fOverallSpeed((FLOAT) 1.0), m_boChainTracks(false)
                                                        { }
    virtual ~CAnimMgr();

    // Accessors
    DWORD       dwGetNumTracks()                        { return m_dwNumTracks; }
    CAnimTrack* lpGetTrack(DWORD dwTrackIndex)          { ASSERT(dwTrackIndex >= 0  &&  dwTrackIndex < m_dwNumTracks); return m_ListTracks[dwTrackIndex].m_lpTrack; }
    CAnimTrack* lpGetCurrentTrack()                     { if(m_dwCurrentTrackIndex == -1) return NULL; ASSERT(m_dwCurrentTrackIndex >= 0  &&  m_dwCurrentTrackIndex < m_dwNumTracks); return m_ListTracks[m_dwCurrentTrackIndex].m_lpTrack; }
    DWORD       dwGetCurrentTrack()                     { return m_dwCurrentTrackIndex; }
    void        vGetNextAnimInfos(DWORD& dwNextTrackIndex,DWORD& dwNextTrackFrame,DWORD dwTrackIndex = -1);

    void        vSetOverallSpeed(FLOAT fOverallSpeed);
    void        vSetChainTracks(bool boChain);
    void        vSetNextTrack(DWORD dwTrackIndex, DWORD dwNextTrackIndex = NO_CHANGE, DWORD dwNextTrackFrame = NO_CHANGE);
    void        vPushTrack(CAnimTrack* lpTrack);

    // Operations
    void        vSetCurrentTrack(DWORD dwTrackIndex,FLOAT fFrame = (FLOAT) 0.0);
    bool        boSetCurrentFrame(FLOAT fFrame);
    bool        boRunForward(FLOAT fElapsedTime);
    void        vStealTrack( DWORD dwTrackDest, CAnimMgr& AnimMgr, DWORD dwTrackSrc );

private:

    // Class used to manage the list of the tracks
    class CAnimMgrTrack
    {
    public:

        CAnimMgrTrack() : m_lpTrack( NULL ), m_dwTrackIndex( 0 ), m_dwNextTrackIndex( 0 ), m_dwNextTrackFrame( 0 ) { }
        CAnimMgrTrack(CAnimTrack* lpTrack, DWORD dwTrackIndex) :
            m_lpTrack(lpTrack), m_dwTrackIndex(dwTrackIndex),
            m_dwNextTrackIndex(dwTrackIndex), m_dwNextTrackFrame(0)
                                                        { }

        CAnimTrack*         m_lpTrack;
        DWORD               m_dwTrackIndex;
        DWORD               m_dwNextTrackIndex;
        DWORD               m_dwNextTrackFrame;
    };

    typedef std::vector<CAnimMgrTrack> AnimListTracks;

    void        vLinkToNextTrack();

    // Data members
    DWORD               m_dwNumTracks;                  // num tracks
    FLOAT               m_fOverallSpeed;                // allow modifications of speed for all anims
    bool                m_boChainTracks;                // true if the next animation is selected according to the "NextAnimation" field, false to loop the current animation
    AnimListTracks      m_ListTracks;                   // list of tracks

    DWORD               m_dwCurrentTrackIndex;          // current track being played
};
