#include "stdafx.h"

#include "camera.h"

using namespace std;

void CAMERA::Cam2Phigs(void)
{
  vpn         = LookAt-CamPos; // normal to the proj. plane
  float d     = vpn.length();
  prp         = vec3(0,0,-d);  // position of the ref. pt (proj. center)
  vpn.normalize();
  vup         = ViewUp;
  umax = vmax = (float)(d*tan(fov/2.0f));// extends of the window in the proj. plane
  vmax       *= RatioYX;
}

void CAMERA::CamCalcMatrix(void)
{
  Cam2Phigs();

  mat4 translat = identity3D();    // translate to the "look at" position
  translat[VX][VW] = -LookAt[VX];
  translat[VY][VW] = -LookAt[VY];
  translat[VZ][VW] = -LookAt[VZ];

  mat4 r = identity3D();    // build a matrix to have :
  vec3 rz = vpn;            // new axe z = vpn
  vec3 ry = vup;            // new axe y = vup
  vec3 rx = (ry ^ rz);      // ^ = cross product
  rx.normalize();
  r[VX] = vec4(rx,0);
  r[VY] = vec4(ry,0);
  r[VZ] = vec4(rz,0);
  matrix = r*translat;

  mat4 translat2 = identity3D(); // translate to the prp = pos. of observer
  translat2[VX][VW] = -prp[VX];
  translat2[VY][VW] = -prp[VY];
  translat2[VZ][VW] = -prp[VZ];
  matrix = translat2*matrix;

  mat4 sper = identity3D();  // scale along x and y to have a pyramid with
  float crvzp = -prp[VZ];    // top angle = 90
  float sx = crvzp/umax;
  float sy = crvzp/vmax;
  sper[VX][VX] = sx;
  sper[VY][VY] = sy;
  matrix = sper*matrix;

  mat4 m = identity3D();     // translate along z so z<z_front_clip => z<0
  m[VZ][VW] = -front_pd;
  m[VW][VZ] = 1;
  m[VW][VW] = 0;
  matrix = m*matrix;

  mat4 sc = identity3D();    // scale & translate along x & y so that
  sc[VX][VW] = 0.5;          // x_ndc & y_ndc are between 0 & 1 instead of
  sc[VY][VW] = 0.5;          // -1 & 1
  sc[VX][VX] = 0.5;
  sc[VY][VY] = 0.5;
  matrix = sc*matrix;
}

void CAMERA::CamRoll(float angle)
{
  Cam2Phigs();
  vec3 xn = vup ^ vpn;
  float an = DEG2RAD(angle);
  float p1 = (float)(-(1-cos(an)));
  float p2 = (float)sin(an);
  vec3 dep = p2*xn+p1*ViewUp;
  ViewUp += dep;
  ViewUp.normalize();
}

// camera moves in (ViewUp,vpn) plane, rotating around the target
void CAMERA::CamRotate(float angle)
{
  Cam2Phigs();
  vec3 xn = vup ^ vpn;
  float an = DEG2RAD(angle);
  float d = (CamPos-LookAt).length();
  float p1 = (float)(d*(1-cos(an)));
  float p2 = (float)(d*sin(an));
  vec3 dep = p1*vpn+p2*ViewUp;
  CamPos += dep;
  vec3 nvpn = LookAt-CamPos;
  nvpn.normalize();
  ViewUp = nvpn ^ xn;
  ViewUp.normalize();
}

// target moves in (ViewUp,vpn) plane, rotating around the camera
void CAMERA::CamLoop(float angle)
{
  Cam2Phigs();
  vec3 xn = vup ^ vpn;
  float an = DEG2RAD(angle);
  float d = (CamPos-LookAt).length();
  float p1 = (float)(-d*(1-cos(an)));
  float p2 = (float)(d*sin(an));
  vec3 dep = p1*vpn+p2*ViewUp;
  LookAt += dep;
  vec3 nvpn = LookAt-CamPos;
  nvpn.normalize();
  ViewUp = nvpn ^ xn;
  ViewUp.normalize();
}

// camera moves in (vpn,xn) plane, rotationg around the target
void CAMERA::CamRotateAxis(float angle)
{
  Cam2Phigs();
  vec3 xn = vup ^ vpn;
  float an = DEG2RAD(angle);
  float d = (CamPos-LookAt).length();
  float p1 = (float)(d*(1-cos(an)));
  float p2 = (float)(d*sin(an));
  vec3 dep = p1*vpn+p2*xn;
  CamPos += dep;
}

// target moves in (vpn,xn) plane, rotationg around the camera
void CAMERA::CamRotateHead(float angle)
{
  Cam2Phigs();
  vec3 xn = vup ^ vpn;
  float an = DEG2RAD(angle);
  float d = (CamPos-LookAt).length();
  float p1 = (float)(-d*(1-cos(an)));
  float p2 = (float)(d*sin(an));
  vec3 dep = p1*vpn+p2*xn;
  LookAt += dep;
}

void CAMERA::CamMoveUp(float pas)
{
  CamPos += ViewUp*pas;
  LookAt += ViewUp*pas;
}

void CAMERA::CamMoveForward(float pas)
{
  Cam2Phigs();
  vec3 depl = pas*vpn;
  CamPos += depl;
//  LookAt += depl;
}

void CAMERA::CamMoveLeft(float pas)
{
  Cam2Phigs();
  vec3 axenx = vup ^ vpn;
  vec3 depl = pas*axenx;
  CamPos += depl;
  LookAt += depl;
}

void CAMERA::CamCalcViewUp(float Spin)
{
  vec3 dov = -(LookAt-CamPos);
  dov.normalize();
  float tet = DEG2RAD(Spin+180);
  float c = (float)cos(tet);
  float s = (float)sin(tet);
  float vx = dov[VX];
  float vy = dov[VY];
  float vz = dov[VZ];
  ViewUp = vec3(-vz*s-vx*vy*c,(vx*vx+vz*vz)*c,vx*s-vz*vy*c);
  if (ViewUp == vec3(0,0,0)) ViewUp = vec3(1,0,0);
  else ViewUp.normalize();
}

ostream& operator << (ostream& s, CAMERA& cam)
{
  vec3 pos = cam.CamGetPosition();
  vec3 lka = cam.CamGetLookAt();
  vec3 viw = cam.CamGetViewUp();
  float fp = cam.CamGetFrontPlane();
  float fv = cam.CamGetFov();
  float r1, r2;
  cam.CamGetRadius(&r1, &r2);
  return s << "Camera :      position = " << pos << '\n'
           << "              look at  = " << lka << '\n'
           << "              view up  = " << viw << '\n'
           << "              front pl = " << fp << '\n'
           << "              fov      = " << fv << '\n'
           << "              range    = " << r1 << '  ' << r2 << '\n';
}

