#ifndef CAMERA_HPP
#define CAMERA_HPP

#include "mymath.h"

class CAMERA
{
  private:
    float front_pd; // front plane distance
    vec3 CamPos;
    vec3 LookAt;
    vec3 ViewUp;
    float fov;
    float RatioYX;
    float min_radius, max_radius;
    mat4 matrix; // transformation world->camera

    vec3 prp, vpn, vup;
    float umax, vmax;

  protected:
    void Cam2Phigs(void);

  public:
    void CamCalcMatrix(void);
    void CamRoll(float angle);
    void CamRotate(float angle);
    void CamLoop(float angle);
    void CamRotateAxis(float angle);
    void CamRotateHead(float angle);
    void CamMoveUp(float pas);
    void CamMoveForward(float pas);
    void CamMoveLeft(float pas);

    void CamSetPosition(vec3 pos) { CamPos = pos; }
    void CamSetLookAt(vec3 look) { LookAt = look; }
    void CamSetFrontPlane(float value) { front_pd = value; }
    void CamSetViewUp(vec3 vup) { ViewUp = vup; }
    void CamSetFov(float value) { fov = (float)DEG2RAD(value); }
    void CamSetAspect(float value) { RatioYX = value; }
    void CamSetRadius(float r1, float r2) { min_radius = r1; max_radius = r2; }
    void CamCalcViewUp(float Spin);

    vec3   CamGetPosition(void) { return CamPos; }
    vec3   CamGetLookAt(void) { return LookAt; }
    float CamGetFrontPlane(void) { return front_pd; }
    vec3   CamGetViewUp(void) { return ViewUp; }
    float CamGetFov(void) { return (float)RAD2DEG(fov); }
    void   CamGetRadius(float *r1, float *r2) { *r1=min_radius; *r2=max_radius; }
    mat4*  CamGetMatrix(void) { return &matrix; }
    vec3    CamGetVpn(void) { return vpn; }

    CAMERA()
    {
      front_pd   = 0.1f;
      CamPos     = vec3(0.0,0.0,-100.0);
      LookAt     = vec3(0.0,0.0,0.0);
      ViewUp     = vec3(0.0,1.0,0.0);
      fov        = (float)DEG2RAD(60.0);
      RatioYX    = 1.0;
      min_radius = 1.0;
      max_radius = 1000.0;
      CamCalcMatrix();
    }

  friend std::ostream& operator << (std::ostream& s, CAMERA& cam); // output to stream
  friend class SPOT_LIGHT;
};

#endif
