// ChildFrm.cpp : implementation of the CChildFrame class
//

#include "stdafx.h"
#include "TRViewer.h"

#include "ChildFrm.h"
#include "TRViewerDoc.h"
#include "TRViewerView.h"
#include "EditBase.h"
#include "EditEmpty.h"
#include "EditTile.h"
#include "EditMesh.h"
#include "EditMoveable.h"
#include "EditSprite.h"
#include "EditSounds.h"

#include "ST_SplitterWnd.h"

#include "ToolboxBase.h"
#include "ToolboxItfBase.h"
#include "ToolboxSheet.h"
#include "TlboxMeshItf.h"
#include "Toolbox2Mesh.h"
#include "TlboxMoveableItf.h"
#include "Toolbox2Moveable.h"
#include "TlboxSpriteItf.h"
#include "Toolbox2Sprite.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


// Helpers to calculate the size of a dialog box
// from MSJ September 1998
class CTempDlg : public CDialog {
public:
     CRect m_rcDlg;
protected:
     virtual BOOL OnInitDialog();
};
 
BOOL CTempDlg::OnInitDialog()
{
    GetWindowRect(&m_rcDlg);
    m_rcDlg -= m_rcDlg.TopLeft();
    EndDialog(0);
    return TRUE;
}

CSize GetDialogSize(LPCTSTR pszTemplName, CWnd* pParent)
{
    CTempDlg dlg;
    dlg.Create(pszTemplName, pParent);
	return dlg.m_rcDlg.Size();
}


/////////////////////////////////////////////////////////////////////////////
// CChildFrame

IMPLEMENT_DYNCREATE(CChildFrame, CMDIChildWnd)

BEGIN_MESSAGE_MAP(CChildFrame, CMDIChildWnd)
	//{{AFX_MSG_MAP(CChildFrame)
		// NOTE - the ClassWizard will add and remove mapping macros here.
		//    DO NOT EDIT what you see in these blocks of generated code !
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CChildFrame construction/destruction

CChildFrame::CChildFrame()
{
	m_lpMainSplitter	= NULL;
	m_lpEditSplitter	= NULL;

	m_eEditPaneType		= eEditPane_None;

	m_boToolboxVisible	= false;

	m_iTboxMeshView		= 0;
	m_iTboxMoveableView	= 0;
	m_iTboxSpriteView	= 0;
}

CChildFrame::~CChildFrame()
{
	delete m_lpMainSplitter;
	delete m_lpEditSplitter;
}

BOOL CChildFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs
	if( !CMDIChildWnd::PreCreateWindow(cs) )
		return FALSE;

	return TRUE;
}



/////////////////////////////////////////////////////////////////////////////
// CChildFrame diagnostics

#ifdef _DEBUG
void CChildFrame::AssertValid() const
{
	CMDIChildWnd::AssertValid();
}

void CChildFrame::Dump(CDumpContext& dc) const
{
	CMDIChildWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CChildFrame message handlers

BOOL CChildFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext) 
{
	// Create the splitter with the tree ctrl in the left pane and nothing (yet) in the right pane
	m_lpMainSplitter = new ST_SplitterWnd;

	m_lpMainSplitter->Create( this, RUNTIME_CLASS(CTRViewerView), NULL, pContext, true );

	// Create the splitter embedded in the right pane of the main splitter
	// This is a horizontal splitter: the upper pane is the object view window and the bottom pane
	// is the toolbox window
	m_lpEditSplitter = m_lpMainSplitter->AddSubDivision( RIGHT_SIDE, RUNTIME_CLASS(CEditEmpty), NULL, pContext, false );

	m_iTboxMeshView		= m_lpEditSplitter->AddView( BOTTOM_SIDE, RUNTIME_CLASS(CToolbox2Mesh), pContext );
	m_iTboxMoveableView	= m_lpEditSplitter->AddView( BOTTOM_SIDE, RUNTIME_CLASS(CToolbox2Moveable), pContext );
	m_iTboxSpriteView	= m_lpEditSplitter->AddView( BOTTOM_SIDE, RUNTIME_CLASS(CToolbox2Sprite), pContext );

	m_lpEditSplitter->LockPane( BOTTOM_SIDE );		// the toolbox pane is not resizable
	m_lpEditSplitter->ToggleSide( BOTTOM_SIDE );	// hide the toolbox pane

    m_lpMainSplitter->SetColumnInfo(0,250,1);
    m_lpMainSplitter->RecalcLayout();

	return true;
}


void CChildFrame::ActivateFrame(int nCmdShow)
{
    if (nCmdShow == -1) nCmdShow = SW_SHOWMAXIMIZED;

    CMDIChildWnd::ActivateFrame(nCmdShow);
} 


enumEditPaneType	CChildFrame::eGetEditPaneType( CWnd** lplpEditView ) const
{
	if( lplpEditView != NULL ) *lplpEditView = m_lpEditSplitter->GetPane(0,0);

	return m_eEditPaneType;
}


CTRViewerView*	CChildFrame::lpGetMainView() const
{
	return dynamic_cast<CTRViewerView*>( m_lpMainSplitter->GetPane(0,0) );
}


CWnd*	CChildFrame::lpGetToolbox( enumEditPaneType eEditType ) const
{
	switch( eEditType )
		{
		case eEditPane_Mesh:	return m_lpEditSplitter->GetView( BOTTOM_SIDE, m_iTboxMeshView );
		case eEditPane_Moveable:return m_lpEditSplitter->GetView( BOTTOM_SIDE, m_iTboxMoveableView );
		case eEditPane_Sprite:	return m_lpEditSplitter->GetView( BOTTOM_SIDE, m_iTboxSpriteView );
		}

	_ASSERTE( "CChildFrame::lpGetToolbox -> unknown edit type !" );

	return NULL;
}


void	CChildFrame::vSetEditPaneType( enumEditPaneType ePaneType )
{
	// Recreate the object view pane according to the type given
	m_eEditPaneType = ePaneType;

    m_lpEditSplitter->DeleteView(0,0);

	switch( ePaneType )
		{
		case eEditPane_None:
			{
			m_lpEditSplitter->CreateView(0,0,RUNTIME_CLASS(CEditEmpty),CSize(1,1),NULL);

			bool boSave = m_boToolboxVisible;

			vShowToolbox( false );

			m_boToolboxVisible = boSave;

			break;
			}

		case eEditPane_Tile:
			{
			m_lpEditSplitter->CreateView(0,0,RUNTIME_CLASS(CEditTile),CSize(1,1),NULL);
			break;
			}

		case eEditPane_Mesh:
			{
			m_lpEditSplitter->CreateView(0,0,RUNTIME_CLASS(CEditMesh),CSize(1,1),NULL);
			break;
			}

		case eEditPane_Moveable:
			{
			m_lpEditSplitter->CreateView(0,0,RUNTIME_CLASS(CEditMoveable),CSize(1,1),NULL);
			break;
			}

		case eEditPane_Sprite:
			{
			m_lpEditSplitter->CreateView(0,0,RUNTIME_CLASS(CEditSprite),CSize(1,1),NULL);
			break;
			}

		case eEditPane_Sound:
			{
			m_lpEditSplitter->CreateView(0,0,RUNTIME_CLASS(CEditSounds),CSize(1,1),NULL);

			bool boSave = m_boToolboxVisible;

			vShowToolbox( false );

			m_boToolboxVisible = boSave;

			break;
			}
	}

    m_lpEditSplitter->RecalcLayout();
}


void	CChildFrame::vShowToolboxPane()
{
	if( m_lpEditSplitter->IsSideHidden( BOTTOM_SIDE ) )
		{
		if( m_boToolboxVisible ) m_lpEditSplitter->ToggleSide( BOTTOM_SIDE );
		}
	else
		{
		if( !m_boToolboxVisible ) m_lpEditSplitter->ToggleSide( BOTTOM_SIDE );
		}
}


void	CChildFrame::vShowToolbox( bool boShow )
{
	m_boToolboxVisible = boShow;

	int	iDlg, iViewIdx;

	switch( m_eEditPaneType )
		{
		case eEditPane_Mesh:
			{
			iDlg = IDD_TOOLBOX2_MESH;
			iViewIdx = m_iTboxMeshView;

			break;
			}

		case eEditPane_Moveable:
			{
			iDlg = IDD_TOOLBOX2_MOVEABLE;
			iViewIdx = m_iTboxMoveableView;

			break;
			}

		case eEditPane_Sprite:
			{
			iDlg = IDD_TOOLBOX2_SPRITE;
			iViewIdx = m_iTboxSpriteView;

			break;
			}

		case eEditPane_None:
		case eEditPane_Tile:
		case eEditPane_Sound:
			{
			if( !m_lpEditSplitter->IsSideHidden( BOTTOM_SIDE ) ) m_lpEditSplitter->ToggleSide( BOTTOM_SIDE );

			return;
			}
		}

	vShowToolboxPane();

	if( !m_boToolboxVisible ) return;

	RECT rect;

	m_lpEditSplitter->GetClientRect( &rect );

	CSize sz = GetDialogSize( MAKEINTRESOURCE( iDlg ), this );

	m_lpEditSplitter->SetRowInfo( 0, rect.bottom - rect.top - sz.cy - 15, 1 );
	m_lpEditSplitter->SetRowInfo( 1, sz.cy + 15, sz.cy );

	m_lpEditSplitter->SwitchToView( BOTTOM_SIDE, iViewIdx );
}
