#include "stdafx.h"

#include "Resource.h"
#include "Mainfrm.h"
#include "ToolboxBase.h"
#include "ToolboxSheet.h"
#include "EditBase.h"


IMPLEMENT_DYNCREATE(CEditBase, CFormView)


#define OPENGLWND_LEFT_OFFSET       8
#define OPENGLWND_TOP_OFFSET        4
#define OPENGLWND_RIGHT_OFFSET      4
#define OPENGLWND_BOTTOM_OFFSET     4


IMPLEMENT_DYNAMIC(CEditGfxBase, CEditBase)


CEditGfxBase::CEditGfxBase() : CEditBase( IDD_EDIT_EMPTY )
{
    m_lpDataHandler     = NULL;
    m_OpenGLWnd         = NULL;
    m_OpenGLWndCreated  = false;
}


CEditGfxBase::CEditGfxBase( int iId ) : CEditBase( iId )
{
    m_lpDataHandler     = NULL;
    m_OpenGLWnd         = NULL;
    m_OpenGLWndCreated  = false;
}


CEditGfxBase::~CEditGfxBase()
{
    vRelease();
}


void CEditGfxBase::vRelease()
{
    m_lpDataHandler = NULL;

    if(m_OpenGLWndCreated)
        {
        m_OpenGLWnd->DestroyWindow();
        delete m_OpenGLWnd;
        }

    m_OpenGLWnd = NULL;
    m_OpenGLWndCreated = false;
}


BEGIN_MESSAGE_MAP(CEditGfxBase, CEditBase)
	//{{AFX_MSG_MAP(CEditGfxBase)
	ON_WM_SIZE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_MOUSEWHEEL()
	ON_WM_LBUTTONDBLCLK()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


#ifdef _DEBUG
void CEditGfxBase::AssertValid() const
{
	CEditBase::AssertValid();
}


void CEditGfxBase::Dump(CDumpContext& dc) const
{
	CEditBase::Dump(dc);
}
#endif //_DEBUG


void CEditGfxBase::DoDataExchange(CDataExchange* pDX)
{
	CEditBase::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CEditGfxBase)
	//}}AFX_DATA_MAP
}


void CEditGfxBase::OnInitialUpdate() 
{
	CEditBase::OnInitialUpdate();

    SetScrollSizes(MM_TEXT,CSize(0,0));
}


void CEditGfxBase::OnDraw(CDC* pDC) 
{
}


void CEditGfxBase::vSetDataTypeHandler(CHandleBase* lpHandler)
{
    m_lpDataHandler = lpHandler;

    if(lpHandler  &&  !m_OpenGLWndCreated) vCreateOpenGLWnd();

	vOnInitEdit();
}


void CEditGfxBase::vCreateOpenGLWnd()
{
    RECT rect;

    GetClientRect(&rect);

    rect.left   = OPENGLWND_LEFT_OFFSET;
    rect.top    = OPENGLWND_TOP_OFFSET;
    rect.bottom -= OPENGLWND_BOTTOM_OFFSET;
    rect.right  -= OPENGLWND_RIGHT_OFFSET;

    m_OpenGLWnd = new COpenGLWnd;
    m_OpenGLWnd->Create("static","",WS_CHILD | WS_VISIBLE | WS_CLIPCHILDREN | WS_CLIPSIBLINGS,rect,this,12302);
    m_OpenGLWnd->ShowWindow(SW_SHOW);

    m_OpenGLWndCreated = true;
}


BOOL CEditGfxBase::boOnIdle()
{
    if(m_OpenGLWndCreated)
        {
        CMainFrame* lpMainFrame = (CMainFrame*) AfxGetApp()->m_pMainWnd;

        //if( lpMainFrame->boIsManualCamera() )
            {
            m_OpenGLWnd->vCheckKeys();

            vCheckKeys( m_OpenGLWnd->fGetFrameRate() );
            }

        vRedrawOpenGLWnd();
        }

	if (lpGetBaseToolbox()) lpGetBaseToolbox()->vCheckKeys();

    return TRUE;
}


void CEditGfxBase::vReloadTextures()
{
    if(m_OpenGLWnd) m_OpenGLWnd->vReleaseTextures();
}


void CEditGfxBase::OnActivateView(BOOL bActivate, CView* pActivateView, CView* pDeactiveView) 
{
    if(m_OpenGLWndCreated  &&  bActivate) m_OpenGLWnd->vActivateWnd();

    if(bActivate)
        {
        //!!lpGetBaseToolbox()->vSetHandlers(bActivate ? m_lpDataHandler : NULL,bActivate ? this : NULL);
        }

    if(pActivateView  ||  pDeactiveView)
	    CEditBase::OnActivateView(bActivate, pActivateView, pDeactiveView);
}


void CEditGfxBase::OnSize(UINT nType, int cx, int cy) 
{
	CEditBase::OnSize(nType, cx, cy);
	
    if(m_OpenGLWndCreated) m_OpenGLWnd->SetWindowPos(NULL,OPENGLWND_LEFT_OFFSET,OPENGLWND_TOP_OFFSET,cx-OPENGLWND_LEFT_OFFSET-OPENGLWND_RIGHT_OFFSET,cy-OPENGLWND_BOTTOM_OFFSET-OPENGLWND_TOP_OFFSET,0);
}


void CEditGfxBase::OnLButtonDown(UINT nFlags, CPoint point) 
{
    if(m_OpenGLWndCreated) m_OpenGLWnd->OnLButtonDown(nFlags, point);
	
	CEditBase::OnLButtonDown(nFlags, point);
}


void CEditGfxBase::OnLButtonUp(UINT nFlags, CPoint point) 
{
    if(m_OpenGLWndCreated) m_OpenGLWnd->OnLButtonUp(nFlags, point);
	
	CEditBase::OnLButtonUp(nFlags, point);
}


void CEditGfxBase::OnMouseMove(UINT nFlags, CPoint point) 
{
    if(m_OpenGLWndCreated) m_OpenGLWnd->OnMouseMove(nFlags, point);
	
	CEditBase::OnMouseMove(nFlags, point);
}


void CEditGfxBase::OnRButtonDown(UINT nFlags, CPoint point) 
{
    if(m_OpenGLWndCreated) m_OpenGLWnd->OnRButtonDown(nFlags, point);
	
	CEditBase::OnRButtonDown(nFlags, point);
}


void CEditGfxBase::OnRButtonUp(UINT nFlags, CPoint point) 
{
    if(m_OpenGLWndCreated) m_OpenGLWnd->OnRButtonUp(nFlags, point);
	
	CEditBase::OnRButtonUp(nFlags, point);
}


BOOL CEditGfxBase::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
    if(m_OpenGLWndCreated) m_OpenGLWnd->OnMouseWheel(nFlags, zDelta, pt);
	
	return CEditBase::OnMouseWheel(nFlags, zDelta, pt);
}

void CEditGfxBase::OnLButtonDblClk(UINT nFlags, CPoint point) 
{
    if(m_OpenGLWndCreated) m_OpenGLWnd->OnLButtonDblClk(nFlags, point);
	
	CEditBase::OnLButtonDblClk(nFlags, point);
}
