//--------------------------------------------------------------------------------------------------------------------//
//                                                .FSCENE FILE FORMAT                                                 //
//--------------------------------------------------------------------------------------------------------------------//

#include  "Math/Quaternion.h"

typedef unsigned  char        u8;
typedef unsigned  long        u32;
typedef float                 f32;


#pragma pack(1)                                             // do NOT align structure members !

// file header

typedef struct {
        u32                   u32Version;                   // MUST come first
        u32                   u32NbMtl;
        u32                   u32NbCamera;
        u32                   u32NbLight;
        u32                   u32NbMesh;
  } FILEHEADER0x100;

typedef struct {
        u32                   u32Version;                   // MUST come first
        u32                   u32FirstFrame;
        u32                   u32LastFrame;
        u32                   u32FrameSpeed;
        u32                   u32FrameTicks;
        u32                   u32NbAnims;
  } ANIMFILEHEADER0x100;

// material

typedef struct {
        char                  szName[32];
        bool                  boTwoSided;

        CVect4D               v4Diffuse;
        char                  szDiffuse[32];
  } MATERIAL0x100;

// camera

typedef struct {
        char                  szName[32];
        CVect3D               v3Pos;
        CQuaternion           qRot;

        float                 fNear;
        float                 fFar;
        float                 fFov;
        float                 fTarget;
  } CAMERA0x100;

// light

typedef struct {
        u32                   u32Type;                      // MUST come first
        char                  szName[32];
        CVect3D               v3Pos;
        CQuaternion           qRot;

        bool                  boUse;
        CVect3D               v3Color;
        float                 fIntensity;
        float                 fAttenStart;
        float                 fAttenEnd;
  } LIGHT0x100;

typedef enum {
        AMBIENT = 0,
        DIR,
        OMNI,
        SPOT
  } LIGHT_TYPE;

// mesh

typedef struct {
        char                  szName[32];
        u32                   u32Parent;
        CVect3D               v3Pos;
        CQuaternion           qRot;
        CVect3D               v3Scale;

        CVect3D               v3BoxMin;
        CVect3D               v3BoxMax;

        u32                   u32NbRange;
        u32                   u32NbVtx;
        u32                   u32NbIdx;

        bool                  boCompressedVtx;
        bool                  boIsBone;
        u32                   u32SkinBones;
} MESHHEADER0x100;

typedef struct {
        u32                   u32Mtl;
        u32                   u32Prim;

        u32                   u32IdxMin;
        u32                   u32IdxMax;
        u32                   u32VtxMin;
        u32                   u32VtxMax;

        u32                   u32Stream1Fmt;
  } MESHRANGE0x100;

// anim

typedef struct {
        u32                   u32Type;                      // MUST come first
        char                  szName[32];

        u32                   u32PosKeys;
        u32                   u32RotKeys;
        u32                   u32ScaleKeys;
  } ANIMHEADER0x100;

typedef enum {
        KEYFRAME = 0,
        MORPH
  } ANIM_TYPE;

// position

typedef struct {
        u32                   u32Type;                      // MUST come first
        u32                   u32Keys;
  } POSHEADER0x100;

typedef enum {
        TCB = 0,
        SAMPLE,
  } KEY_TYPE;

// rotation

typedef struct {
        u32                   u32Type;                      // MUST come first
        u32                   u32Keys;
  } ROTHEADER0x100;

// scale

typedef struct {
        u32                   u32Type;                      // MUST come first
        u32                   u32Keys;
  } SCALEHEADER0x100;

//

#pragma pack()

typedef FILEHEADER0x100       FILEHEADER;
typedef MATERIAL0x100         MATERIAL;
typedef CAMERA0x100           CAMERA;
typedef LIGHT0x100            LIGHT;
typedef MESHHEADER0x100       MESHHEADER;
typedef MESHRANGE0x100        MESHRANGE;

typedef ANIMFILEHEADER0x100   ANIMFILEHEADER;
typedef ANIMHEADER0x100       ANIMHEADER;
typedef POSHEADER0x100        POSHEADER;
typedef ROTHEADER0x100        ROTHEADER;
typedef SCALEHEADER0x100      SCALEHEADER;

// stream1: vertex format (copied from VBuffer.h)

typedef struct
  {
  union
    {
      u32                     Format;
      struct 
        {
        u8                    XYZ       : 1;
        u8                    Weights   : 2;                // 0 to 3 weights
        u8                    Indices   : 1;                // last weight = 4 [0;255] indices
        u8                    Normal    : 1;
        u8                    PtSize    : 1;
        u8                    Diffuse   : 1;
        u8                    Specular  : 1;
        u8                    Tangent   : 1;
        u8                    Binormal  : 1;

        u8                    Tex       : 4;                // 0 to 8 texture coords
        };
    };
  } VTX_COMPONENTS;
