// FileTree.cpp : implementation file
//

#include "stdafx.h"

#include "TRViewer.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "TRViewerDoc.h"
#include "FileTree.h"
#include "HandleBase.h"
#include "HandleRootTR123.h"
#include "HandleEmpty.h"

//#include "ToolboxMoveable.h"
//#include "ToolboxMesh.h"


#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif


/////////////////////////////////////////////////////////////////////////////
// CFileTree

CFileTree::CFileTree(CChildFrame* lpFrame,CTRFile* lpTRFile,CTRViewerDoc* lpDocument)
{
    m_lpSelectedNode        = NULL;
    m_lpOldSelectedNode     = NULL;
    m_lpFrame               = lpFrame;
    m_lpTRFile              = lpTRFile;
    m_lpHandlerPopup        = NULL;
    m_pOldSel               = NULL;
    m_lpDocument            = lpDocument;
    m_boDeleting            = false;

    m_lpHandleMoveables     = NULL;
    m_lpHandleStaticMeshes  = NULL;
    m_lpHandleSprites       = NULL;
}

CFileTree::~CFileTree()
{
}


BEGIN_MESSAGE_MAP(CFileTree, CTreeCtrl)
	//{{AFX_MSG_MAP(CFileTree)
	ON_NOTIFY_REFLECT(TVN_SELCHANGED, OnSelchanged)
	ON_NOTIFY_REFLECT(TVN_ITEMEXPANDING, OnItemexpanding)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_CONTEXTMENU()
	ON_WM_DESTROY()
	ON_WM_CREATE()
	ON_NOTIFY_REFLECT(TVN_BEGINDRAG, OnBegindrag)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CFileTree message handlers


void    CFileTree::OnSelchanged(NMHDR* pNMHDR, LRESULT* pResult) 
{
    // Get the CHandleBase class of the selected node
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;

	*pResult = 0;

    TV_ITEM*    lpItem = &pNMTreeView->itemNew;

    m_lpSelectedNode = (CHandleBase*)lpItem->lParam;

    // Show the edit pane according to the type of the selected node
    bool boRecreatePane = !m_lpOldSelectedNode  ||  !m_lpSelectedNode  ||  (m_lpOldSelectedNode->dwGetDataType() != m_lpSelectedNode->dwGetDataType());

    if(m_lpSelectedNode  &&  !m_boDeleting)
        {
        m_lpSelectedNode->vShowEditDialog(boRecreatePane,(TCHAR*)(LPCTSTR)GetItemText(lpItem->hItem));
        }

    m_lpOldSelectedNode = m_lpSelectedNode;
}


void    CFileTree::OnItemexpanding(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	*pResult = 0;

    if(pNMTreeView->action == TVE_EXPAND)
        {
        TV_ITEM*    lpItem = &pNMTreeView->itemNew;

        CHandleBase* lpNode = (CHandleBase*)lpItem->lParam;

        if(lpNode)
            {
            lpNode->vExpandTree_(lpItem->hItem);
            }
        }
}


void    CFileTree::OnLButtonDown(UINT nFlags, CPoint point) 
{
    UINT        uFlags;
    HTREEITEM   htItem = HitTest(point, &uFlags);

    if((htItem != NULL) && (uFlags & TVHT_ONITEM)) Select(htItem, TVGN_DROPHILITE);

	CTreeCtrl::OnLButtonDown(nFlags, point);
}

void    CFileTree::OnRButtonDown(UINT nFlags, CPoint point) 
{
    UINT        uFlags;
    HTREEITEM   htItem = HitTest(point, &uFlags);
    if((htItem != NULL) && (uFlags & TVHT_ONITEM))
        {
        //m_pOldSel = GetSelectedItem();
        Select(htItem, TVGN_DROPHILITE | TVGN_CARET);
        }
}

void    CFileTree::OnContextMenu(CWnd* pWnd, CPoint point) 
{
    UINT    uFlags;
    CPoint  ptTree = point;

    ScreenToClient(&ptTree);
    
    HTREEITEM htItem = HitTest(ptTree, &uFlags);

    if((htItem != NULL) && (uFlags & TVHT_ONITEM))
        {
        ShowPopupMenu(point,htItem);
        SetItemState(htItem, 0, TVIS_DROPHILITED);
        }
    else
        CTreeCtrl::OnContextMenu(pWnd, point);

    if(m_pOldSel != NULL)
        {
        Select(m_pOldSel, TVGN_DROPHILITE);
        m_pOldSel = NULL;
        }
}


void    CFileTree::ShowPopupMenu(CPoint& point, HTREEITEM hItem)
{
    CHandleBase*    lpHandler = (CHandleBase*)GetItemData(hItem);

    CMenu*  lpMenu = lpHandler->lpGetPopupupMenu(hItem);

    if(!lpHandler  ||  !lpMenu) return;

    if(point.x == -1 && point.y == -1)
        {
        //keystroke invocation
        CRect rect;
        GetClientRect(rect);
        ClientToScreen(rect);
        point = rect.TopLeft();
        point.Offset(5, 5);
        }
        
    m_lpHandlerPopup = lpHandler;

    lpMenu->TrackPopupMenu(TPM_LEFTALIGN | TPM_RIGHTBUTTON, point.x, point.y, this);
    lpHandler->vReleasePopupupMenu();
}


BOOL CFileTree::OnCommand(WPARAM wParam, LPARAM lParam) 
{
    if(m_lpHandlerPopup) m_lpHandlerPopup->vHandleMenu(LOWORD(wParam));
	
	return CTreeCtrl::OnCommand(wParam, lParam);
}


int CFileTree::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CTreeCtrl::OnCreate(lpCreateStruct) == -1)
		return -1;
	
    // Register Tree control as a drop target	
	m_CTreeDropTarget.Register(this);

	// Create the image list for the treeview
    m_LstIcons.Create( 16, 16, ILC_COLOR32, 0, 256 );

    const CResMgr&  ResMgr = ((CTRViewerApp*)AfxGetApp())->GetResMgr();

	m_LstIcons.Add( const_cast<CBitmap*>( ResMgr.lpGetIconsImage() ), RGB(0, 0, 0) );

	SetImageList( &(m_LstIcons), LVSIL_NORMAL );
	
	if( !((CTRViewerApp*)AfxGetApp())->boShowIconItems() ) SetImageList( NULL, LVSIL_NORMAL );

	return 0;
}


void CFileTree::vShowIcons( bool boShow )
{
	if( boShow ) SetImageList( &(m_LstIcons), LVSIL_NORMAL ); else SetImageList( NULL, LVSIL_NORMAL );
}


void CFileTree::OnBegindrag(NMHDR* pNMHDR, LRESULT* pResult) 
{
	NM_TREEVIEW* pNMTreeView = (NM_TREEVIEW*)pNMHDR;
	
	HTREEITEM hTSelItem = pNMTreeView->itemNew.hItem;

	// Highlight selected item
	SelectItem(hTSelItem);
	Select(hTSelItem, TVGN_DROPHILITE);
	
	COleDataSource *poleSourceObj = new COleDataSource ;
	CTreeDropTarget::m_shWndTreeCtrl = m_hWnd;

	// Begin Drag operation
	DROPEFFECT dropeffect = poleSourceObj->DoDragDrop();

	// Remove the highlighting
	SendMessage(TVM_SELECTITEM, TVGN_DROPHILITE,0);

	// If user is moving item by pressing Shift, delete selected item
	//if ( dropeffect == DROPEFFECT_MOVE)	DeleteItem(hTSelItem); 

	delete poleSourceObj;

	*pResult = 0;
}


//
//
//
//
//
//
//


void    CFileTree::vInitialize()
{
    CString s(m_lpTRFile->lpszGetShortLevelName());
    HTREEITEM h;
    enumTRType		trType = m_lpTRFile->GetTRType();
    CHandleBase*	lpRoot = NULL;
    
    const CResMgr&  ResMgr = ((CTRViewerApp*)AfxGetApp())->GetResMgr();

    m_hRootItem = NULL;

    switch(trType)
        {
        case TR1_FILE:
        case TR2_FILE:
        case TR3_FILE:
        case TR4_FILE:
            {
			CResLevel	Level = ResMgr.GetLevelInfo( ResMgr.bGetResId( trType ), m_lpTRFile->lpszGetShortLevelName() );
			s = Level.strGetLongName().c_str();
            lpRoot = new CHandleRootTR123(this,m_lpFrame);
            INSERT(this,m_hRootItem,TVI_ROOT,lpRoot)
			SetItemImage( m_hRootItem, Level.dwGetIconId(), Level.dwGetIconId() );
            break;
            }
        }

    if(m_hRootItem)
        {
        INSERT(this,h,m_hRootItem,new CHandleEmpty(this,m_lpFrame))
        if(lpRoot) lpRoot->vExpandTree_(m_hRootItem);
        }
}


void    CFileTree::vReleaseNode(HTREEITEM hItem,bool boDeleteItems,bool boDontDeleteFirstItem)
{
    CHandleBase*    lpNode = (CHandleBase*) GetItemData(hItem);

    if(lpNode  &&  !boDontDeleteFirstItem)
        {
        if(m_lpOldSelectedNode == lpNode) m_lpOldSelectedNode = NULL;
        delete lpNode;
        }

    if(!ItemHasChildren(hItem)) return;

    HTREEITEM hNextItem;
    HTREEITEM hChildItem = GetChildItem(hItem);

    while(hChildItem != NULL)
        {
        hNextItem = GetNextItem(hChildItem, TVGN_NEXT);
        vReleaseNode(hChildItem,boDeleteItems,false);
        if(boDeleteItems) DeleteItem(hChildItem);
        hChildItem = hNextItem;
        }
}


bool    CFileTree::boSave(LPCTSTR lpszFileName)
{
    CWaitCursor dummy;

    return m_lpTRFile ? m_lpTRFile->boSave(lpszFileName) : false;
}


void    CFileTree::vSetChanged()
{
    if(m_lpDocument) m_lpDocument->SetModifiedFlag(TRUE);
}


void    CFileTree::OnDestroy() 
{
    vRelease();

	CTreeCtrl::OnDestroy();
}


void    CFileTree::vRelease()
{
/*!    CToolboxMoveable* lpToolbox = reinterpret_cast<CToolboxMoveable*> (((CMainFrame*)(AfxGetApp()->m_pMainWnd))->lpGetToolbox(TOOLBOX_MOVEABLE));

    if(lpToolbox) lpToolbox->vSetHandlers(NULL,NULL);

    CToolboxMesh* lpToolbox2 = reinterpret_cast<CToolboxMesh*> (((CMainFrame*)(AfxGetApp()->m_pMainWnd))->lpGetToolbox(TOOLBOX_MESH));

    if(lpToolbox2) lpToolbox2->vSetHandlers(NULL,NULL);
*/
    vReleaseNode(m_hRootItem);
}
