// MainFrm.cpp : implementation of the CMainFrame class
//

#include "stdafx.h"
#include "TRViewer.h"

#include "MainFrm.h"
#include "ChildFrm.h"
#include "TRViewerDoc.h"
#include "TRViewerView.h"

#include "Debug.h"
#include "DiagLevelPath.h"
#include "DlgDisplayOpt.h"
#include "DlgSoundOpt.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

//#define TOOLBOX_TOPMOST

#define NO_DEBUG_MENU

/////////////////////////////////////////////////////////////////////////////
// CMainFrame

IMPLEMENT_DYNAMIC(CMainFrame, CMDIFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CMDIFrameWnd)
	//{{AFX_MSG_MAP(CMainFrame)
	ON_WM_CREATE()
	ON_UPDATE_COMMAND_UI(ID_VIEW_OBJECTTOOLBOX, OnUpdateViewObjecttoolbox)
	ON_COMMAND(ID_VIEW_OBJECTTOOLBOX, OnViewObjecttoolbox)
#if !defined(NO_DEBUG_MENU)
	ON_COMMAND(ID_MISC_DEBUG, OnMiscDebug)
#endif
	ON_UPDATE_COMMAND_UI(ID_RENDERING_MANUALCAMERA, OnUpdateRenderingManualcamera)
	ON_COMMAND(ID_RENDERING_MANUALCAMERA, OnRenderingManualcamera)
	ON_UPDATE_COMMAND_UI(ID_RENDERING_SHOWAXIS, OnUpdateRenderingShowAxis)
	ON_COMMAND(ID_RENDERING_SHOWAXIS, OnRenderingShowaxis)
	ON_COMMAND(ID_OPTIONS_ORIGINALLEVELPATHS, OnOptionsOriginallevelpaths)
	ON_COMMAND(ID_OPTIONS_DISPLAY, OnOptionsDisplay)
	ON_COMMAND(ID_OPTIONS_SOUND, OnOptionsSound)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

/////////////////////////////////////////////////////////////////////////////
// CMainFrame construction/destruction

CMainFrame::CMainFrame() : m_pWndCurrentChild( NULL )
{
	m_boToolboxVisible	= false;
	m_boManualCamera	= false;
    m_boShowAxis		= false;

    //m_BkgColor = RGB(212,208,200);
    m_BkgColor = RGB(192,192,192);
}

CMainFrame::~CMainFrame()
{
	m_wndMDIClient.Detach();
}


int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CMDIFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;


    if (m_wndMDIClient.Attach(m_hWndMDIClient) == 0)
    {
        TRACE0("Failed to attach MDIClient.\n");
        return -1;      // fail to create
    }
  
	if (!m_wndToolBar.CreateEx(this, TBSTYLE_FLAT, WS_CHILD | WS_VISIBLE | CBRS_TOP
		| CBRS_GRIPPER | CBRS_TOOLTIPS | CBRS_FLYBY | CBRS_SIZE_DYNAMIC) ||
		!m_wndToolBar.LoadToolBar(IDR_MAINFRAME))
	{
		TRACE0("Failed to create toolbar\n");
		return -1;      // fail to create
	}

	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}

	// TODO: Delete these three lines if you don't want the toolbar to
	//  be dockable
	m_wndToolBar.EnableDocking(CBRS_ALIGN_ANY);
	EnableDocking(CBRS_ALIGN_ANY);
	DockControlBar(&m_wndToolBar);

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(12) | TBBS_CHECKBOX;
    if(m_boToolboxVisible) iStyle |= TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(12, iStyle);

    iStyle = m_wndToolBar.GetButtonStyle(13) | TBBS_CHECKBOX;
    if(m_boManualCamera) iStyle |= TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(13, iStyle);

    iStyle = m_wndToolBar.GetButtonStyle(14) | TBBS_CHECKBOX;
    if(m_boShowAxis) iStyle |= TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(14, iStyle);

	return 0;
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CMDIFrameWnd::PreCreateWindow(cs) )
		return FALSE;
	// TODO: Modify the Window class or styles here by modifying
	//  the CREATESTRUCT cs

	return TRUE;
}

//----------------------------------------------------------------
// This function finds the CMDIChildWnd in the list of windows
// maintained by the application's MDIClient window following the
// one pointed to by the member variable m_pWndCurrentChild. If no
// further CMDIChildWnds are in the list, NULL is returned.
//----------------------------------------------------------------

CMDIChildWnd* CMainFrame::GetNextMDIChildWnd()
{
   if (!m_pWndCurrentChild)
     {
      // Get the first child window.
      m_pWndCurrentChild = m_wndMDIClient.GetWindow(GW_CHILD);
     }
   else
     {
      // Get the next child window in the list.
        m_pWndCurrentChild = (CMDIChildWnd*)m_pWndCurrentChild->GetWindow(GW_HWNDNEXT);
     }

   if (!m_pWndCurrentChild)
     {
      // No child windows exist in the MDIClient,
      // or you are at the end of the list. This check
      // will terminate any recursion.
      return NULL;
     }

	// Check the kind of window
    if (!m_pWndCurrentChild->GetWindow(GW_OWNER))
      {
        if (m_pWndCurrentChild->IsKindOf(RUNTIME_CLASS(CMDIChildWnd)))
          {
			 // CMDIChildWnd or a derived class.
			 return (CMDIChildWnd*)m_pWndCurrentChild;
          }
        else
          {
			 // Window is foreign to the MFC framework.
			 // Check the next window in the list recursively.
			 return GetNextMDIChildWnd();
          }
      }
    else
      {
      // Title window associated with an iconized child window.
      // Recurse over the window manager's list of windows.
      return GetNextMDIChildWnd();
      }
}

//-----------------------------------------------------------------
// This function counts the number of CMDIChildWnd objects
// currently maintained by the MDIClient.
//-----------------------------------------------------------------

int CMainFrame::GetCountCMDIChildWnds()
{
	int count = 0;

	CMDIChildWnd* pChild = GetNextMDIChildWnd();

	while (pChild)
		{
		count++;
		pChild = GetNextMDIChildWnd();
		}

	return count;
} 


/////////////////////////////////////////////////////////////////////////////
// CMainFrame diagnostics

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CMDIFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CMDIFrameWnd::Dump(dc);
}

#endif //_DEBUG

/////////////////////////////////////////////////////////////////////////////
// CMainFrame message handlers

void CMainFrame::OnUpdateViewObjecttoolbox(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_boToolboxVisible ? 1 : 0);
}


void CMainFrame::OnViewObjecttoolbox() 
{
    vSetToolboxType(!m_boToolboxVisible);
}


void CMainFrame::OnUpdateRenderingManualcamera(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_boManualCamera ? 1 : 0);
}


void CMainFrame::OnRenderingManualcamera() 
{
    vSetCameraMode(!m_boManualCamera);
}


void CMainFrame::OnUpdateRenderingShowAxis(CCmdUI* pCmdUI) 
{
	pCmdUI->SetCheck(m_boShowAxis? 1 : 0);
}


void CMainFrame::OnRenderingShowaxis() 
{
	vSetShowAxis(!m_boShowAxis);
}


void CMainFrame::vSetCameraMode(bool boManual)
{
    m_boManualCamera = boManual;

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(13);
    if(m_boManualCamera) iStyle |= TBBS_CHECKED; else iStyle &= ~TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(13, iStyle);

    CTRViewerView*  lpView = static_cast<CTRViewerView*>(CTRViewerView::GetActiveView());

    if(!lpView) return;

    lpView->vSetCameraMode(boManual);
}


void CMainFrame::vSetShowAxis(bool boShow)
{
    m_boShowAxis = boShow;

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(14);
    if(m_boShowAxis) iStyle |= TBBS_CHECKED; else iStyle &= ~TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(14, iStyle);

    CTRViewerView*  lpView = static_cast<CTRViewerView*>(CTRViewerView::GetActiveView());

    if(!lpView) return;

    lpView->vSetShowAxis(boShow);
}


void CMainFrame::OnMiscDebug()
{
    vMiscDebug();
}


void CMainFrame::vSetToolboxType(bool boShow /*= true*/, bool boSetVisibility /*= true*/)
{
	if( boSetVisibility ) m_boToolboxVisible = boShow;

    UINT iStyle;
    
    iStyle = m_wndToolBar.GetButtonStyle(12);
    if(m_boToolboxVisible) iStyle |= TBBS_CHECKED; else iStyle &= ~TBBS_CHECKED;
    m_wndToolBar.SetButtonStyle(12, iStyle);

	while( CChildFrame* lpChildFrm = (CChildFrame*) GetNextMDIChildWnd() )
		{
		lpChildFrm->vShowToolbox( m_boToolboxVisible );
		}
}


void CMainFrame::OnOptionsOriginallevelpaths() 
{
	CDiagLevelPath  Diag;

    CResMgr& ResMgr = const_cast<CResMgr&>(((CTRViewerApp*)AfxGetApp())->GetResMgr());

    Diag.vSetTR1LevelPath(ResMgr.cstrGetDataPath(__RESMGR_TR1__));
    Diag.vSetTR1GoldLevelPath(ResMgr.cstrGetDataPath(__RESMGR_TR1GOLD__));
    Diag.vSetTR2LevelPath(ResMgr.cstrGetDataPath(__RESMGR_TR2__));
    Diag.vSetTR2GoldLevelPath(ResMgr.cstrGetDataPath(__RESMGR_TR2GOLD__));
    Diag.vSetTR3LevelPath(ResMgr.cstrGetDataPath(__RESMGR_TR3__));
    Diag.vSetTR3GoldLevelPath(ResMgr.cstrGetDataPath(__RESMGR_TR3GOLD__));
    Diag.vSetTR4LevelPath(ResMgr.cstrGetDataPath(__RESMGR_TR4__));

    if(Diag.DoModal() == IDOK)
        {
        ResMgr.vSetDataPath(__RESMGR_TR1__,     Diag.cstrGetTR1LevelPath());
        ResMgr.vSetDataPath(__RESMGR_TR1GOLD__, Diag.cstrGetTR1GoldLevelPath());
        ResMgr.vSetDataPath(__RESMGR_TR2__,     Diag.cstrGetTR2LevelPath());
        ResMgr.vSetDataPath(__RESMGR_TR2GOLD__, Diag.cstrGetTR2GoldLevelPath());
        ResMgr.vSetDataPath(__RESMGR_TR3__,     Diag.cstrGetTR3LevelPath());
        ResMgr.vSetDataPath(__RESMGR_TR3GOLD__, Diag.cstrGetTR3GoldLevelPath());
        ResMgr.vSetDataPath(__RESMGR_TR4__,     Diag.cstrGetTR4LevelPath());
        }
}


void CMainFrame::OnOptionsDisplay() 
{
	CDlgDisplayOpt  Diag;

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();
    CResMgr& ResMgr = const_cast<CResMgr&>(lpApp->GetResMgr());

	Diag.vSetBkgColor( m_BkgColor );

	Diag.vSetShowFriendlyNames( lpApp->boShowFriendlyNamesInTreeview() );
	Diag.vSetShowIconItems( lpApp->boShowIconItems() );
	Diag.vSetShowRealMeshes( lpApp->boShowRealMeshes() );
	Diag.vSetShowAssociatedMoveable( lpApp->boShowAssociatedMoveable() );

    if(Diag.DoModal() == IDOK)
        {
		m_BkgColor = Diag.GetBkgColor();

		if( CTRViewerView* lpView = CTRViewerView::lpGetAndActivateTreeCtrlView() )
			{
			lpView->vSetBkgColor( m_BkgColor );
			lpView->vShowIconsInTreeview( Diag.boGetShowIconItems() ? true : false );
			}

		lpApp->vSetShowFriendlyNamesInTreeview( Diag.boGetShowFriendlyNames() ? true : false );
		lpApp->vSetShowIconItems( Diag.boGetShowIconItems() ? true : false );
		lpApp->vSetShowRealMeshes( Diag.boGetShowRealMeshes() ? true : false );
		lpApp->vSetShowAssociatedMoveable( Diag.boGetShowAssociatedMoveable() ? true : false );
        }
}


void CMainFrame::OnOptionsSound() 
{
	CDlgSoundOpt  Diag;

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();
    CResMgr& ResMgr = const_cast<CResMgr&>(lpApp->GetResMgr());

	Diag.vSetUncompressSounds( lpApp->boSaveUncompressedSounds() );

    if(Diag.DoModal() == IDOK)
        {
		lpApp->vSetSaveUncompressedSounds( Diag.boGetUncompressSounds() ? true : false );
        }
}
