// MarkupSTL.h: interface for the CMarkupSTL class.
//
// Markup Release 6.1
// Copyright (C) 1999-2001 First Objective Software, Inc. All rights reserved
// Go to www.firstobject.com for the latest CMarkup and EDOM documentation
// Use in commercial applications requires written permission
// This software is provided "as is", with no warranty.

#if !defined(AFX_MARKUPSTL_H__948A2705_9E68_11D2_A0BF_00105A27C570__INCLUDED_)
#define AFX_MARKUPSTL_H__948A2705_9E68_11D2_A0BF_00105A27C570__INCLUDED_

#if _MSC_VER > 1000
#pragma once
#endif // _MSC_VER > 1000


class CMarkupSTL  
{
public:
	CMarkupSTL() { SetDoc( NULL ); };
	CMarkupSTL( const char* szDoc ) { SetDoc( szDoc ); };
	CMarkupSTL( const CMarkupSTL& markup ) { *this = markup; };
	void operator=( const CMarkupSTL& markup );
	virtual ~CMarkupSTL() {};

	// Navigate
	bool Load( const char* szFileName );
	bool SetDoc( const char* szDoc );
	bool IsWellFormed();
	bool FindElem( const char* szName=NULL );
	bool FindChildElem( const char* szName=NULL );
	bool IntoElem();
	bool OutOfElem();
	void ResetChildPos() { m_iPosChild = 0; };
	void ResetPos();
	std::string GetTagName() const { return x_GetTagName(m_iPos); };
	std::string GetChildTagName() const { return x_GetTagName(m_iPosChild); };
	std::string GetData() const { return x_GetData(m_iPos); };
	std::string GetChildData() const { return x_GetData(m_iPosChild); };
	std::string GetAttrib( const char* szAttrib ) const { return x_GetAttrib(m_iPos,szAttrib); };
	std::string GetChildAttrib( const char* szAttrib ) const { return x_GetAttrib(m_iPosChild,szAttrib); };
	std::string GetAttribName( int n ) const;
	bool SavePos( const char* szPosName="" );
	bool RestorePos( const char* szPosName="" );
	bool GetOffsets( int& nStart, int& nEnd ) const;
	std::string GetError() const { return m_strError; };

	void*	GetPos() const;
	bool	SetPos(const void* savedpos);
	void	ReleasePos( void* savedpos ) const;

	// Create
	bool Save( const char* szFileName );
	std::string GetDoc() const { return m_strDoc; };
	bool AddElem( const char* szName, const char* szValue=NULL );
	bool AddChildElem( const char* szName, const char* szValue=NULL );
	bool InsertChildElem( const char* szName, const char* szValue=NULL );
	bool AddAttrib( const char* szAttrib, const char* szValue );
	bool AddChildAttrib( const char* szAttrib, const char* szValue );
	bool AddAttrib( const char* szAttrib, int nValue );
	bool AddChildAttrib( const char* szAttrib, int nValue );
	bool AddChildSubDoc( const char* szSubDoc );
	bool InsertChildSubDoc( const char* szSubDoc );
	std::string GetChildSubDoc() const;

	// Modify
	bool RemoveElem();
	bool RemoveChildElem();
	bool SetAttrib( const char* szAttrib, const char* szValue );
	bool SetChildAttrib( const char* szAttrib, const char* szValue );
	bool SetAttrib( const char* szAttrib, int nValue );
	bool SetChildAttrib( const char* szAttrib, int nValue );
	bool SetData( const char* szData, int nCDATA=0 ) { return x_SetData(m_iPos,szData,nCDATA); };
	bool SetChildData( const char* szData, int nCDATA=0 ) { return x_SetData(m_iPosChild,szData,nCDATA); };

protected:
	std::string m_strDoc;
	int m_nLevel;
	std::string m_strError;
	int m_nAllocDoc;

	struct ElemPos
	{
		ElemPos() { Clear(); };
		ElemPos( const ElemPos& pos ) { *this = pos; };
		bool IsEmptyElement() const { return (nStartR == nEndL + 1); };
		void Clear()
		{
			nStartL=0; nStartR=0; nEndL=0; nEndR=0; nNext=0;
			iElemParent=0; iElemChild=0; iElemNext=0;
		};
		int nStartL;
		int nStartR;
		int nEndL;
		int nEndR;
		int nNext;
		int iElemParent;
		int iElemChild;
		int iElemNext;
	};

	typedef std::vector<ElemPos> vectorElemPosT;
	vectorElemPosT m_aPos;
	int m_iPos;
	int m_iPosChild;
	int m_iPosFree;

	int x_GetFreePos();
	int x_ReleasePos();

	struct TokenPos
	{
		TokenPos() { Clear(); };
		bool IsValid() const { return (nL <= nR); };
		void Clear() { nL=0; nR=-1; bIsString=false; };
		int nL;
		int nR;
		int nNext;
		bool bIsString;
	};

	struct SavedPos
	{
		int iPos;
		int iPosChild;
		int iLevel;
	};
	typedef std::map<std::string,SavedPos> mapSavedPosT;
	mapSavedPosT m_mapSavedPos;

	int x_ParseElem( int iPos );
	int x_ParseError( const char* szError, const char* szTag = NULL );
	bool x_FindChar( int&n, char c ) const;
	bool x_FindToken( TokenPos& token ) const;
	std::string x_GetToken( const TokenPos& token ) const;
	std::string x_GetTagName( int iPos ) const;
	bool x_SetData( int iPos, const char* szData, int nCDATA=0 );
	std::string x_GetData( int iPos ) const;
	std::string x_GetAttrib( int iPos, const char* szAttrib ) const;
	int x_Add( int iPosParent, int iPosBefore, const char* szName, const char* szValue );
	int x_AddSubDoc( int iPosParent, int iPosBefore, const char* szSubDoc );
	bool x_FindAttrib( TokenPos& token, const char* szAttrib=NULL ) const;
	int x_AddAttrib( int iPos, const char* szAttrib, const char* szValue );
	int x_SetAttrib( int iPos, const char* szAttrib, const char* szValue );
	int x_Remove( int iPos );
	void x_DocChange( int nLeft, int nReplace, std::string& strInsert );
	void x_PosInsert( int iPos, int nInsertLength );
	void x_Adjust( int iPos, int nShift );
	std::string x_TextToDoc( const char* szText, bool bAttrib = false ) const;
	std::string x_TextFromDoc( int nLeft, int nRight ) const;
};

#endif // !defined(AFX_MARKUPSTL_H__948A2705_9E68_11D2_A0BF_00105A27C570__INCLUDED_)
