// OpenGLWnd1.cpp : implementation file
//

#include "stdafx.h"
#include <math.h>

#include "TRViewer.h"
#include "MainFrm.h"
#include "ChildFrm.h"
#include "OpenGLWnd1.h"
#include "TextureTools.h"
#include "TGAFile.h"

#include "ToolboxBase.h"
#include "ToolboxItfBase.h"
#include "TlboxMoveableItf.h"
#include "Toolbox2Moveable.h"

#include "AnimData.h"
#include "AnimKey.h"
#include "AnimTrack.h"
#include "AnimMgr.h"

#ifdef _DEBUG
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

extern bool g_boSavingLevel;

/////////////////////////////////////////////////////////////////////////////
// COpenGLWnd

COpenGLWnd::COpenGLWnd()
{
    m_hRC = NULL;
    m_hDC = NULL;
    m_boMakeCurrentOk = false;

    CMainFrame* lpMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);

    // General / rendering parameters
    m_boAutomaticCamera     = !lpMainFrame->boIsManualCamera();
    m_boShowAxis            = lpMainFrame->boShowAxis();
    m_BkgColor              = lpMainFrame->colGetBkgColor();

    m_dwOldTime         = -1;
    m_OldTime.QuadPart  = 0;
    m_fFrameRate        = 1.0;

    m_boPickingMode = false;
    m_dwShowNumVertex = -1;         // in picking mode, num vertex to select

    m_boRotWithMouse = false;
    m_boTransWithMouse = false;

	m_dwNumGfx			= 0;
	m_lOldTextureBinded	= -1;

    // Mesh / static mesh parameters
    m_boShowCollSphere      = false;
    m_boShowVertices        = false;
    m_boShowStaticBB1       = false;
    m_boStaticBB1IsSel      = false;
    m_boShowStaticBB2       = false;
    m_boStaticBB2IsSel      = false;
    m_dwNumMeshToRender     = 0;
	m_boShowMoveableBB		= false;

    // Anim parameters
    m_boAutomaticAnim       = true;
    m_boAnimShow            = false;
    m_boChainAnimations     = false;
    m_boAnimInterp          = true;
    m_boRecalcInterp        = false;
    m_dwAnimCurAnim         = 0;
    m_fAnimOverallSpeed     = (FLOAT) 1.0;

    // Sprite parameters
    m_fSprSeqSpeed          = 20.0;
    m_boSprSeqShowBorder    = false;
    m_boSprSeqTypeAnimated  = true;
    m_fSprSeqListWidth      = 256.0;

	m_szTextToDisplay[0]	= 0;
}


COpenGLWnd::~COpenGLWnd()
{
	for( DWORD dwI = 0; dwI < 2; ++dwI )
		{
		m_Gfx[ dwI ].AnimatedGfx.lpTrGfx	= NULL;
		m_Gfx[ dwI ].DisplayedGfx.lpTrGfx	= NULL;
		}

    vReleaseTextures();

    if(m_hRC)
        {
        //wglMakeCurrent( NULL, NULL );
        wglDeleteContext(m_hRC);
        }

    m_hRC = NULL;
}


void COpenGLWnd::vReleaseTextures()
{
	for( DWORD dwI = 0; dwI < 2; ++dwI )
		{
		vReleaseTextures_( m_Gfx[ dwI ].AnimatedGfx );
		vReleaseTextures_( m_Gfx[ dwI ].DisplayedGfx );
		}
}


void COpenGLWnd::vReleaseTextures_( sGfxInfo& Gfx )
{
    if(Gfx.LstTextId)
        {
        glDeleteTextures(Gfx.dwNumTextures,Gfx.LstTextId);
        delete [] Gfx.LstTextId;
        }

    Gfx.LstTextId = NULL;

	glDeleteLists(10000, 96);
}


BEGIN_MESSAGE_MAP(COpenGLWnd, CWnd)
	//{{AFX_MSG_MAP(COpenGLWnd)
	ON_WM_PAINT()
	ON_WM_SIZE()
    ON_WM_CREATE()
	ON_WM_LBUTTONDOWN()
	ON_WM_LBUTTONUP()
	ON_WM_MOUSEMOVE()
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()


/////////////////////////////////////////////////////////////////////////////
// COpenGLWnd message handlers

void COpenGLWnd::OnPaint() 
{
	CPaintDC dc(this); // device context for painting
	
    vDraw((HDC)dc);

	// Do not call CFrameWnd::OnPaint() for painting messages
}


int COpenGLWnd::OnCreate(LPCREATESTRUCT lpCreateStruct) 
{
	if (CWnd::OnCreate(lpCreateStruct) == -1)
		return -1;

	m_hDC = ::GetDC(m_hWnd);			// Get the Device context

    static PIXELFORMATDESCRIPTOR pfd =
    {
        sizeof(PIXELFORMATDESCRIPTOR),          // Size of this structure
        1,                                      // Version of this structure    
        0
        | PFD_SUPPORT_OPENGL                    // The buffer supports OpenGL 
        | PFD_GENERIC_ACCELERATED               // The pixel format is 
        | PFD_DOUBLEBUFFER                      // The buffer is 
        | PFD_SWAP_COPY                         // Specifies the content of 
        | PFD_SWAP_EXCHANGE                     // Specifies the content of 
        ,
        PFD_TYPE_RGBA,                          // RGBA Color mode
        16,                                     // Want 16bit color 
        0,0,0,0,0,0,                            // Not used to select mode
        0,0,                                    // Not used to select mode
        0,0,0,0,0,                              // Not used to select mode
        16,                                     // Size of depth buffer
        0,                                      // Not used to select mode or ignored
        0,                                      //  idem
        PFD_MAIN_PLANE ,                        // Draw in main plane
        0,                                      // Not used to select mode or ignored
        0,0,0                                   //  idem
    };

    int nMyPixelFormatID = ChoosePixelFormat( m_hDC, &pfd );

    if(!nMyPixelFormatID)
        {
        CString s;

        s.Format("Error when calling OpenGL ChoosePixelFormat function (%ld)",GetLastError());
        AfxMessageBox(s,MB_OK | MB_ICONSTOP | MB_TOPMOST);
        }
    else
        {
        SetPixelFormat( m_hDC, nMyPixelFormatID, &pfd );

        m_hRC = wglCreateContext( m_hDC );

        //wglMakeCurrent(m_hDC, m_hRC);
        m_boMakeCurrentOk = true;

        RECT rect;
        
        GetClientRect(&rect);        
        
        vChangeSize(rect.right - rect.left,rect.bottom - rect.top);
        }

	return 0;
}


void COpenGLWnd::OnSize(UINT nType, int cx, int cy) 
{
	CWnd::OnSize(nType, cx, cy);
	
    vChangeSize(cx,cy);
}


void COpenGLWnd::vChangeSize(int cx, int cy)
{
    if(!m_boMakeCurrentOk) return;

	m_iWidth	= cx;
	m_iHeight	= cy;

    m_gldAspect = (GLdouble)cx / (GLdouble)cy;
}


void COpenGLWnd::vSetProjMode()
{
    glViewport( 0, 0, m_iWidth, m_iHeight );

    glMatrixMode( GL_PROJECTION );
    glLoadIdentity();

    gluPerspective(
        60.0,           // Field-of-view angle
        m_gldAspect,    // Aspect ratio of view volume
        0.1,            // Distance to near clipping plane
        10.0 );         // Distance to far clipping plane

    glMatrixMode( GL_MODELVIEW );
}


void COpenGLWnd::vSetOrthoMode()
{
    glMatrixMode( GL_MODELVIEW );
	glLoadIdentity();

	glMatrixMode(GL_PROJECTION);
	glLoadIdentity();

	glOrtho(0, m_iWidth, m_iHeight, 0, 0.f, -1.f);
	glViewport(0, 0, m_iWidth, m_iHeight);

    glMatrixMode( GL_MODELVIEW );
}


void COpenGLWnd::vInitiateCamera(FLOAT fMinX,FLOAT fMaxX,FLOAT fMinY,FLOAT fMaxY,FLOAT fMinZ,FLOAT fMaxZ, bool boCenterCamera)
{
    // Center of the bounding box
    FLOAT x = (fMaxX + fMinX) / (FLOAT) 2.0;
    FLOAT y = (fMaxY + fMinY) / (FLOAT) 2.0;
    FLOAT z = (fMaxZ + fMinZ) / (FLOAT) 2.0;

    // Calculate the largest side of the bounding box
    FLOAT fZRange = fMaxZ - fMinZ;
    FLOAT fYRange = fMaxY - fMinY;
    FLOAT fXRange = fMaxX - fMinX;

    FLOAT fMax = fZRange;
    
    if(fMax < fYRange) fMax = fYRange;
    if(fMax < fXRange) fMax = fXRange;

    // Save 1/fMax for later use (scaling of the objects before the rendering, so that it fits in a box of 1 unit side)
    m_fMax = 1 / fMax;

    // x,y,z = coord. of the lookat -> the camera will focus at this point
    x = x * m_fMax;
    y = y * m_fMax;
    z = z * m_fMax;

    // Calculate the camera's position (we know the mesh fits in a box of 1 unit side now)
    FLOAT fEyeX = -(1.0 - 1.0);
    FLOAT fEyeY = (FLOAT) -(-1.0 + 0.3);
    FLOAT fEyeZ = 1.0 + 1.0;

    FLOAT fLookAtX = x;
    FLOAT fLookAtY = y;
    FLOAT fLookAtZ = z;

    if(boCenterCamera)
        {
        fEyeX = x;
        fEyeY = y;
        fEyeZ = 2.0;

        fLookAtX = x;
        fLookAtY = y;
        fLookAtZ = z;
        }

    // Set the parameters of the camera
    m_Camera.CamSetPosition(vec3(fEyeX,fEyeY,fEyeZ));
    m_Camera.CamSetLookAt(vec3(fLookAtX,fLookAtY,fLookAtZ));
    m_Camera.CamCalcViewUp(0);
    m_Camera.CamCalcMatrix();

    m_dwOldTime         = -1;
    m_OldTime.QuadPart  = 0;
    m_fFrameRate        = 1.0;
}


void    COpenGLWnd::vSetAutomaticCamera(bool boOk)
{
    m_boAutomaticCamera = boOk;
    
    if(boOk)
        {
        m_dwOldTime         = -1;
        m_OldTime.QuadPart  = 0;
        }
}


void    COpenGLWnd::vSetShowAxis(bool boOk)
{
    m_boShowAxis = boOk;
}


void    COpenGLWnd::vSetBkgColor(COLORREF BkgColor)
{
    m_BkgColor = BkgColor;
}


void COpenGLWnd::vGenerateTextures_( sGfxInfo& Gfx )
{
    CTrObjGfx* lpGfx = Gfx.lpTrGfx;
    
    DWORD dwWidth   = lpGfx->dwGetTextureWidth();
    DWORD dwHeight  = lpGfx->dwGetTextureHeight();

	if(dwWidth < 256) dwWidth = 256;
	
	dwHeight = (((dwHeight - 1) / 256) + 1) * 256;

    BYTE* lpData = new BYTE[dwWidth * dwHeight * 4];

    ASSERT(lpGfx != NULL);

    vPromoteTo32Bits(const_cast<BYTE*>(lpGfx->lpGetTextureData()),dwWidth,dwHeight,lpGfx->dwGetTextureDepth(),NULL,lpData,FALSE,lpGfx->dwGetTextureWidth(),lpGfx->dwGetTextureHeight());

	Gfx.dwNumTextures = dwHeight / 256;

    Gfx.LstTextId = new GLuint[ Gfx.dwNumTextures ];

    glPixelStorei(GL_UNPACK_ALIGNMENT,1);
    glGenTextures(Gfx.dwNumTextures,Gfx.LstTextId);

	for(DWORD dwTexture = 0; dwTexture < Gfx.dwNumTextures; dwTexture++)
		{
		glBindTexture(GL_TEXTURE_2D, Gfx.LstTextId[dwTexture]);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_S, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_WRAP_T, GL_REPEAT);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MAG_FILTER, GL_LINEAR);
		glTexParameteri(GL_TEXTURE_2D, GL_TEXTURE_MIN_FILTER, GL_LINEAR);

		BYTE*	lpbTextureData = lpData + dwTexture * 256 * 256 * 4;

		glTexImage2D(GL_TEXTURE_2D,0,GL_RGBA,256,256,0,GL_BGRA_EXT,GL_UNSIGNED_BYTE,lpbTextureData);
		}

    delete [] lpData;

	wglUseFontBitmaps (m_hDC, 32, 96, 10000);
}


void COpenGLWnd::vGenerateTextures( sGfxDisplay& Gfx )
{
    // Generate textures for OpenGL
    if( !Gfx.AnimatedGfx.LstTextId  &&  Gfx.AnimatedGfx.lpTrGfx )
        {
		vGenerateTextures_( Gfx.AnimatedGfx );

		if( Gfx.DisplayedGfx.lpTrGfx ) vGenerateTextures_( Gfx.DisplayedGfx );

		m_lOldTextureBinded = -1;

		vActivateWnd();
        }
}


void COpenGLWnd::vActivateWnd()
{
    m_dwOldTime         = -1;
    m_OldTime.QuadPart  = 0;
}


void	COpenGLWnd::vRenderAndAnimate( sGfxDisplay& Gfx )
{
	glPushMatrix();

    // Render the geometry
    switch( Gfx.AnimatedGfx.trObjType)
        {
        case TrObj_Type_Moveable:
            if( Gfx.AnimatedGfx.boShow ) vRenderMoveable( Gfx );
            break;
		case TrObj_Type_Sprite:
			vRenderSprSeq( Gfx.AnimatedGfx );
			break;
		default:
            vRenderMesh( Gfx.AnimatedGfx, m_dwNumMeshToRender, m_boShowCollSphere, false, m_boShowVertices );
            break;
        }

    // Animate the moveable
    if(m_boAnimShow  &&  Gfx.AnimatedGfx.trObjType == TrObj_Type_Moveable)
        {
		CToolbox2Moveable* lpToolbox = GET_TOOLBOX( CToolbox2Moveable );

        if(m_boAutomaticAnim  ||  m_boRecalcInterp)
            {
            if(CAnimTrack* lpAnim = static_cast<CTrMvb*>(Gfx.AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack())
                {
                lpAnim->vActivateInterpolation(m_boAnimInterp);

				if( Gfx.boShowDataInToolbox )
					{
					lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
					lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
					lpToolbox->vSetInterp(lpAnim->fGetInterpolationFactor());
					}

                if(m_boAutomaticAnim)
                    {
                    if(!static_cast<CTrMvb*>(Gfx.AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->boRunForward((float)(1.0 / m_fFrameRate))  &&  Gfx.boShowDataInToolbox)
                        {
						CAnimTrack* lpAnim = static_cast<CTrMvb*>(Gfx.AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();
						DWORD       dwNextTrackIndex, dwNextTrackFrame;

						m_dwAnimCurAnim = static_cast<CTrMvb*>(Gfx.AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->dwGetCurrentTrack();

						lpToolbox->vSetCurAnim(m_dwAnimCurAnim);
						lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
						lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
						lpToolbox->vSetAnimationFrameRate(lpAnim->dwGetFrameRate());
						static_cast<CTrMvb*>(Gfx.AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vGetNextAnimInfos(dwNextTrackIndex,dwNextTrackFrame);
						lpToolbox->vSetAnimationNextAnim(dwNextTrackIndex,dwNextTrackFrame);
                        }
                    }

                if( Gfx.boShowDataInToolbox ) m_boRecalcInterp = false;
                }
            }
        }

	glPopMatrix();
}



void COpenGLWnd::vSetTextToDisplay(const char* pszText)
{
	if (pszText == 0L) return;

	lstrcpyn(m_szTextToDisplay, pszText, sizeof(m_szTextToDisplay) - 1);
}


void COpenGLWnd::vRenderText()
{
	if (m_szTextToDisplay[0] == 0) return;

	vSetOrthoMode();

	glColor3d(0, 0, 0);
	glListBase(10000 - 32); 
	glRasterPos2f(1, (GLfloat)(m_iHeight - 4));

	glCallLists(strlen(m_szTextToDisplay), GL_UNSIGNED_BYTE, m_szTextToDisplay); 
}


void COpenGLWnd::vDraw(HDC hDC)
{
    if(!m_boMakeCurrentOk  ||  !m_Gfx[ 0 ].AnimatedGfx.lpTrGfx  ||  g_boSavingLevel) return;

    // Calculate the framerate
    bool  boUpdateCamera = (m_dwOldTime != -1)  &&  m_boAutomaticCamera;

    DWORD dwNewTime = GetTickCount();
    DWORD dwElapsedTime = dwNewTime - m_dwOldTime;

    if(!dwElapsedTime) dwElapsedTime = 1;

    m_fFrameRate = (float)(1000.0 / dwElapsedTime);

    m_dwOldTime = dwNewTime;

    // Calculate a more accurate framerate (if supported by the OS)
    LARGE_INTEGER NewTime;
    LARGE_INTEGER Frequency;

    if(QueryPerformanceCounter(&NewTime)  &&  QueryPerformanceFrequency(&Frequency))
        {
        LONGLONG llElapsedTime = NewTime.QuadPart - m_OldTime.QuadPart;

        m_fFrameRate = (FLOAT) ((double)Frequency.QuadPart / (double)llElapsedTime);

        if(m_fFrameRate < 0) m_fFrameRate = 20.0;

        m_OldTime = NewTime;
        }

    // Update the camera orientation (if automatic mode)
    if(boUpdateCamera)
        {
        if(m_fFrameRate < 5.0) m_fFrameRate = 30.0;
        m_Camera.CamRotateAxis((float)(360.0 / (3 * m_fFrameRate)));
        m_Camera.CamCalcViewUp(0);
        m_Camera.CamCalcMatrix();
        }

    // Set OpenGL parameters for the rendering
    wglMakeCurrent(m_hDC, m_hRC);

    glClearColor(GetRValue(m_BkgColor) / 255.0f, GetGValue(m_BkgColor) / 255.0f, GetBValue(m_BkgColor) / 255.0f,0x00);

    glClearDepth(1.0);
    glShadeModel(GL_SMOOTH);
    glEnable( GL_DEPTH_TEST );
    glDepthFunc(GL_LESS);
    glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );
    glEnable(GL_BLEND);
    glBlendFunc(GL_SRC_ALPHA,GL_ONE_MINUS_SRC_ALPHA);
    glEnable(GL_CULL_FACE);
    glEnable(GL_TEXTURE_2D);
    glTexEnvf(GL_TEXTURE_ENV,GL_TEXTURE_ENV_MODE,GL_MODULATE);
    glAlphaFunc(GL_GREATER, 0.5);

	for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI ) vGenerateTextures( m_Gfx[ dwI ] );

    // Push the camera matrix transformation on the stack
    vec3 vec3Eye    = m_Camera.CamGetPosition();
    vec3 vec3LookAt = m_Camera.CamGetLookAt();
    vec3 vec3ViewUp = m_Camera.CamGetViewUp();

	vSetProjMode();

    glLoadIdentity();
    gluLookAt(vec3Eye[0],vec3Eye[1],vec3Eye[2],vec3LookAt[0],vec3LookAt[1],vec3LookAt[2],-vec3ViewUp[0],-vec3ViewUp[1],-vec3ViewUp[2]);

    // Transformation in model space (a scale to fit the geometry in a box of 1 unit side)
    glScaled(m_fMax,m_fMax,m_fMax);

	for( dwI = 0; dwI < m_dwNumGfx; ++dwI ) vRenderAndAnimate( m_Gfx[ dwI ] );

    // Display the framerate
    TCHAR szFrameRate[32];

    wsprintf(szFrameRate,"%.4d fps",(DWORD)m_fFrameRate);

	//vSetTextToDisplay(szFrameRate);

	vRenderText();

    SwapBuffers(hDC);

    wglMakeCurrent(NULL, NULL);
}


void COpenGLWnd::vPick(DWORD dwPosX,DWORD dwPosY)
{
    #define BUFSIZE     512

    GLuint  selectBuf[BUFSIZE];
    GLint   hits;
    GLint   viewport[4];

    m_boPickingMode = true;

    wglMakeCurrent(m_hDC, m_hRC);

    glGetIntegerv(GL_VIEWPORT, viewport);
    glSelectBuffer(BUFSIZE, selectBuf);
    glRenderMode(GL_SELECT);

    glInitNames();
    glPushName(0);

    glMatrixMode( GL_PROJECTION );
    glPushMatrix();
    glLoadIdentity();
    gluPickMatrix((GLdouble)dwPosX,(GLdouble)(viewport[3] - dwPosY),8.0,8.0,viewport);
    gluPerspective(
        60.0,           // Field-of-view angle
        m_gldAspect,    // Aspect ratio of view volume
        0.1,            // Distance to near clipping plane
        10.0 );         // Distance to far clipping plane

    // Push the camera matrix transformation on the stack
    vec3 vec3Eye    = m_Camera.CamGetPosition();
    vec3 vec3LookAt = m_Camera.CamGetLookAt();
    vec3 vec3ViewUp = m_Camera.CamGetViewUp();

    glMatrixMode( GL_MODELVIEW );
    glLoadIdentity();
    gluLookAt(vec3Eye[0],vec3Eye[1],vec3Eye[2],vec3LookAt[0],vec3LookAt[1],vec3LookAt[2],-vec3ViewUp[0],-vec3ViewUp[1],-vec3ViewUp[2]);

    // Transformation in model space (a scale to fit the geometry in a box of 1 unit side)
    glScaled(m_fMax,m_fMax,m_fMax);

    // Render the geometry
    if(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx  &&  m_dwNumMeshToRender != -1) vRenderMeshVertices( static_cast<CTrLstMsh*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetMesh(m_dwNumMeshToRender));

    glFlush();
    hits = glRenderMode(GL_RENDER);
    glMatrixMode( GL_PROJECTION );
    glPopMatrix();
    glMatrixMode( GL_MODELVIEW );
    m_boPickingMode = false;
    wglMakeCurrent(NULL, NULL);

    DWORD dwMinZ = 0xFFFFFFFF;
    m_dwShowNumVertex = -1;

    while(hits--)
        {
        if(selectBuf[4*hits+1] < dwMinZ)
            {
            dwMinZ = selectBuf[4*hits+1];
            m_dwShowNumVertex = selectBuf[4*hits+3]-1;
            }
        }

    InvalidateRect(NULL);
}


void COpenGLWnd::vCheckKeys()
{
    bool boCapslocked   = GetAsyncKeyState(VK_CAPITAL) & 0x8001 ? true : false;

    if(!boCapslocked) return;

    bit16 sXSgn = GetAsyncKeyState(VK_LEFT)  & 0x8001 ? -1 : GetAsyncKeyState(VK_RIGHT) & 0x8001 ? +1 : 0;
    bit16 sYSgn = GetAsyncKeyState(VK_PRIOR) & 0x8001 ? +1 : GetAsyncKeyState(VK_NEXT)  & 0x8001 ? -1 : 0;
    bit16 sZSgn = GetAsyncKeyState(VK_UP)    & 0x8001 ? -1 : GetAsyncKeyState(VK_DOWN)  & 0x8001 ? +1 : 0;
    bit16 sWSgn = GetAsyncKeyState(VK_END)   & 0x8001 ? -1 : GetAsyncKeyState(VK_DELETE)& 0x8001 ? +1 : 0;

    if(!sXSgn  &&  !sYSgn  &&  !sZSgn  &&  !sWSgn) return;

    m_Camera.CamMoveForward(sYSgn * 0.05f * 100.0f/m_fFrameRate);
    m_Camera.CamRotateAxis(sXSgn * 100.0f / m_fFrameRate);
    m_Camera.CamRotate(sZSgn * 100.0f / m_fFrameRate);
    m_Camera.CamRoll(sWSgn * 100.0f / m_fFrameRate);

    m_Camera.CamCalcMatrix();
}


//void COpenGLWnd::vSetGfxAddr(CTrObjGfx* lpTrMsh,enumTrObjType trObjType,bool boDontReleaseTextures,DWORD dwNumMeshToRender,CTrMvb* lpMvbToDisplay /*= NULL*/)
void COpenGLWnd::vSetGfxAddr( sGfxDisplay* lpLstGfx, DWORD dwNumGfx, bool boDontReleaseTextures, DWORD dwNumMeshToRender )
{
    if( !boDontReleaseTextures )
        {
        vReleaseTextures();
        }

	for( DWORD dwI = 0; dwI < dwNumGfx; ++dwI )
		{
		m_Gfx[ dwI ] = lpLstGfx[ dwI ];
		}

	m_dwNumGfx			= dwNumGfx;
    m_dwNumMeshToRender	= dwNumMeshToRender;

    m_dwShowNumVertex       = -1;
    m_boAnimShow            = false;

    //
    // Calculate the initial position / lookat of the camera. We calculate a coeff to apply to each coord. of the
    // mesh's vertices so that the mesh fits in a box of 1 unit side. So, in calculating the camera's position/lookat,
    // we assume the mesh is in this box
    //

    CBoundingBox BoundingBox;

	for( dwI = 0; dwI < dwNumGfx; ++dwI )
		{
		CBoundingBox BoundingBox_;

	    static_cast<CTrLstMsh*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->vComputeBoundingBox(BoundingBox_,m_dwNumMeshToRender,NULL);

		BoundingBox.vEnlarge( BoundingBox_ );

	    if( m_Gfx[ dwI ].DisplayedGfx.lpTrGfx ) static_cast<CTrLstMsh*>(m_Gfx[ dwI ].DisplayedGfx.lpTrGfx)->vComputeBoundingBox(BoundingBox_,m_dwNumMeshToRender,NULL);

		BoundingBox.vEnlarge( BoundingBox_ );
		}

    if(BoundingBox.GetBBLo()[0] >= BoundingBox.GetBBHi()[0])
        {
        // Handle the case where all meshes are dummy mesh
        // (if so, there's no valid bounding box, so create a default one)
        BoundingBox.vSetBB(CVect3D(-50,-50,-50),CVect3D(+50,+50,+50));
        }

    const CVect3D& v1 = BoundingBox.GetBBLo();
    const CVect3D& v2 = BoundingBox.GetBBHi();

    vInitiateCamera(v1[0],v2[0],v1[1],v2[1],v1[2],v2[2]);

    InvalidateRect(NULL,FALSE);

	for( dwI = 0; dwI < dwNumGfx; ++dwI )
		{
		if( m_Gfx[ dwI ].AnimatedGfx.trObjType == TrObj_Type_Moveable )
			{
			static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vSetChainTracks(m_boChainAnimations);
			static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vSetOverallSpeed(m_fAnimOverallSpeed);
			}
		}
}


void COpenGLWnd::vSetGfxVisiblity( DWORD dwNumGfx, bool boAnimatedVisible, bool boDisplayedVisible )
{
	if( dwNumGfx >= m_dwNumGfx ) return;

	m_Gfx[ dwNumGfx ].AnimatedGfx.boShow = boAnimatedVisible;
	m_Gfx[ dwNumGfx ].DisplayedGfx.boShow = boDisplayedVisible;
}


void COpenGLWnd::vSetSprSeqAddr( sGfxDisplay* lpLstGfx, DWORD dwNumGfx, bool boDontReleaseTextures )
{
    if( !boDontReleaseTextures )
        {
        vReleaseTextures();
        }

	for( DWORD dwI = 0; dwI < dwNumGfx; ++dwI )
		{
		m_Gfx[ dwI ] = lpLstGfx[ dwI ];
		}

	m_dwNumGfx			= dwNumGfx;

    m_dwShowNumVertex   = -1;
    m_boAnimShow        = false;
    m_boAutomaticCamera = false;

    CMainFrame* lpMainFrame = (CMainFrame*)(AfxGetApp()->m_pMainWnd);

    lpMainFrame->vSetCameraMode(!m_boAutomaticCamera);

    //
    // Calculate the initial position / lookat of the camera. We calculate a coeff to apply to each coord. of the
    // mesh's vertices so that the mesh fits in a box of 1 unit side. So, in calculating the camera's position/lookat,
    // we assume the mesh is in this box
    //

    // Get the extents of the bounding box for the mesh
    const WORLD_POSITIONNING* lpWorldP = static_cast<CTrLstSpr*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetWorldPositionning();

    FLOAT fMinX = lpWorldP[0].sLeftSide, fMaxX = lpWorldP[0].sRightSide;
    FLOAT fMinY = (FLOAT)-lpWorldP[0].sBottomSide, fMaxY = (FLOAT)-lpWorldP[0].sTopSide;
    FLOAT fMinZ = -200, fMaxZ = 200;

    if(fMinX < fMaxX)
        {
        vInitiateCamera(fMinX,fMaxX,fMinY,fMaxY,fMinZ,fMaxZ,true);

        InvalidateRect(NULL,FALSE);
        }
}


void COpenGLWnd::vModifyIntensOfSelectedVertices(bit16 sStep)
{
    if(!sStep  ||  m_dwShowNumVertex == -1  ||  !m_Gfx[ 0 ].AnimatedGfx.lpTrGfx  ||  m_dwNumMeshToRender == -1) return;
    
    TR123_MESH* lpMesh = const_cast<TR123_MESH*>(static_cast<CTrLstMsh*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetMesh(m_dwNumMeshToRender));

    if(lpMesh->Lighting.LightingType != MESH_LIGHTING_INTERNAL) return;

    bit16 sIntens = lpMesh->Lighting.lpLights[m_dwShowNumVertex];

    sIntens += sStep;
    if(sIntens >= 8192) sIntens = 8192;

    lpMesh->Lighting.lpLights[m_dwShowNumVertex] = sIntens;
}


void    COpenGLWnd::vSetMoveableAnimation(DWORD dwNumAnim)
{
    if(m_Gfx[ 0 ].AnimatedGfx.trObjType != TrObj_Type_Moveable) return;

    if(dwNumAnim == -1)
        {
        m_boAnimShow = false;
        return;
        }

    m_boAnimShow = true;
    m_dwAnimCurAnim = dwNumAnim;

	for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI )
		static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vSetCurrentTrack(m_dwAnimCurAnim);

    CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();
    DWORD       dwNextTrackIndex,dwNextTrackFrame;

	CToolbox2Moveable* lpToolbox = GET_TOOLBOX( CToolbox2Moveable );

    lpToolbox->vSetCurAnim(m_dwAnimCurAnim);
    lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
    lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
    lpToolbox->vSetAnimationFrameRate(lpAnim->dwGetFrameRate());
    static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vGetNextAnimInfos(dwNextTrackIndex,dwNextTrackFrame);
    lpToolbox->vSetAnimationNextAnim(dwNextTrackIndex,dwNextTrackFrame);
}


void    COpenGLWnd::vSetAnimationType(bool boChainAnimations,bool boResetMoveable)
{
    if(m_Gfx[ 0 ].AnimatedGfx.trObjType != TrObj_Type_Moveable) return;

    m_boChainAnimations = boChainAnimations;

	for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI )
		static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vSetChainTracks(boChainAnimations);

    if(boResetMoveable) vSetMoveableAnimation(m_dwAnimCurAnim);
}


void    COpenGLWnd::vSetAnimOverallSpeed(FLOAT fSpeed)
{
    if(m_Gfx[ 0 ].AnimatedGfx.trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

    FLOAT m_fAnimOverallSpeed;

    if(fSpeed >= 0.5f)
        m_fAnimOverallSpeed = (FLOAT) ((fSpeed - 0.5) * 5.0 + 1.0);
    else
        m_fAnimOverallSpeed = (FLOAT) (fSpeed * 2.0);

	for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI )
		static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vSetOverallSpeed(m_fAnimOverallSpeed);
}


void    COpenGLWnd::vSetAnimCurFrame(DWORD dwNumFrame)
{
    if(m_Gfx[ 0 ].AnimatedGfx.trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

	for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI )
		{
		CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

		lpAnim->boSetCurrentFrame((float)dwNumFrame);
		}

	CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

	CToolbox2Moveable* lpToolbox = GET_TOOLBOX( CToolbox2Moveable );

    lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
    lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
    lpToolbox->vSetInterp(lpAnim->fGetInterpolationFactor());
}


void    COpenGLWnd::vSetAnimCurKey(DWORD dwNumKey)
{
    if(m_Gfx[ 0 ].AnimatedGfx.trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

	for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI )
		{
		CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

		lpAnim->boSetCurrentFrame((float)(dwNumKey * lpAnim->dwGetFrameRate()));
		}

	CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

	CToolbox2Moveable* lpToolbox = GET_TOOLBOX( CToolbox2Moveable );

    lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
    lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
    lpToolbox->vSetInterp(lpAnim->fGetInterpolationFactor());
}


void    COpenGLWnd::vSetAnimInterp(bool boOk)
{
    m_boAnimInterp = boOk;
}


void    COpenGLWnd::vChangeAnimInterp(int iDir)
{
    if(m_Gfx[ 0 ].AnimatedGfx.trObjType != TrObj_Type_Moveable  ||  !static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()) return;

	CToolbox2Moveable* lpToolbox = GET_TOOLBOX( CToolbox2Moveable );

	for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI )
		{
		CAnimTrack* lpAnim = static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

		FLOAT fFrame = lpAnim->fGetCurFrame();

		fFrame += iDir * ((FLOAT)1.0 / (FLOAT)30.0) * lpAnim->fGetOverallSpeed();

		if(!static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->boSetCurrentFrame(fFrame))
			{
			DWORD dwNextTrackIndex,dwNextTrackFrame;

			lpAnim = static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack();

			m_dwAnimCurAnim = static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->dwGetCurrentTrack();

			if( dwI == 0 )
				{
				lpToolbox->vSetCurAnim(m_dwAnimCurAnim);
				lpToolbox->vSetAnimationFrameRate(lpAnim->dwGetFrameRate());
				static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->vGetNextAnimInfos(dwNextTrackIndex,dwNextTrackFrame);
				lpToolbox->vSetAnimationNextAnim(dwNextTrackIndex,dwNextTrackFrame);
				}
			}

		lpToolbox->vSetCurFrame((DWORD)lpAnim->fGetCurFrame(),lpAnim->dwGetNumFrames());
		lpToolbox->vSetCurKey(lpAnim->dwGetCurKey(),lpAnim->dwGetNumKeys());
		}

    m_boRecalcInterp = true;
}


void    COpenGLWnd::vSetAnimInterp(FLOAT fInterp)
{
    if(m_Gfx[ 0 ].AnimatedGfx.trObjType == TrObj_Type_Moveable  &&  static_cast<CTrMvb*>(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack())
		{
		for( DWORD dwI = 0; dwI < m_dwNumGfx; ++dwI )
			static_cast<CTrMvb*>(m_Gfx[ dwI ].AnimatedGfx.lpTrGfx)->lpGetAnimMgr()->lpGetCurrentTrack()->vSetInterpolationFactor(fInterp);
		}
}


void COpenGLWnd::OnLButtonDown(UINT nFlags, CPoint point) 
{
	m_boRotWithMouse = true;
    m_RotMousePos = point;
}

void COpenGLWnd::OnLButtonUp(UINT nFlags, CPoint point) 
{
	m_boRotWithMouse = false;
}

void COpenGLWnd::OnRButtonDown(UINT nFlags, CPoint point) 
{
	m_boTransWithMouse = true;
    m_TransMousePos = point;
}

void COpenGLWnd::OnRButtonUp(UINT nFlags, CPoint point) 
{
	m_boTransWithMouse = false;
}

void COpenGLWnd::OnMouseWheel(UINT nFlags, short zDelta, CPoint pt) 
{
    //if(m_boAutomaticCamera) return;

    m_Camera.CamMoveForward(zDelta * 0.2f / m_fFrameRate);
    m_Camera.CamCalcMatrix();
}

void COpenGLWnd::OnLButtonDblClk(UINT nFlags, CPoint point)
{
    //if(m_boAutomaticCamera) return;

    if(m_Gfx[ 0 ].AnimatedGfx.lpTrGfx)
		{
		if(m_Gfx[ 0 ].AnimatedGfx.trObjType == TrObj_Type_Sprite)
			{
			vSetSprSeqAddr(m_Gfx,m_dwNumGfx,true);
			}
		else
			{
	        vSetGfxAddr(m_Gfx,m_dwNumGfx,true,m_dwNumMeshToRender);
			}
		}
}

void COpenGLWnd::OnMouseMove(UINT nFlags, CPoint point) 
{
    //if(m_boAutomaticCamera) return;

	if(m_boRotWithMouse)
        {
        int iDeltaX = point.x - m_RotMousePos.x;
        int iDeltaY = point.y - m_RotMousePos.y;

        FLOAT fXSgn = -iDeltaX * 2 * 100.0f / m_fFrameRate;
        FLOAT fZSgn = -iDeltaY * 2 * 100.0f / m_fFrameRate;

        m_Camera.CamRotateAxis(fXSgn);
        m_Camera.CamRotate(fZSgn);
        m_Camera.CamCalcMatrix();

        m_RotMousePos = point;
        }

    if(m_boTransWithMouse)
        {
        int iDeltaX = point.x - m_TransMousePos.x;
        int iDeltaY = point.y - m_TransMousePos.y;

        FLOAT fXSgn = -iDeltaX * 0.01f * 100.0f / m_fFrameRate;
        FLOAT fYSgn = -iDeltaY * 0.01f * 100.0f / m_fFrameRate;

        m_Camera.CamMoveLeft(fXSgn);
        m_Camera.CamMoveUp(fYSgn);
        m_Camera.CamCalcMatrix();

        m_TransMousePos = point;
        }
}

void COpenGLWnd::vZoom(FLOAT fValue)
{
    //if(m_boAutomaticCamera) return;

    m_Camera.CamMoveForward(fValue);
    m_Camera.CamCalcMatrix();
}

