#include "stdafx.h"

#include "Resource.h"
#include "TRViewer.h"
#include "Split.h"
#include "ResMgr.h"
#include "MarkupSTL.h"


CResItem::CResItem(TCHAR* lpszName, TCHAR* lpszLevelName)
{
    lstrcpyn(m_szName,lpszName,128);
    lstrcpyn(m_szLevelName,lpszLevelName,128);
}


void    CResItem::vAddObject(enumTrObjType ObjType, DWORD dwObjId)
{
    m_ListObjects.push_back(CResObject(ObjType, dwObjId));
}


CResMgr::CResMgr()
{
    lstrcpy(m_szPath, _T(".\\"));

	m_lpIconsImage = NULL;
}


CResMgr::~CResMgr()
{
    vRelease();
}


void    CResMgr::vRelease()
{
    for(DWORD dwI = 0; dwI < __RESMGR_MAXTR__; dwI++)
        {
        m_Menu[dwI].DestroyMenu();

        for(ListResItem::iterator theIter = m_List[dwI].begin(); theIter != m_List[dwI].end(); theIter++)
            {
            delete *theIter;
            }

        m_List[dwI].clear();

        for(ListSfx::iterator theIter2 = m_ListSFX[dwI].begin(); theIter2 != m_ListSFX[dwI].end(); theIter2++)
            {
            delete *theIter2;
            }

        m_ListSFX[dwI].clear();

		m_XMLRes.ReleasePos( m_XMLTr[ dwI ].lpPosRoot );
		m_XMLRes.ReleasePos( m_XMLTr[ dwI ].lpPosLevels );
		m_XMLRes.ReleasePos( m_XMLTr[ dwI ].lpPosMoveables );
		m_XMLRes.ReleasePos( m_XMLTr[ dwI ].lpPosSprites );
		m_XMLRes.ReleasePos( m_XMLTr[ dwI ].lpPosSounds );
        }

	m_XMLRes.ReleasePos( m_XMLIconsPos );

	delete m_lpIconsImage;

	m_lpIconsImage = NULL;
}


void    CResMgr::vInitialize(TCHAR* lpszPath)
{
    vRelease();

    if(lpszPath) lstrcpyn(m_szPath, lpszPath, MAX_PATH);

    CString cstrMenuResource(m_szPath);

    cstrMenuResource += _T("MenuResource.txt");

    for(DWORD dwI = 0; dwI < __RESMGR_MAXTR__; dwI++)
        {
        // Create the "add resource" menu
        m_Menu[dwI].CreatePopupMenu();
    
        CMenu   SubMenu;

        SubMenu.CreatePopupMenu();
    
        m_Menu[dwI].AppendMenu(MF_POPUP,(UINT)SubMenu.m_hMenu,_T("Add resource"));

        enumTRType trType = TR_UNKNOWN;

        switch(dwI)
            {
            case __RESMGR_TR1__: trType = TR1_FILE; break;
            case __RESMGR_TR2__: trType = TR2_FILE; break;
            case __RESMGR_TR3__: trType = TR3_FILE; break;
            case __RESMGR_TR4__: trType = TR4_FILE; break;
            }

        vParseFile_MenuResource(SubMenu, (LPTSTR)(LPCTSTR)cstrMenuResource, trType, m_List[dwI]);

        SubMenu.Detach();

        // Load the SFXs
        CString cstrSFXFilename(m_szPath);

        switch(dwI)
            {
            case __RESMGR_TR1__:        cstrSFXFilename += _T("SFX_TR1.txt"); break;
            case __RESMGR_TR1GOLD__:    cstrSFXFilename += _T("SFX_TR1Gold.txt"); break;
            case __RESMGR_TR2__:        cstrSFXFilename += _T("SFX_TR2.txt"); break;
            case __RESMGR_TR2GOLD__:    cstrSFXFilename += _T("SFX_TR2Gold.txt"); break;
            case __RESMGR_TR3__:        cstrSFXFilename += _T("SFX_TR3.txt"); break;
            case __RESMGR_TR3GOLD__:    cstrSFXFilename += _T("SFX_TR3Gold.txt"); break;
            case __RESMGR_TR4__:        cstrSFXFilename += _T("SFX_TR4.txt"); break;
            }

        vParseFile_SFX(cstrSFXFilename, m_ListSFX[dwI]);
        }

	vCreateIconsImage();

	// Load the catalog
	CString cstrCatalog( m_szPath );
	CString sErr;

	cstrCatalog += _T("TRCatalog.xml");

	if( !m_XMLRes.Load( cstrCatalog ) )
		{
		sErr.Format( _T("Can't read the catalog: file %s not found or bad format (error: %s) !"), cstrCatalog, m_XMLRes.GetError().c_str() );

		AfxMessageBox( sErr, MB_ICONEXCLAMATION );

		return;
		}

	ZeroMemory( m_XMLTr, sizeof( sXMLTr ) * __RESMGR_MAXTR__ );

	if( m_XMLRes.FindChildElem( "icons" ) )
		{
		m_XMLRes.IntoElem();
		m_XMLIconsPos = m_XMLRes.GetPos();
		m_XMLRes.OutOfElem();
		}

	while( m_XMLRes.FindChildElem( "game" ) )
		{
		void*	lpPos = m_XMLRes.GetPos();

		std::string strTR				= m_XMLRes.GetChildAttrib( "id" );

		m_XMLRes.IntoElem();

		BYTE bResId = bGetResIdFromTRAbrev( strTR.c_str() );

		if( bResId != __RESMGR_TRUNK__ )
			{
			m_XMLTr[ bResId ].lpPosRoot = m_XMLRes.GetPos();

			if( m_XMLRes.FindChildElem( "levels" ) )
				{
				m_XMLRes.IntoElem();
				m_XMLTr[ bResId ].lpPosLevels = m_XMLRes.GetPos();
				m_XMLRes.OutOfElem();
				}

			if( m_XMLRes.FindChildElem( "moveables" ) )
				{
				m_XMLRes.IntoElem();
				m_XMLTr[ bResId ].lpPosMoveables = m_XMLRes.GetPos();
				m_XMLRes.OutOfElem();
				}

			if( m_XMLRes.FindChildElem( "sprites" ) )
				{
				m_XMLRes.IntoElem();
				m_XMLTr[ bResId ].lpPosSprites = m_XMLRes.GetPos();
				m_XMLRes.OutOfElem();
				}

			if( m_XMLRes.FindChildElem( "sounds" ) )
				{
				m_XMLRes.IntoElem();
				m_XMLTr[ bResId ].lpPosSounds = m_XMLRes.GetPos();
				m_XMLRes.OutOfElem();
				}
			}
		else
			{
			sErr.Format( _T("Catalog parsing: invalid id '%s' for the <game> tag !"), strTR.c_str() );

			AfxMessageBox( sErr, MB_ICONEXCLAMATION );
			}

		m_XMLRes.SetPos( lpPos );
		m_XMLRes.ReleasePos( lpPos );
		}
}


DWORD		CResMgr::dwGetIconId( const std::string& strIconName ) const
{
	if( !m_XMLIconsPos  ||  strIconName.empty() ) return 0;

	void*	lpPos = m_XMLRes.GetPos();

	m_XMLRes.SetPos( m_XMLIconsPos );

	while( m_XMLRes.FindChildElem( "icon" ) )
		{
		if( lstrcmpi( m_XMLRes.GetChildAttrib( "name" ).c_str(), strIconName.c_str() ) == 0 )
			{
			DWORD	dwIconId = atol( m_XMLRes.GetChildAttrib( "id" ).c_str() );

			m_XMLRes.SetPos( lpPos );
			m_XMLRes.ReleasePos( lpPos );
			
			return dwIconId;
			}
		}

	m_XMLRes.SetPos( lpPos );
	m_XMLRes.ReleasePos( lpPos );

	return 0;
}


bool	CResMgr::boFindXMLNode( BYTE bTRResId, const std::string& strLevelShortName, const std::string& strCategName, const std::string& strNodeName, const std::string& strIdToFind ) const
{
	if( bTRResId == __RESMGR_TRUNK__ ) return false;

	bool	boFound = false;

	void*	lpPos = m_XMLRes.GetPos();

	if( m_XMLTr[ bTRResId ].lpPosLevels  &&  !strLevelShortName.empty() )
		{
		m_XMLRes.SetPos( m_XMLTr[ bTRResId ].lpPosLevels );

		if( boFindXMLNode( "level", strLevelShortName ) )
			{
			if( strCategName.empty() )
				{
				boFound = true;
				}
			else
				{
				if( strNodeName.empty() )
					{
					boFound = boFindXMLNode( strCategName, strIdToFind );
					}
				else if( boFindXMLNode( strCategName, "" ) )
					{
					boFound = boFindXMLNode( strNodeName, strIdToFind );
					}
				}
			}
		}

	if( !boFound  &&  m_XMLTr[ bTRResId ].lpPosRoot )
		{
		m_XMLRes.SetPos( m_XMLTr[ bTRResId ].lpPosRoot );

		if( strNodeName.empty() )
			{
			boFound = boFindXMLNode( strCategName, strIdToFind );
			}
		else if( boFindXMLNode( strCategName, "" ) )
			{
			boFound = boFindXMLNode( strNodeName, strIdToFind );
			}
		}

	if( !boFound ) m_XMLRes.SetPos( lpPos );

	m_XMLRes.ReleasePos( lpPos );

	return boFound;
}


bool	CResMgr::boFindXMLNode( const std::string& strNodeName, const std::string& strIdToFind ) const
{
	bool boOk = false;

	if( !strNodeName.empty() )
		{
		while( m_XMLRes.FindChildElem( strNodeName.c_str() ) )
			{
			if( strIdToFind.empty() )
				{
				boOk = true;
				break;
				}

			if( lstrcmpi( m_XMLRes.GetChildAttrib( "id" ).c_str(), strIdToFind.c_str() ) == 0 )
				{
				boOk = true;
				break;
				}
			}
		}

	if( boOk ) m_XMLRes.IntoElem();

	return boOk;
}


CResLevel	CResMgr::GetLevelInfo( BYTE bTRResId, const std::string& strLevelShortName ) const
{
	if( !boFindXMLNode( bTRResId, strLevelShortName, "", "", "" ) ) return CResLevel( strLevelShortName, strLevelShortName, false, 0 );

	bool	boCutScene	= lstrcmpi( m_XMLRes.GetAttrib( "type" ).c_str(), "cutscene" ) == 0;
	DWORD	dwIconId	= dwGetIconId( m_XMLRes.GetAttrib( "icon" ) );

	std::string& strName = m_XMLRes.GetAttrib( "name" );

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();

	if( strName.empty()  ||  !lpApp->boShowFriendlyNamesInTreeview() ) strName = strLevelShortName;

	return CResLevel( strLevelShortName, strName, boCutScene, dwIconId );
}


CResCategory	CResMgr::GetCategoryInfo( BYTE bTRResId, const std::string& strLevelShortName, const std::string& strCategoryName ) const
{
	if( !boFindXMLNode( bTRResId, strLevelShortName, strCategoryName, "", "" ) ) return CResCategory( strCategoryName, 0 );

	DWORD	dwIconId	= dwGetIconId( m_XMLRes.GetAttrib( "icon" ) );

	std::string& strName = m_XMLRes.GetAttrib( "name" );

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();

	if( strName.empty()  ||  !lpApp->boShowFriendlyNamesInTreeview() ) strName = strCategoryName;

	return CResCategory( strName, dwIconId );
}


CResMoveable*	CResMgr::lpGetMoveableInfo( BYTE bTRResId, const std::string& strLevelShortName, DWORD dwObjId ) const
{
	CString s;

	s.Format( "%ld", dwObjId );

	if( !boFindXMLNode( bTRResId, strLevelShortName, "moveables", "moveable", (LPCTSTR) s ) )
		{
		s.Format( "Moveable #%.3d", dwObjId );

		return new CResMoveable( dwObjId,  (LPCTSTR) s, 0, -1, -1, NULL );
		}

	std::string& strName		= m_XMLRes.GetAttrib( "name" );
	std::string& strUseBodyFrom	= m_XMLRes.GetAttrib( "use_body_from" );
	std::string& strShowWith	= m_XMLRes.GetAttrib( "show_with" );
	DWORD		dwBodyMvbId		= -1;
	DWORD		dwShowWithId	= -1;

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();

	if( strName.empty()  ||  !lpApp->boShowFriendlyNamesInTreeview() )
		{
		s.Format( "Moveable #%.3d", dwObjId );

		strName = s;
		}

	if( !strUseBodyFrom.empty() )	dwBodyMvbId		= atol( strUseBodyFrom.c_str() );
	if( !strShowWith.empty() )		dwShowWithId	= atol( strShowWith.c_str() );

	// Loop through states
	CResMoveable::MapStateId2Descr*	lpLstStates = new CResMoveable::MapStateId2Descr;

	while( boFindXMLNode( "state", "" ) )
		{
		std::string& strId		= m_XMLRes.GetAttrib( "id" );
		std::string& strName	= m_XMLRes.GetAttrib( "name" );

		lpLstStates->insert( std::pair< DWORD, std::string>( atol( strId.c_str() ), strName ) );

		m_XMLRes.OutOfElem();
		}

	return new CResMoveable( dwObjId, strName, dwGetIconId( m_XMLRes.GetAttrib( "icon" ) ), dwBodyMvbId, dwShowWithId, lpLstStates );
}


CResStaticMesh	CResMgr::GetStaticMeshInfo( BYTE bTRResId, const std::string& strLevelShortName, DWORD dwObjId ) const
{
	CString s;

	s.Format( "%ld", dwObjId );

	if( !boFindXMLNode( bTRResId, strLevelShortName, "static_meshes", "static_mesh", (LPCTSTR) s ) )
		{
		s.Format( "Static_mesh #%.3d", dwObjId );

		return CResStaticMesh( dwObjId,  (LPCTSTR) s, 0 );
		}

	std::string& strName = m_XMLRes.GetAttrib( "name" );

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();

	if( strName.empty()  ||  !lpApp->boShowFriendlyNamesInTreeview() )
		{
		s.Format( "Static_mesh #%.3d", dwObjId );

		strName = s;
		}

	return CResStaticMesh( dwObjId, strName, dwGetIconId( m_XMLRes.GetAttrib( "icon" ) ) );
}


CResSprite	CResMgr::GetSpriteInfo( BYTE bTRResId, const std::string& strLevelShortName, DWORD dwObjId ) const
{
	CString s;

	s.Format( "%ld", dwObjId );

	if( !boFindXMLNode( bTRResId, strLevelShortName, "sprites", "sprite", (LPCTSTR) s ) )
		{
		s.Format( "Sprite #%.3d", dwObjId );

		return CResSprite( dwObjId,  (LPCTSTR) s, 0, true, 0.0f );
		}

	std::string& strName = m_XMLRes.GetAttrib( "name" );

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();

	if( strName.empty()  ||  !lpApp->boShowFriendlyNamesInTreeview() )
		{
		s.Format( "Sprite #%.3d", dwObjId );

		strName = s;
		}

	bool boIsAnimated = lstrcmpi( m_XMLRes.GetAttrib( "type" ).c_str(), "animated" ) == 0;
	float fZoom = (float)atof( m_XMLRes.GetAttrib( "zoom" ).c_str() );

	return CResSprite( dwObjId, strName, dwGetIconId( m_XMLRes.GetAttrib( "icon" ) ), boIsAnimated, fZoom );
}


CString	CResMgr::cstrGetTRAbrevFromResId( BYTE bTRResId )
{
    CString cstrName;

    switch(bTRResId)
        {
        case __RESMGR_TR1__:        cstrName = _T("TR1");       break;
        case __RESMGR_TR1GOLD__:    cstrName = _T("TR1Gold");   break;
        case __RESMGR_TR2__:        cstrName = _T("TR2");       break;
        case __RESMGR_TR2GOLD__:    cstrName = _T("TR2Gold");   break;
        case __RESMGR_TR3__:        cstrName = _T("TR3");       break;
        case __RESMGR_TR3GOLD__:    cstrName = _T("TR3Gold");   break;
        case __RESMGR_TR4__:        cstrName = _T("TR4");       break;
		default:					cstrName = _T("TR???");		break;
        }

	return cstrName;
}


BYTE	CResMgr::bGetResIdFromTRAbrev( const CString& cstrTRAbrev )
{
    BYTE	bId = __RESMGR_TRUNK__;

	if( cstrTRAbrev.CompareNoCase( _T("TR1") ) == 0 )			bId = __RESMGR_TR1__;
	else if( cstrTRAbrev.CompareNoCase( _T("TR1Gold") ) == 0 )	bId = __RESMGR_TR1GOLD__;
	else if( cstrTRAbrev.CompareNoCase( _T("TR2") ) == 0 )		bId = __RESMGR_TR2__;
	else if( cstrTRAbrev.CompareNoCase( _T("TR2Gold") ) == 0 )	bId = __RESMGR_TR2GOLD__;
	else if( cstrTRAbrev.CompareNoCase( _T("TR3") ) == 0 )		bId = __RESMGR_TR3__;
	else if( cstrTRAbrev.CompareNoCase( _T("TR3Gold") ) == 0 )	bId = __RESMGR_TR3GOLD__;
	else if( cstrTRAbrev.CompareNoCase( _T("TR4") ) == 0 )		bId = __RESMGR_TR4__;

	return bId;
}


void    CResMgr::vSetDataPath(BYTE bTRResId, CString& cstrDataPath)
{
    m_DataPath[bTRResId] = cstrDataPath;

    CString cstrName = cstrGetTRAbrevFromResId( bTRResId );

    cstrName += _T(" Data Path");

    AfxGetApp()->WriteProfileString(_T("Settings"),cstrName,m_DataPath[bTRResId]);
}


void    CResMgr::vParseFile_MenuResource(CMenu& Menu,TCHAR* lpszFilename, enumTRType trType, ListResItem& List)
{
    FILE*   hFile = _tfopen(lpszFilename,_T("rt"));

    if(hFile ==  NULL) return;

    bool    boSectionFound = false;
    DWORD   dwMenuId = 0;

    CMenu*  lpStack[10];
    DWORD   dwStackIndex = 1;

    ZeroMemory(lpStack,sizeof(CMenu*) * 10);

    lpStack[0] = &Menu;

    while(!feof(hFile))
        {
        TCHAR   szBuf[1025];

        _fgetts(szBuf,1024,hFile);

        if(!boSectionFound)
            {
            _tcsupr(szBuf);
            switch(trType)
                {
                case TR1_FILE:
                    {
                    if(_tcsncmp(szBuf,_T("[TR1]"),5) == 0) boSectionFound = true;
                    break;
                    }
                case TR2_FILE:
                    {
                    if(_tcsncmp(szBuf,_T("[TR2]"),5) == 0) boSectionFound = true;
                    break;
                    }
                case TR3_FILE:
                    {
                    if(_tcsncmp(szBuf,_T("[TR3]"),5) == 0) boSectionFound = true;
                    break;
                    }
                case TR4_FILE:
                    {
                    if(_tcsncmp(szBuf,_T("[TR4]"),5) == 0) boSectionFound = true;
                    break;
                    }
                }
            }
        else
            {
            CSplit Split(szBuf,":",TRUE);

            if(Split.lUBound() == 0)
                {
                if(strnicmp(Split[0],"[END]",5) == 0) break;

                // Menu delimiter
                char*   lpsz1 = strchr(szBuf,'[');
                char*   lpsz2 = NULL;

                if(lpsz1) lpsz2 = strchr(lpsz1 + 1,']');

                if(lpsz1 && lpsz2)
                    {
                    *lpsz2 = 0;
                
                    dwStackIndex = lpsz1 - szBuf;

                    if(lpStack[dwStackIndex])
                        {
                        lpStack[dwStackIndex]->Detach();
                        delete lpStack[dwStackIndex];
                        }

                    lpStack[dwStackIndex] = new CMenu;
                    lpStack[dwStackIndex]->CreatePopupMenu();

                    lpStack[dwStackIndex-1]->AppendMenu(MF_POPUP,(UINT)lpStack[dwStackIndex]->m_hMenu,lpsz1 + 1);
                    }
                }
            else
                {
                // Menu item
                char* lpsz1 = Split[0];

                while(*lpsz1 == 32  ||  *lpsz1 == 9) lpsz1++;

                lpStack[dwStackIndex]->AppendMenu(MF_STRING, dwMenuId++, lpsz1);

                CResItem*   lpResItem = new CResItem(lpsz1,Split[1]);

                CSplit Split2(Split[2],";",TRUE);

                for(long dwI = 0; dwI < Split2.lUBound(); dwI++)
                    {
                    enumTrObjType   ObjType = TrObj_NoObj;
                    DWORD           dwObjId = atol(&(Split2[dwI][4]));

                    if(strnicmp(Split2[dwI],"mvb",3) == 0) ObjType = TrObj_Type_Moveable;
                    else if(strnicmp(Split2[dwI],"spr",3) == 0) ObjType = TrObj_Type_Sprite;
                    else if(strnicmp(Split2[dwI],"msh",3) == 0) ObjType = TrObj_Type_StaticMesh;

                    lpResItem->vAddObject(ObjType,dwObjId);
                    }

                List.push_back(lpResItem);
                }
            }
        }

    for(DWORD dwI = 1; dwI < 10; dwI++)
        {
        if(lpStack[dwI])
            {
            lpStack[dwI]->Detach();
            delete lpStack[dwI];
            }
        }

    fclose(hFile);
}


void    CResMgr::vParseFile_SFX(CString& cstrFilename, ListSfx& ListSFX)
{
    FILE*   hFile = _tfopen((LPTSTR)(LPCTSTR)cstrFilename,_T("rt"));

    if(hFile ==  NULL) return;

    long        lSoundIndex = -1;
    CResSFX*    lpResSFX = NULL;
    DWORD       dwFirstSoundIndex = 0;

    while(!feof(hFile))
        {
        TCHAR   szBuf[1025];

        _fgetts(szBuf,1024,hFile);

        CSplit Split(szBuf,"--",TRUE);

        if(Split.lUBound() > 0)
            {
            TCHAR   sz[5];

            memcpy(sz, Split[0], 4); sz[4] = 0;

            DWORD dwInternalIndex = _ttol(sz);

            if(dwInternalIndex != lSoundIndex)
                {
                CString s(&Split[1][1]);
                s = s.Left(s.GetLength() - 1);
                lpResSFX = new CResSFX((LPTSTR)(LPCTSTR)s, 1, dwFirstSoundIndex);
                for(long lI = lSoundIndex + 1; lI < (long)dwInternalIndex; lI++)
                    {
                    s.Format(_T("! Not used (id #%.3d) !"),lI);
                    ListSFX.push_back(new CResSFX((LPTSTR)(LPCTSTR)s,0,0));
                    }
                ListSFX.push_back(lpResSFX);
                lSoundIndex = dwInternalIndex;
                dwFirstSoundIndex++;
                }
            else
                {
                if(lpResSFX) lpResSFX->m_dwNumSamples++;
                dwFirstSoundIndex++;
                }
            }
        }

    fclose(hFile);
}


CString CResMgr::cstrGetSfxName(BYTE bTRResId, DWORD dwInternalIndex) const
{
    if(dwInternalIndex >= m_ListSFX[bTRResId].size())
        {
        CString s;
        s.Format(_T("! Unknown internal index #%.3d !"),dwInternalIndex);
        return CString(s);
        }

    return m_ListSFX[bTRResId][dwInternalIndex]->m_cstrName;
}


DWORD   CResMgr::dwGetSfxNumSamples(BYTE bTRResId, DWORD dwInternalIndex) const
{
    if(dwInternalIndex >= m_ListSFX[bTRResId].size()) return 0;

    return m_ListSFX[bTRResId][dwInternalIndex]->m_dwNumSamples;
}


DWORD   CResMgr::dwGetSfxFirstSoundIndex(BYTE bTRResId, DWORD dwInternalIndex) const
{
    if(dwInternalIndex >= m_ListSFX[bTRResId].size()) return 0;

    return m_ListSFX[bTRResId][dwInternalIndex]->m_dwFirstSoundIndex;
}


BYTE    CResMgr::bGetResId(enumTRType trType) const
{
    switch(trType)
        {
        case TR1_FILE:  return __RESMGR_TR1__;
        case TR2_FILE:  return __RESMGR_TR2__;
        case TR3_FILE:  return __RESMGR_TR3__;
        case TR4_FILE:  return __RESMGR_TR4__;
        }

    return __RESMGR_TR2__;
}


void	CResMgr::vCreateIconsImage()
{
	// Load the icons
	CBitmap bm_;

    CString cstrImage( m_szPath );

    cstrImage += _T("TreeviewIcons_.bmp");

	HBITMAP hBitmap = NULL;// (HBITMAP) LoadImage( NULL, cstrImage, IMAGE_BITMAP, 0, 0, LR_LOADFROMFILE | LR_CREATEDIBSECTION ); 
	
	if( hBitmap )
		{
		bm_.Attach( hBitmap );
		}
	else
		{
		bm_.LoadBitmap( IDB_TREEVIEWICONS );
		}

	BITMAP	bmInfo;
	
	bm_.GetBitmap( &bmInfo );

	// Build an image map with all the icons on a single row
	DWORD	dwNumRows = bmInfo.bmHeight / 16;
	DWORD	dwNumIconsPerRow = bmInfo.bmWidth / 16;
	DWORD	dwBytesPerLine = bmInfo.bmWidthBytes;
	DWORD	dwNewBytesPerLine = bmInfo.bmWidthBytes * dwNumRows;

	BYTE*	lpbSrc = new BYTE [ dwNewBytesPerLine * 16 ];

	bm_.GetBitmapBits( dwNewBytesPerLine * 16, lpbSrc );

	BYTE*	lpbBits = new BYTE [ dwNewBytesPerLine * 16 ];

	for( DWORD dwRow = 0; dwRow < dwNumRows; ++dwRow )
		{
		for( DWORD dwLine = 0; dwLine < 16; ++dwLine )
			{
			memcpy( lpbBits + dwLine * dwNewBytesPerLine + dwRow * dwBytesPerLine, lpbSrc + ( dwRow * 16 + dwLine ) * dwBytesPerLine, dwBytesPerLine );
			}
		}

	delete [] lpbSrc;

	// Create the bitmap and set the image list
	m_lpIconsImage = new CBitmap;

	m_lpIconsImage->CreateBitmap( dwNumRows * dwNumIconsPerRow * 16, 16, bmInfo.bmPlanes, bmInfo.bmBitsPixel, lpbBits );

	delete [] lpbBits;
}
