                      #include "stdafx.h"

#include "Resource.h"
#include "TRViewer.h"
#include "Mainfrm.h"
#include "DlgLevelLoad.h"
#include "DlgLevelSave.h"
#include "TextureTools.h"
#include "TGAFile.h"

const int __OBJID_TR3_SKY__ = 355;


//----------------------------------------------------------------------------
//
//  Function:   boLoad
//
//  Synopsis:   Load a TR level, then parse it and populate the m_trFile
//              data member
//
//  Arguments:  [lpszFileName] -- file name of the level
//
//  Return:     true if the level could be loaded / parsed, else false
//
//----------------------------------------------------------------------------
bool    CTRFile::boLoad(TCHAR* lpszFileName, bool boDontConvert, bool boShowProgress)
{
    HANDLE  hSrcFile = INVALID_HANDLE_VALUE;
    bool    boOk = false;
    BYTE*   lpData = NULL;
    DWORD   dwDataSize = 0;
    
    CString s;
    lstrcpy(m_szFileName,lpszFileName);

    _splitpath(m_szFileName,NULL,NULL,m_szLevelName,NULL);

    // Pump messages to let the toolbox window disapear
    MSG Msg;

    while(PeekMessage(&Msg,NULL,0,0,TRUE))
        {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
        }

try
    {
    // Open the file
    hSrcFile = CreateFile(lpszFileName,GENERIC_READ,0,NULL,OPEN_EXISTING,FILE_ATTRIBUTE_NORMAL,NULL);

    if(hSrcFile  ==  INVALID_HANDLE_VALUE)
        {
        if (boShowProgress)
			{
			s.Format(_T("File %s not found.\n"),lpszFileName);
			AfxMessageBox(s,MB_ICONSTOP | MB_OK | MB_TOPMOST);
			}
        throw((DWORD)1);
        }

    // Get the file size
    dwDataSize = GetFileSize(hSrcFile,NULL);

    if(dwDataSize  ==  0xFFFFFFFF)
        {
        dwDataSize = 0;
        if (boShowProgress)
			{
			s.Format(_T("Can't get the size of the file (error %ld).\n"),GetLastError());
			AfxMessageBox(s,MB_ICONSTOP | MB_OK | MB_TOPMOST);
			}
        throw((DWORD)2);
        }

    // Allocate the memory and load the data
    DWORD   dwNumRead;

    lpData = new BYTE[dwDataSize];

    ReadFile(hSrcFile,lpData,dwDataSize,&dwNumRead,NULL);

    if(dwNumRead  !=  dwDataSize)
        {
        if (boShowProgress)
			{
			s.Format(_T("Can't read the data (error %ld).\n"),GetLastError());
			AfxMessageBox(s,MB_ICONSTOP | MB_OK | MB_TOPMOST);
			}
        throw((DWORD)3);
        }

    boOk = true;
    throw((DWORD)0);
    }
catch( DWORD )
    {
    if(hSrcFile != INVALID_HANDLE_VALUE)
		CloseHandle(hSrcFile);
    }

    if(boOk)
        {
        CWaitCursor Dummy;

        // Parse the level
        boOk = boParseTRLevel(lpData,dwDataSize,lpszFileName, boDontConvert, boShowProgress);
        if(!boOk)
            {
            vRelease();
			if (boShowProgress)
				{
				AfxMessageBox(_T("Unknown file type !"),MB_ICONSTOP | MB_OK | MB_TOPMOST);
				}
            }
        }

    delete [] lpData;

    return boOk;
}


//----------------------------------------------------------------------------
//
//  Function:   boSave
//
//  Synopsis:   Save the current TR level
//
//  Arguments:  [lpszFileName] -- file name of the level
//
//  Return:     true if the level could be saved, else false
//
//----------------------------------------------------------------------------
bool    CTRFile::boSave(LPCTSTR lpszFileName, int DontAsk, bool boDontConvert, bool boShowProgress)
{
    if(m_TRType == TR_UNKNOWN) return false;

    HANDLE  hSrcFile = INVALID_HANDLE_VALUE;
    bool    boOk = false;
    BYTE*   lpData = NULL;
    DWORD   dwDataSize = 0;

    CString s;

    // Pump messages to let the toolbox window disapear
    MSG Msg;

    while(PeekMessage(&Msg,NULL,0,0,TRUE))
        {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
        }

try
    {
    // Open the file
    hSrcFile = CreateFile(lpszFileName,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_NORMAL,NULL);

    if(hSrcFile  ==  INVALID_HANDLE_VALUE)
        {
        if (boShowProgress)
			{
			s.Format(_T("Can't create the file %s (err 0x%0.8X).\n"),lpszFileName, GetLastError());
			AfxMessageBox(s,MB_ICONSTOP | MB_OK | MB_TOPMOST);
			}
        throw((DWORD)1);
        }

    // Get the data
    if(m_TRType == TR4_FILE)
        {
        CWaitCursor Dummy;

        lpData = lpbMakeTR4Level(&dwDataSize,!boDontConvert,DontAsk,boShowProgress);
        }
    else
        {
        CWaitCursor Dummy;

        lpData = lpbMakeTR123Level(&dwDataSize,!boDontConvert,DontAsk,boShowProgress);
        }

    // Write the data
    DWORD   dwNbWritten;

    WriteFile(hSrcFile,lpData,dwDataSize,&dwNbWritten,NULL);

    if(dwNbWritten  !=  dwDataSize)
        {
        if (boShowProgress)
			{
			s.Format(_T("Can't write the data (error %ld).\n"),GetLastError());
			AfxMessageBox(s,MB_ICONSTOP | MB_OK | MB_TOPMOST);
			}
        throw((DWORD)3);
        }

    boOk = true;
    throw((DWORD)0);
    }
catch( DWORD )
    {
    if(hSrcFile != INVALID_HANDLE_VALUE) CloseHandle(hSrcFile);
    }

    if(boOk)
        {
        lstrcpy(m_szFileName,lpszFileName);
        }

    delete [] lpData;

    return boOk;
}


//----------------------------------------------------------------------------
//
//  Function:   vRelease
//
//  Synopsis:   Release the memory of a level
//
//  Arguments:  -
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTRFile::vRelease()
{
    TR123_FILE_STRUCT tr123File = m_trFile.tr123File;

    if(tr123File.lplpTextiles16)
        {
        for(DWORD dwI = 0; dwI < tr123File.dwNumTextiles; dwI++)
            {
            delete tr123File.lplpTextiles16[dwI];
            }
        }

    delete [] tr123File.lplpTextiles16;

    for(DWORD dwI = 0; dwI < tr123File.wNumRooms; dwI++)
        {
        TR123_ROOM* lpRoom = &tr123File.lpRooms[dwI];

        delete [] lpRoom->RoomData.lpVertices;
        delete [] lpRoom->RoomData.lpRectangles;
        delete [] lpRoom->RoomData.lpTriangles;
        delete [] lpRoom->RoomData.lpSprites;
        delete [] lpRoom->lpPortals;
        delete [] lpRoom->lpSectorList;
        delete [] lpRoom->lpLights;
        delete [] lpRoom->lpStaticMeshes;
        }

    delete [] tr123File.lpRooms;
    delete [] tr123File.lpFloorData;

    for(dwI = 0; dwI < tr123File.dwNumMeshes; dwI++)
        {
        TR123_MESH* lpMesh = tr123File.lplpMeshes[dwI];

		vReleaseMeshMemory(lpMesh);

        delete lpMesh;
        }

    for(dwI = 0; dwI < tr123File.dwNumSounds; dwI++)
        {
		delete [] tr123File.lpSounds[ dwI ].lpbData;
        }

	delete [] tr123File.lpSounds;

    TRListStaticMesh* lpList = m_trFile.tr123File.lpListStaticMesh;

    if(lpList)
        for(TRListStaticMesh::iterator theIterator = lpList->begin(); theIterator != lpList->end(); theIterator++)
            {
            CTrStatMsh* lpStatMsh = *theIterator;

            delete lpStatMsh;
            }

    delete m_trFile.tr123File.lpListStaticMesh;

    TRListSpriteSeq* lpList2 = m_trFile.tr123File.lpListSpriteSeq;

    if(lpList2)
        for(TRListSpriteSeq::iterator theIterator2 = lpList2->begin(); theIterator2 != lpList2->end(); theIterator2++)
            {
            CTrLstSpr* lpSprSeq = *theIterator2;

            delete lpSprSeq;
            }

    delete m_trFile.tr123File.lpListSpriteSeq;

    TRListMoveable* lpList3 = m_trFile.tr123File.lpListMoveable;

    if(lpList3)
        for(TRListMoveable::iterator theIterator3 = lpList3->begin(); theIterator3 != lpList3->end(); theIterator3++)
            {
            CTrMvb* lpMoveable = *theIterator3;

            delete lpMoveable;
            }

    delete m_trFile.tr123File.lpListMoveable;

    TRListItem* lpList4 = m_trFile.tr123File.lpListItem;

    if(lpList4)
        for(TRListItem::iterator theIterator4 = lpList4->begin(); theIterator4 != lpList4->end(); theIterator4++)
            {
            TR123_ITEM* lpItem= *theIterator4;

            delete lpItem;
            }

    delete m_trFile.tr123File.lpListItem;

    delete [] tr123File.lplpMeshes;
    delete [] tr123File.lpMeshPointers;
    delete [] tr123File.lpAnimations;
    delete [] tr123File.lpStateChanges;
    delete [] tr123File.lpAnimDispatches;
    delete [] tr123File.lpAnimCommands;
    delete [] tr123File.lpMeshTrees;
    delete [] tr123File.lpFrames;
    delete [] tr123File.lpMoveables;
    delete [] tr123File.lpStaticMeshes;
    delete [] tr123File.lpSpriteSequences;
    delete [] tr123File.lpItems;
    delete [] tr123File.lpSpriteTextures;
    delete [] tr123File.lpAnimatedTextures;
    delete [] tr123File.lpObjectTextures;
    delete [] tr123File.lpSoundSources;
    delete [] tr123File.lpSoundDetails;
    delete [] tr123File.lpSampleIndices;
    delete [] tr123File.lpSamples;
    
    delete  tr123File.lpSndMgr;
    delete  tr123File.lpListUnusedObjText;

    delete [] tr123File.lpCameras;
    delete [] tr123File.lpBoxes;
    delete [] tr123File.lpOverlaps;
    delete [] tr123File.lpZones;
    delete [] tr123File.lpCinematicFrames;
    delete [] tr123File.lpDemoData;

    if(m_TRType == TR4_FILE)
        {
        delete [] tr123File.Texture1.lpbTextData;
        delete [] tr123File.Texture2.lpbTextData;
        delete [] tr123File.Texture3.lpbTextData;
        delete [] tr123File.lpFlybyCameras;
		delete [] tr123File.lpRoomObjText;
        }

    ZeroMemory(&m_trFile,sizeof(TR_FILE));

    m_TRType = TR_UNKNOWN;
    m_szFileName[0] = 0;
}


//----------------------------------------------------------------------------
//
//  Function:   dwCalcGeometryDataSize
//
//  Synopsis:   Calculate the size of the geometry part of a TR1/2/3/4 level
//
//  Arguments:  -
//
//  Return:     Data size of the level
//
//----------------------------------------------------------------------------
DWORD   CTRFile::dwCalcGeometryDataSize()
{
    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;
    bool    boTR1 = (m_TRType == TR1_FILE);
    bool    boTR4 = (m_TRType == TR4_FILE);

	DWORD	dwOfsCinem = boTR4 ? 0 : 8;
	DWORD	dwOfsAnim = boTR4 ? 0 : 8;
	DWORD	dwOfsObjText = boTR4 ? 0 : 18;
	
	DWORD	dwSize = 0;

	dwSize +=
        sizeof(bitu32)                                          // unused
      + sizeof(bitu16);                                         // Num rooms

    for(DWORD dwI = 0; dwI < lp->wNumRooms; dwI++)
        {
        dwSize += dwCalcRoomDataSize(&lp->lpRooms[dwI],m_TRType);
        }

    dwSize += 
        sizeof(bitu32)                                          // Num floor data
      + sizeof(bitu16) * lp->dwNumFloorData                     // Floor data
      + sizeof(bitu32);                                         // Num mesh datas

    DWORD dwStartingMeshOffset = dwSize;

    for(dwI = 0; dwI < lp->dwNumMeshes; dwI++)
        {
        if( boTR4 ) dwSize += dwCalcTR4MeshDataSize(lp->lplpMeshes[dwI],dwSize - dwStartingMeshOffset); else dwSize += dwCalcMeshDataSize(lp->lplpMeshes[dwI],dwSize - dwStartingMeshOffset);
        }

    dwSize +=
        sizeof(bitu32)                                          // Num mesh pointers
      + sizeof(bitu32) * lp->dwNumMeshPointers                  // Mesh pointers
      + sizeof(bitu32)                                          // Num animations
      + (sizeof(TR2_ANIMATION) - dwOfsAnim) * lp->dwNumAnimations             // Animations
      + sizeof(bitu32)                                          // Num state changes
      + sizeof(TR2_STATE_CHANGE) * lp->dwNumStateChanges        // State changes
      + sizeof(bitu32)                                          // Num anim dispatches
      + sizeof(TR2_ANIM_DISPATCH) * lp->dwNumAnimDispatches     // Anim dispatches
      + sizeof(bitu32)                                          // Num anim commands
      + sizeof(TR2_ANIM_COMMAND) * lp->dwNumAnimCommands        // Anim commands
      + sizeof(bitu32)                                          // Num mesh trees
      + sizeof(TR2_MESHTREE) * lp->dwNumMeshTrees               // Mesh trees
      + sizeof(bitu32)                                          // Num frames
      + sizeof(bitu16) * lp->dwNumFrames                        // Frames
      + sizeof(bitu32)                                          // Num moveables
      + sizeof(TR2_MOVEABLE) * lp->dwNumMoveables;              // Moveables

	dwSize +=
        sizeof(bitu32)                                          // Num static meshes
      + sizeof(TR2_STATICMESH) * lp->dwNumStaticMeshes          // Static meshes
      + sizeof(bitu32)                                          // Num sprite sequences
      + sizeof(TR2_SPRITE_SEQUENCE) * lp->dwNumSpriteSequences  // Sprite sequences
      + sizeof(bitu32)                                          // Num items
      + lp->dwNumItems * (boTR1 ? sizeof(TR1_ITEM) : sizeof(TR2_ITEM))  // Items
      + sizeof(bitu32)                                          // Num sprite textures
      + sizeof(TR2_SPRITE_TEXTURE) * lp->dwNumSpriteTextures    // Sprite textures
      + sizeof(bitu32)                                          // Num animated textures
      + sizeof(bitu16) * lp->dwNumAnimatedTextures              // Animated textures
      + sizeof(bitu32)                                          // Num object textures
      + (sizeof(TR2_OBJECT_TEXTURE) - dwOfsObjText) * lp->dwNumObjectTextures;    // Object textures

	dwSize +=
        sizeof(bitu32)                                          // Num sound sources
      + sizeof(TR2_SOUND_SOURCE) * lp->dwNumSoundSources        // Sound sources
      + (boTR1 ? sizeof(TR1_SOUNDMAP) : sizeof(TR2_SOUNDMAP))   // Sound map
      + sizeof(bitu32)                                          // Num sound details
      + sizeof(TR2_SOUND_DETAILS) * lp->dwNumSoundDetails       // Sound details
      + (boTR1 ? sizeof(bitu32) : 0)                            // Num samples
      + (boTR1 ? sizeof(bitu8) * lp->dwNumSamples : 0)          // Samples
      + sizeof(bitu32)                                          // Num sample indices
      + sizeof(bitu32) * lp->dwNumSampleIndices;                // Sample indices

	dwSize +=
        sizeof(bitu32)                                          // Num cameras
      + sizeof(TR2_CAMERA) * lp->dwNumCameras                   // Cameras
      + sizeof(bitu32)                                          // Num boxes
      + lp->dwNumBoxes * (boTR1 ? sizeof(TR1_BOX) : sizeof(TR2_BOX))    // Boxes
      + sizeof(bitu32)                                          // Num overlaps
      + sizeof(bitu16) * lp->dwNumOverlaps                      // Overlaps
      + lp->dwNumBoxes * (boTR1 ? sizeof(TR1_ZONE) : sizeof(TR2_ZONE))  // Zones
      + (boTR4 ? 0 : sizeof(TR2_LIGHTMAP))                      // Lightmap
      + (boTR4 ? sizeof(bitu32) : sizeof(bitu16))               // Num cinematic frames
      + (sizeof(TR2_CINEMATIC_FRAME) - dwOfsCinem) * lp->dwNumCinematicFrames   // Cinematic frames
      + sizeof(bitu16)                                          // Num demo data
      + sizeof(bitu8) * lp->wNumDemoData;                       // Demo data

	if( boTR4 )
		{
		dwSize +=
			  3													// 'SPR'
			+ 4													// '\0TEX'
			+ sizeof(bitu32)									// Num flyby cameras
			+ sizeof(TR4_FLYBY_CAMERA) * lp->dwNumFlybyCameras	// Flyby Cameras
			+ 6;												// unused
		}

    return dwSize;
}


//----------------------------------------------------------------------------
//
//  Function:   dwCalcLevelTR123DataSize
//
//  Synopsis:   Calculate the data size of a whole level for TR1/2 or 3
//
//  Arguments:  -
//
//  Return:     Data size of the level
//
//----------------------------------------------------------------------------
DWORD   CTRFile::dwCalcLevelTR123DataSize()
{
    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;
    bool    boTR1 = (m_TRType == TR1_FILE);

	DWORD	dwSize = 0;

	dwSize +=
		sizeof(bitu32)                                          // Version
	  + sizeof(TR2_PALETTE8)                                    // Palette 8 bits
	  + (boTR1 ? 0 : sizeof(TR2_PALETTE16))                     // Palette 16 bits
	  + sizeof(bitu32)                                          // Num textiles
	  + lp->dwNumTextiles * sizeof(TR2_TEXTILE8)                // Textiles 8 bits
	  + (boTR1 ? 0 : lp->dwNumTextiles * sizeof(TR2_TEXTILE16));// Textiles 16 bits

	dwSize += dwCalcGeometryDataSize();

	return dwSize;
}


//----------------------------------------------------------------------------
//
//  Function:   dwCalcLevelTR4DataSize
//
//  Synopsis:   Calculate the data size of a whole level for TR4
//
//  Arguments:  -
//
//  Return:     Data size of the level
//
//----------------------------------------------------------------------------
DWORD   CTRFile::dwCalcLevelTR4DataSize( bool boIncludeGeometry /*= true*/ )
{
    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;

	DWORD	dwSize = 0;

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();

	// If we must decompress sounds, we assume the uncompressed data size of a sound to be 4 times the compressed data size
	// -> this is a maximum (ADPCM), generally decompressed data will be less than 4 * compressed data
	DWORD dwMultSounds = lpApp->boSaveUncompressedSounds() ? 4 : 1;

	dwSize +=
		sizeof(bitu32)                                          // Version
      + sizeof(bitu16)                                          // Num textiles no bump
      + sizeof(bitu16)                                          // Num textiles inhabitant
      + sizeof(bitu16)                                          // Num textiles bump
      + sizeof(bitu32)                                          // Texture 1
      + sizeof(bitu32)
      + sizeof(bitu8) * lp->Texture1.dwTextCompSize
      + sizeof(bitu32)                                          // Texture 2
      + sizeof(bitu32)
      + sizeof(bitu8) * lp->Texture2.dwTextCompSize
      + sizeof(bitu32)                                          // Texture 3
      + sizeof(bitu32)
      + sizeof(bitu8) * lp->Texture3.dwTextCompSize
      + sizeof(bitu32)                                          // Geometry data uncompressed size
      + sizeof(bitu32)											// Geometry data comppressed size
      + sizeof(bitu32);											// num sounds

	for( DWORD dwSound = 0; dwSound < lp->dwNumSounds; ++dwSound )
		{
		DWORD	dwFactor = 1;

		// Take into account the MultSounds factor only if the sounds is compressed
		if( lp->lpSounds[ dwSound ].dwSoundLen != lp->lpSounds[ dwSound ].dwUncompSoundLen )
			{
			dwFactor = dwMultSounds;
			}

		dwSize +=
			sizeof( bitu32 ) +								// unknown
			sizeof( bitu32 ) +								// sound len
			sizeof( bitu8 ) * lp->lpSounds[ dwSound ].dwSoundLen * dwFactor;	// sound data size
		}

	if( boIncludeGeometry )	dwSize += dwCalcGeometryDataSize();

	return dwSize;
}


//----------------------------------------------------------------------------
//
//  Function:   vMakeGeometry
//
//  Synopsis:   Write (into memory) the geometry of a TR1/2/3/4 level
//
//  Arguments:  -
//
//  Return:     -
//
//----------------------------------------------------------------------------
void	CTRFile::vMakeGeometry(BYTE* lpbData,CDlgLevelSave* DlgSave, DWORD dwOfsProgress /*= 0*/)
{
    DWORD   dwI,dwSize;
    BYTE*   lpData = lpbData;

    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;

    bool boTR1 = (m_TRType == TR1_FILE);

    * ((bitu32*) lpData) = 0; lpData += sizeof(bitu32);   // unused

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write rooms data
    //
    * ((bitu16*) lpData) = lp->wNumRooms; lpData += sizeof(bitu16);
    for(dwI = 0; dwI < lp->wNumRooms; dwI++)
        {
        TR123_ROOM*    lpRoom = &lp->lpRooms[dwI];

        * ((TR2_ROOM_INFO*) lpData) = lpRoom->RoomInfo; lpData += sizeof(TR2_ROOM_INFO);

        bitu32* lpdwNumData = (bitu32*) lpData; lpData += sizeof(bitu32);

        TR2_ROOM_DATA* lpRoomData = (TR2_ROOM_DATA*) &lpRoom->RoomData;

        WRITE_ARRAYNUM_TR(bitu16,lpRoomData->wNumVertices,lpRoomData->lpVertices,TR1_VERTEX_ROOM,TR2_VERTEX_ROOM)
        WRITE_ARRAYNUM2(bitu16,TR2_FACE4,lpRoomData->wNumRectangles,lpRoomData->lpRectangles,sizeof(bitu16))
        WRITE_ARRAYNUM2(bitu16,TR2_FACE3,lpRoomData->wNumTriangles,lpRoomData->lpTriangles,sizeof(bitu16))
        WRITE_ARRAYNUM(bitu16,TR2_ROOM_SPRITE,lpRoomData->wNumSprites,lpRoomData->lpSprites)
        *lpdwNumData = (lpData - ((BYTE*)lpdwNumData + sizeof(bitu32))) / 2;
        WRITE_ARRAYNUM(bitu16,TR2_ROOM_PORTAL,lpRoom->wNumPortals,lpRoom->lpPortals)

        * ((bitu16*) lpData) = lpRoom->wNumZSectors; lpData += sizeof(bitu16);
        * ((bitu16*) lpData) = lpRoom->wNumXSectors; lpData += sizeof(bitu16);
        WRITE_ARRAY(lpRoom->wNumZSectors * lpRoom->wNumXSectors,TR2_ROOM_SECTOR,lpRoom->lpSectorList)

        * ((bit16*) lpData) = lpRoom->sAmbientIntensity1; lpData += sizeof(bit16);
        if(m_TRType != TR1_FILE)                                                            
            {
            * ((bit16*) lpData) = lpRoom->sAmbientIntensity2; lpData += sizeof(bit16);
            }
        if(m_TRType == TR2_FILE)                                                            
            {
            * ((bit16*) lpData) = lpRoom->sLightMode; lpData += sizeof(bit16);
            }

        WRITE_ARRAYNUM_TR2(bitu16,lpRoom->wNumLights,lpRoom->lpLights,TR1_ROOM_LIGHT,TR2_ROOM_LIGHT,TR4_ROOM_LIGHT)
        WRITE_ARRAYNUM_TR(bitu16,lpRoom->wNumStaticMeshes,lpRoom->lpStaticMeshes,TR1_ROOM_STATICMESH,TR2_ROOM_STATICMESH)

        * ((bit16*) lpData) = lpRoom->sAlternateRoom; lpData += sizeof(bit16);
        * ((bit16*) lpData) = lpRoom->sFlags; lpData += sizeof(bit16);
        if(m_TRType >= TR3_FILE)
            {
            * ((TR2_COLOR*) lpData) = lpRoom->RoomLightColor; lpData += sizeof(TR2_COLOR);
            }
        }

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write behaviours data
    //

    WRITE_ARRAYNUM(bitu32,bitu16,lp->dwNumFloorData,lp->lpFloorData)

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write Meshes geometry data
    //
    bitu32* lpdwNumMeshData = (bitu32*) lpData; lpData += sizeof(bitu32);
    BYTE* lpbAddrStartMesh = lpData;

    bitu32* lpLstAddr = new bitu32 [lp->dwNumMeshPointers];

    for(dwI = 0; dwI < lp->dwNumMeshes; dwI++)
        {
        lpLstAddr[dwI] = (bitu32) (lpData - lpbAddrStartMesh);
        if(m_TRType == TR4_FILE) lpData = lpbWriteTR4Mesh(lp->lplpMeshes[dwI],lpData,lpbAddrStartMesh); else lpData = lpbWriteMesh(lp->lplpMeshes[dwI],lpData,lpbAddrStartMesh);
        }

    *lpdwNumMeshData = (lpData - lpbAddrStartMesh) / 2;

    bitu32* lpMeshPointers = new bitu32 [lp->dwNumMeshPointers];
    memcpy(lpMeshPointers,lp->lpMeshPointers,lp->dwNumMeshPointers * sizeof(bitu32));

    for(dwI = 0; dwI < lp->dwNumMeshPointers; dwI++)
        {
        lpMeshPointers[dwI] = lpLstAddr[lpMeshPointers[dwI]];
        }

    WRITE_ARRAYNUM(bitu32,bitu32,lp->dwNumMeshPointers,lpMeshPointers)

    delete [] lpMeshPointers;
    delete [] lpLstAddr;

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write meshes animations data
    //

	if( m_TRType == TR4_FILE )
		{
		WRITE_ARRAYNUM(bitu32,TR2_ANIMATION,lp->dwNumAnimations,lp->lpAnimations)
		}
	else
		{
		* ((bitu32*) lpData) = lp->dwNumAnimations; lpData += sizeof(bitu32);
		for( DWORD dwI = 0; dwI < lp->dwNumAnimations; ++dwI )
			{
			memcpy( lpData, &lp->lpAnimations[ dwI ], 16 ); lpData += 16;
			memcpy( lpData, &lp->lpAnimations[ dwI ].wFrameStart, 16 ); lpData += 16;
			}
		}

    WRITE_ARRAYNUM(bitu32,TR2_STATE_CHANGE,lp->dwNumStateChanges,lp->lpStateChanges)
    WRITE_ARRAYNUM(bitu32,TR2_ANIM_DISPATCH,lp->dwNumAnimDispatches,lp->lpAnimDispatches)
    WRITE_ARRAYNUM(bitu32,TR2_ANIM_COMMAND,lp->dwNumAnimCommands,lp->lpAnimCommands)
    WRITE_ARRAYNUM(bitu32,TR2_MESHTREE,lp->dwNumMeshTrees,lp->lpMeshTrees)
    WRITE_ARRAYNUM(bitu32,bitu16,lp->dwNumFrames,lp->lpFrames)

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write meshes lists data
    //
    WRITE_ARRAYNUM(bitu32,TR2_MOVEABLE,lp->dwNumMoveables,lp->lpMoveables)
    WRITE_ARRAYNUM(bitu32,TR2_STATICMESH,lp->dwNumStaticMeshes,lp->lpStaticMeshes)

    if(m_TRType <= TR2_FILE)
        {
		* ((bitu32*) lpData) = lp->dwNumObjectTextures; lpData += sizeof(bitu32);
		for( DWORD dwI = 0; dwI < lp->dwNumObjectTextures; ++dwI )
			{
			memcpy( lpData, &lp->lpObjectTextures[ dwI ], 4 ); lpData += 4;
			memcpy( lpData, &lp->lpObjectTextures[ dwI ].Vertices, 4 * sizeof(TR2_OBJECT_TEXTURE_VERT) ); lpData += 4 * sizeof(TR2_OBJECT_TEXTURE_VERT);
			}
        }

    if(m_TRType == TR4_FILE)
		{
		*lpData++ = 'S';
		*lpData++ = 'P';
		*lpData++ = 'R';
		}

    WRITE_ARRAYNUM(bitu32,TR2_SPRITE_TEXTURE,lp->dwNumSpriteTextures,lp->lpSpriteTextures)
    WRITE_ARRAYNUM(bitu32,TR2_SPRITE_SEQUENCE,lp->dwNumSpriteSequences,lp->lpSpriteSequences)

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write other & sound data
    //
    if(m_TRType == TR1_FILE  &&  m_boTUBFile)
        {
        * ((TR2_PALETTE8*) lpData) = lp->Palette8;  lpData += sizeof(TR2_PALETTE8);
        }

    WRITE_ARRAYNUM(bitu32,TR2_CAMERA,lp->dwNumCameras,lp->lpCameras)

	if( m_TRType == TR4_FILE )
		{
		WRITE_ARRAYNUM(bitu32,TR4_FLYBY_CAMERA,lp->dwNumFlybyCameras,lp->lpFlybyCameras)
		}

    WRITE_ARRAYNUM(bitu32,TR2_SOUND_SOURCE,lp->dwNumSoundSources,lp->lpSoundSources)

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write NPCs data
    //
    WRITE_ARRAYNUM_TR(bitu32,lp->dwNumBoxes,lp->lpBoxes,TR1_BOX,TR2_BOX)
    WRITE_ARRAYNUM(bitu32,bitu16,lp->dwNumOverlaps,lp->lpOverlaps)
    WRITE_ARRAY_TR(lp->dwNumBoxes,lp->lpZones,TR1_ZONE,TR2_ZONE)

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write textures data
    //
    WRITE_ARRAYNUM(bitu32,bitu16,lp->dwNumAnimatedTextures,lp->lpAnimatedTextures)

    if(m_TRType >= TR3_FILE)
        {
		if( m_TRType == TR4_FILE )
			{
			*lpData++ = lp->bNumUVRotate;
			*lpData++ = 'T';
			*lpData++ = 'E';
			*lpData++ = 'X';
			}

		if( m_TRType == TR4_FILE )
			{
			WRITE_ARRAYNUM(bitu32,TR2_OBJECT_TEXTURE,lp->dwNumObjectTextures,lp->lpObjectTextures)
			}
		else
			{
			* ((bitu32*) lpData) = lp->dwNumObjectTextures; lpData += sizeof(bitu32);
			for( DWORD dwI = 0; dwI < lp->dwNumObjectTextures; ++dwI )
				{
				memcpy( lpData, &lp->lpObjectTextures[ dwI ], 4 ); lpData += 4;
				memcpy( lpData, &lp->lpObjectTextures[ dwI ].Vertices, 4 * sizeof(TR2_OBJECT_TEXTURE_VERT) ); lpData += 4 * sizeof(TR2_OBJECT_TEXTURE_VERT);
				}
			}
        }

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write other data
    //
    WRITE_ARRAYNUM_TR(bitu32,lp->dwNumItems,lp->lpItems,TR1_ITEM,TR2_ITEM)

	if( m_TRType != TR4_FILE )
		{
		* ((TR2_LIGHTMAP*) lpData) = lp->LightMap;  lpData += sizeof(TR2_LIGHTMAP);
		}

    if(m_TRType == TR1_FILE  &&  !m_boTUBFile)
        {
        * ((TR2_PALETTE8*) lpData) = lp->Palette8;  lpData += sizeof(TR2_PALETTE8);
        }

	if( m_TRType != TR4_FILE )
		{
		WRITE_ARRAYNUM2(bitu16,TR2_CINEMATIC_FRAME,lp->dwNumCinematicFrames,lp->lpCinematicFrames,8)
		}
	else
		{
		WRITE_ARRAYNUM(bitu32,TR2_CINEMATIC_FRAME,lp->dwNumCinematicFrames,lp->lpCinematicFrames)
		}

    WRITE_ARRAYNUM(bitu16,bitu8,lp->wNumDemoData,lp->lpDemoData)

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);

    //
    // Write sound data
    //
    if(m_TRType != TR1_FILE)
        {
        * ((TR2_SOUNDMAP*) lpData) = lp->SoundMap.tr2;
        lpData += sizeof(TR2_SOUNDMAP);
        }
    else
        {
        * ((TR1_SOUNDMAP*) lpData) = lp->SoundMap.tr1;
        lpData += sizeof(TR1_SOUNDMAP);
        }

    WRITE_ARRAYNUM(bitu32,TR2_SOUND_DETAILS,lp->dwNumSoundDetails,lp->lpSoundDetails)

    if(m_TRType == TR1_FILE)
        {
        WRITE_ARRAYNUM(bitu32,bitu8,lp->dwNumSamples,lp->lpSamples)
        }

    WRITE_ARRAYNUM(bitu32,bitu32,lp->dwNumSampleIndices,lp->lpSampleIndices)

	if( m_TRType == TR4_FILE )
		{
		ZeroMemory( lpData, 6 );
		lpData += 6;
		}

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData + dwOfsProgress);
}


//----------------------------------------------------------------------------
//
//  Function:   lpbMakeTR123Level
//
//  Synopsis:   Write (into memory) a level to the TR1/2/3 format
//
//  Arguments:  [lpdwDataSize] -- at return, hold the data size
//
//  Return:     pointer on data size.
//              This pointer needs to be freed after use (delete [] 'pointer')
//
//----------------------------------------------------------------------------
BYTE*   CTRFile::lpbMakeTR123Level(DWORD* lpdwDataSize, bool boConvert /*= true*/, int DontAsk /*=2 */, bool boShowProgress)
{
    CDlgLevelSave*   DlgSave=0;

	if (boShowProgress)
	{
		DlgSave = new CDlgLevelSave;
		DlgSave->vSetLevelName( m_szLevelName );
		DlgSave->Create(IDD_LEVEL_SAVE);
	}

    bool boKeepUnused = DontAsk != 2 ? (DontAsk==1) : AfxMessageBox( _T("Do you want to keep the unused textures in the level ?"), MB_YESNO | MB_TOPMOST) == IDYES;

    // Pump messages to let the dialog box disappear
    MSG Msg;

    while(PeekMessage(&Msg,NULL,0,0,TRUE))
        {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
        }

    if( boConvert ) vConvertToTRLevel(DlgSave, boKeepUnused);

    *lpdwDataSize = dwCalcLevelTR123DataSize();

    if (DlgSave) DlgSave->vSetSaveRange(*lpdwDataSize);

    BYTE*   lpbData = new BYTE [*lpdwDataSize];
    BYTE*   lpData = lpbData;

    ZeroMemory(lpbData,*lpdwDataSize);

    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;

    bool boTR1 = (m_TRType == TR1_FILE);

    //
    // Write version
    //
    * ((bitu32*) lpData) = dwGetVersionNumber(m_TRType); lpData += sizeof(bitu32);

    //
    // Write palettes data
    //
    if(!boTR1)
        {
        * ((TR2_PALETTE8*) lpData)  = lp->Palette8;     lpData += sizeof(TR2_PALETTE8);
        * ((TR2_PALETTE16*) lpData) = lp->Palette16;    lpData += sizeof(TR2_PALETTE16);
        }

    if (DlgSave) DlgSave->vSetSaveProgress(lpData - lpbData);

    //
    // Write textiles data
    //
	TR2_TEXTILE8    Textile8;

	memset(&Textile8, 255, sizeof(TR2_TEXTILE8));

	* ((bitu32*) lpData) = lp->dwNumTextiles; lpData += sizeof(bitu32);

	// Create the 8-bits textiles
	BYTE*   lpMap = new BYTE [32 * 32 * 32];

	ZeroMemory(lpMap,32 * 32 * 32);

	for(DWORD dwR = 0; dwR < 32; dwR++)
		for(DWORD dwG = 0; dwG < 32; dwG++)
			for(DWORD dwB = 0; dwB < 32; dwB++)
				lpMap[dwB + dwG * 32 + dwR * 32 * 32] = ::bFindColor((BYTE*)lp->Palette8.aRGB,(BYTE)(dwR << 3),(BYTE)(dwG << 3),(BYTE)(dwB << 3));

	for(DWORD dwI = 0; dwI < lp->dwNumTextiles; dwI++)
		{
		vConvert16BitsTo8Bits((BYTE*)lp->lplpTextiles16[dwI]->awTile,256,256,Textile8.abTile,lpMap);

		* ((TR2_TEXTILE8*) lpData) = Textile8; lpData += sizeof(TR2_TEXTILE8);
		}

	delete [] lpMap;

	// Save the 16-bits textiles
	if(!boTR1)
		{
		for(dwI = 0; dwI < lp->dwNumTextiles; dwI++)
			{
			* ((TR2_TEXTILE16*) lpData) = *lp->lplpTextiles16[dwI]; lpData += sizeof(TR2_TEXTILE16);
			}
		}

	vMakeGeometry(lpData, DlgSave, lpData - lpbData);

	vConvertFromTRLevel( DlgSave );

    if (DlgSave) DlgSave->DestroyWindow();

    delete DlgSave;

    return lpbData;
}


//----------------------------------------------------------------------------
//
//  Function:   lpbMakeTR4Level
//
//  Synopsis:   Write (into memory) a level to the TR4 format
//
//  Arguments:  [lpdwDataSize] -- at return, hold the data size
//
//  Return:     pointer on data size.
//              This pointer needs to be freed after use (delete [] 'pointer')
//
//----------------------------------------------------------------------------
BYTE*   CTRFile::lpbMakeTR4Level(DWORD* lpdwDataSize, bool boConvert /*= true*/, int DontAsk /*=2*/, bool boShowProgress)
{
    CDlgLevelSave*   DlgSave=0;

	if (boShowProgress)
	{
		DlgSave = new CDlgLevelSave;
		DlgSave->vSetLevelName( m_szLevelName );
	    DlgSave->Create(IDD_LEVEL_SAVE);
	}

    bool boKeepUnused = DontAsk!=2 ? (DontAsk==1) : AfxMessageBox( _T("Do you want to keep the unused textures in the level ?"), MB_YESNO | MB_TOPMOST) == IDYES;

    // Pump messages to let the dialog box disappear
    MSG Msg;

    while(PeekMessage(&Msg,NULL,0,0,TRUE))
        {
        TranslateMessage(&Msg);
        DispatchMessage(&Msg);
        }

	// Convert to TR level format
    if( boConvert ) vConvertToTRLevel(DlgSave, boKeepUnused);

	// Build the geometry data to TR format
	DWORD	dwGeometryDataSize = dwCalcGeometryDataSize();
	BYTE*	lpbGeometry = new BYTE [ dwGeometryDataSize ];
	TR4_COMPR_TEXTURE Geometry;

    if (DlgSave) DlgSave->vSetSaveRange( dwGeometryDataSize * 2 );

	vMakeGeometry( lpbGeometry, DlgSave );

	// Compress the geometry data
	vCompressTexture( lpbGeometry, dwGeometryDataSize, &Geometry );

	delete [] lpbGeometry;

    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;

	delete [] lp->Texture1.lpbTextData; lp->Texture1.lpbTextData = NULL;

	delete [] lp->Texture2.lpbTextData; lp->Texture2.lpbTextData = NULL;

	// Build the 16 bits texture
	DWORD dwNumTotTiles = lp->dwNumTextiles;

	TR2_TEXTILE16*	lpTexture = new TR2_TEXTILE16 [ dwNumTotTiles ];

	for( DWORD dwTile = 0; dwTile < dwNumTotTiles; ++dwTile )
		{
		lpTexture[ dwTile ] = *lp->lplpTextiles16[ dwTile ];
		}

	vCompressTexture( (BYTE*) lpTexture, dwNumTotTiles * sizeof( TR2_TEXTILE16 ), &lp->Texture2 );

    if (DlgSave) DlgSave->vSetSaveProgress( dwGeometryDataSize + dwGeometryDataSize / 2 );

	// Build the 32 bits texture
	BYTE*	lpTexture32 = new BYTE [ dwNumTotTiles * 256 * 256 * 4 ];

	vPromoteTo32Bits( (BYTE*) lpTexture, 256, 256 * dwNumTotTiles, 16, NULL, lpTexture32, FALSE );

	vCompressTexture( lpTexture32, dwNumTotTiles * 256 * 256 * 4, &lp->Texture1 );

	delete [] lpTexture32;

	delete [] lpTexture;

	// Calc the size of the whole level and allocate enough memory to hold it
    *lpdwDataSize = dwCalcLevelTR4DataSize( false ) + Geometry.dwTextCompSize;

    DWORD   dwSize;
    BYTE*   lpbData = new BYTE [*lpdwDataSize];
    BYTE*   lpData = lpbData;

    ZeroMemory(lpbData,*lpdwDataSize);

    //
    // Write version
    //
    * ((bitu32*) lpData) = dwGetVersionNumber(m_TRType); lpData += sizeof(bitu32);

    //
    // Write num tiles
    //
    * ((bitu16*) lpData) = lp->wNumTextNoBump; lpData += sizeof(bitu16);
    * ((bitu16*) lpData) = lp->wNumTextInhab; lpData += sizeof(bitu16);
    * ((bitu16*) lpData) = lp->wNumTextBump; lpData += sizeof(bitu16);

    //
    // Write textures
    //
    * ((bitu32*) lpData) = lp->Texture1.dwTextUncompSize;   lpData += sizeof(bitu32);
    * ((bitu32*) lpData) = lp->Texture1.dwTextCompSize;     lpData += sizeof(bitu32);
    WRITE_ARRAY(lp->Texture1.dwTextCompSize,bitu8,lp->Texture1.lpbTextData)

    * ((bitu32*) lpData) = lp->Texture2.dwTextUncompSize;   lpData += sizeof(bitu32);
    * ((bitu32*) lpData) = lp->Texture2.dwTextCompSize;     lpData += sizeof(bitu32);
    WRITE_ARRAY(lp->Texture2.dwTextCompSize,bitu8,lp->Texture2.lpbTextData)

    * ((bitu32*) lpData) = lp->Texture3.dwTextUncompSize;   lpData += sizeof(bitu32);
    * ((bitu32*) lpData) = lp->Texture3.dwTextCompSize;     lpData += sizeof(bitu32);
    WRITE_ARRAY(lp->Texture3.dwTextCompSize,bitu8,lp->Texture3.lpbTextData)

	//
	// Write geometry data
	//
    * ((bitu32*) lpData) = Geometry.dwTextUncompSize;		lpData += sizeof(bitu32);
    * ((bitu32*) lpData) = Geometry.dwTextCompSize;			lpData += sizeof(bitu32);
	memcpy( lpData, Geometry.lpbTextData, Geometry.dwTextCompSize ); lpData += Geometry.dwTextCompSize;

    //
    // Write sounds
    //
    * ((bitu32*) lpData) = lp->dwNumSounds;			lpData += sizeof(bitu32);

	CTRViewerApp*	lpApp = (CTRViewerApp*) AfxGetApp();

	bool	boUncompressSounds = lpApp->boSaveUncompressedSounds();

	for( DWORD dwSound = 0; dwSound < lp->dwNumSounds; ++dwSound )
		{
		TR4_SOUND*	lpSound = &lp->lpSounds[ dwSound ];
		bool		boSaveOriginalSound = true;

		if( boUncompressSounds  &&  ( lpSound->dwSoundLen != lpSound->dwUncompSoundLen ) )
			{
			DWORD	dwSoundSize;
			BYTE*	lpbSound = lpbUncompressSound( lpSound, dwSoundSize );

			if( lpbSound != NULL )
				{
				boSaveOriginalSound = false;

				* ((bitu32*) lpData) = dwSoundSize;		lpData += sizeof(bitu32);
				* ((bitu32*) lpData) = dwSoundSize;		lpData += sizeof(bitu32);

				memcpy( lpData, lpbSound, dwSoundSize );

				lpData += dwSoundSize;

				delete [] lpbSound;
				}
			}

		if( boSaveOriginalSound )
			{
			* ((bitu32*) lpData) = lpSound->dwUncompSoundLen;	lpData += sizeof(bitu32);
			* ((bitu32*) lpData) = lpSound->dwSoundLen;			lpData += sizeof(bitu32);

			memcpy( lpData, lpSound->lpbData, lpSound->dwSoundLen );

			lpData += lpSound->dwSoundLen;
			}
		}

	delete [] Geometry.lpbTextData;

	delete [] lp->Texture1.lpbTextData; lp->Texture1.lpbTextData = NULL;
	delete [] lp->Texture2.lpbTextData; lp->Texture2.lpbTextData = NULL;

    if (DlgSave) DlgSave->vSetSaveProgress( dwGeometryDataSize * 2 );

	vConvertFromTRLevel( DlgSave );

	*lpdwDataSize = lpData - lpbData;

    if (DlgSave) DlgSave->DestroyWindow();

    delete DlgSave;

    return lpbData;
}


void	CTRFile::vParseGeometry( BYTE* lpData, DWORD dwBytesRead, CDlgLevelLoad* DlgLoad )
{
    DWORD   dwI, dwSize;
    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;
    BYTE*   lpbData = lpData;

    lpData += sizeof(bitu32); // unused

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read Rooms data
    //
    lp->wNumRooms           = *(bitu16*)            lpData;     lpData += sizeof(bitu16);
    lp->lpRooms = new TR123_ROOM [lp->wNumRooms];

    for(dwI = 0; dwI < lp->wNumRooms; dwI++)
        {
        TR123_ROOM*    lpRoom = &lp->lpRooms[dwI];

        lpRoom->RoomInfo = *(TR2_ROOM_INFO*)    lpData;         lpData += sizeof(TR2_ROOM_INFO);
        lpData += sizeof(bitu32); // NumData - unused

        TR2_ROOM_DATA* lpRoomData = (TR2_ROOM_DATA*) &lpRoom->RoomData;

        READ_ARRAYNUM_TR(bitu16,TR123_VERTEX_ROOM,lpRoomData->wNumVertices,lpRoomData->lpVertices,TR1_VERTEX_ROOM,TR2_VERTEX_ROOM)
        READ_ARRAYNUM2(bitu16,TR2_FACE4,lpRoomData->wNumRectangles,lpRoomData->lpRectangles,sizeof(bitu16))
 		READ_ARRAYNUM2(bitu16,TR2_FACE3,lpRoomData->wNumTriangles,lpRoomData->lpTriangles,sizeof(bitu16))
        READ_ARRAYNUM(bitu16,TR2_ROOM_SPRITE,lpRoomData->wNumSprites,lpRoomData->lpSprites)
        READ_ARRAYNUM(bitu16,TR2_ROOM_PORTAL,lpRoom->wNumPortals,lpRoom->lpPortals)

        lpRoom->wNumZSectors = *(bitu16*) lpData;        lpData += sizeof(bitu16);
        lpRoom->wNumXSectors = *(bitu16*) lpData;        lpData += sizeof(bitu16);
        READ_ARRAY(lpRoom->wNumZSectors * lpRoom->wNumXSectors,TR2_ROOM_SECTOR,lpRoom->lpSectorList)

        lpRoom->sAmbientIntensity1 = *(bit16*) lpData;        lpData += sizeof(bit16);
        if(m_TRType != TR1_FILE)
            {
            lpRoom->sAmbientIntensity2 = *(bit16*) lpData;        lpData += sizeof(bit16);
            }
        if(m_TRType == TR2_FILE)
            {
            lpRoom->sLightMode = *(bit16*) lpData;        lpData += sizeof(bit16);
            }
		else lpRoom->sLightMode = 0;
			
        READ_ARRAYNUM_TR2(bitu16,TR123_ROOM_LIGHT,lpRoom->wNumLights,lpRoom->lpLights,TR1_ROOM_LIGHT,TR2_ROOM_LIGHT,TR4_ROOM_LIGHT)
        READ_ARRAYNUM_TR(bitu16,TR123_ROOM_STATICMESH,lpRoom->wNumStaticMeshes,lpRoom->lpStaticMeshes,TR1_ROOM_STATICMESH,TR2_ROOM_STATICMESH)

        lpRoom->sAlternateRoom = *(bit16*) lpData;        lpData += sizeof(bit16);
        lpRoom->sFlags = *(bit16*) lpData;        lpData += sizeof(bit16);
        if(m_TRType >= TR3_FILE)
            {
            lpRoom->RoomLightColor        = *(TR2_COLOR*) lpData;  lpData += sizeof(TR2_COLOR);
            }
		else lpRoom->RoomLightColor.bBlue = lpRoom->RoomLightColor.bGreen = lpRoom->RoomLightColor.bRed = 0;
        }

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read behaviours data
    //

    READ_ARRAYNUM(bitu32,bitu16,lp->dwNumFloorData,lp->lpFloorData)

    //
    // Read Meshes geometry data
    //
    lp->dwNumMeshes = 0;

    bitu32 dwNumMeshData      = *(bitu32*)             lpData;  lpData += sizeof(bitu32);
    BYTE* lpbAddrStartMesh = lpData;
    BYTE* lpbMeshEndAddr = lpData + dwNumMeshData * 2;

	lpData = lpbMeshEndAddr;

    READ_ARRAYNUM(bitu32,bitu32,lp->dwNumMeshPointers,lp->lpMeshPointers)

	lp->dwNumMeshes = lp->dwNumMeshPointers;
    lp->lplpMeshes = new TR123_MESH* [lp->dwNumMeshes];

    for(dwI = 0; dwI < lp->dwNumMeshPointers; dwI++)
        {
        bitu32  dwAddr = lp->lpMeshPointers[dwI];

        lp->lplpMeshes[dwI] = new TR123_MESH;
        if(m_TRType == TR4_FILE) lpbReadTR4Mesh(lp->lplpMeshes[dwI],lpbAddrStartMesh + dwAddr,lpbAddrStartMesh); else lpbReadMesh(lp->lplpMeshes[dwI],lpbAddrStartMesh + dwAddr,lpbAddrStartMesh);
		
        lp->lpMeshPointers[dwI] = dwAddr ? dwI : 0;
        }

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read meshes animations data
    //

	if( m_TRType == TR4_FILE )
		{
		READ_ARRAYNUM(bitu32,TR2_ANIMATION,lp->dwNumAnimations,lp->lpAnimations)
		}
	else
		{
		lp->dwNumAnimations = *((bitu32*) lpData); lpData += sizeof(bitu32);
		lp->lpAnimations = new TR2_ANIMATION [lp->dwNumAnimations];

		for( DWORD dwI = 0; dwI < lp->dwNumAnimations; ++dwI )
			{
			memcpy( &lp->lpAnimations[ dwI ], lpData, 16 ); lpData += 16;
			memcpy( &lp->lpAnimations[ dwI ].wFrameStart, lpData, 16 ); lpData += 16;

			ZeroMemory( &lp->lpAnimations[ dwI ].Unknown2, 8 );
			}
		}

    READ_ARRAYNUM(bitu32,TR2_STATE_CHANGE,lp->dwNumStateChanges,lp->lpStateChanges)
    READ_ARRAYNUM(bitu32,TR2_ANIM_DISPATCH,lp->dwNumAnimDispatches,lp->lpAnimDispatches)
    READ_ARRAYNUM(bitu32,TR2_ANIM_COMMAND,lp->dwNumAnimCommands,lp->lpAnimCommands)
    READ_ARRAYNUM(bitu32,TR2_MESHTREE,lp->dwNumMeshTrees,lp->lpMeshTrees)
    READ_ARRAYNUM(bitu32,bitu16,lp->dwNumFrames,lp->lpFrames)

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read meshes lists data
    //
    READ_ARRAYNUM(bitu32,TR2_MOVEABLE,lp->dwNumMoveables,lp->lpMoveables)
    READ_ARRAYNUM(bitu32,TR2_STATICMESH,lp->dwNumStaticMeshes,lp->lpStaticMeshes)

    if(m_TRType <= TR2_FILE)
        {
		lp->dwNumObjectTextures = * ((bitu32*) lpData); lpData += sizeof(bitu32);
		lp->lpObjectTextures = new TR2_OBJECT_TEXTURE[ lp->dwNumObjectTextures ];

		for( DWORD dwI = 0; dwI < lp->dwNumObjectTextures; ++dwI )
			{
			memcpy( &lp->lpObjectTextures[ dwI ], lpData, 4 ); lpData += 4;
			memcpy( &lp->lpObjectTextures[ dwI ].Vertices, lpData, 4 * sizeof(TR2_OBJECT_TEXTURE_VERT) ); lpData += 4 * sizeof(TR2_OBJECT_TEXTURE_VERT);
			ZeroMemory( &lp->lpObjectTextures[ dwI ].dwXOffset, 16 );
			lp->lpObjectTextures[ dwI ].Unknown = 0;
			}
        }

    if(m_TRType == TR4_FILE)
		{
		lpData += 3;	// skip 'SPR'
		}

    READ_ARRAYNUM(bitu32,TR2_SPRITE_TEXTURE,lp->dwNumSpriteTextures,lp->lpSpriteTextures)
    READ_ARRAYNUM(bitu32,TR2_SPRITE_SEQUENCE,lp->dwNumSpriteSequences,lp->lpSpriteSequences)

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read other & sound data
    //
    if(m_TRType == TR1_FILE  &&  m_boTUBFile)
        {
        lp->Palette8        = *(TR2_PALETTE8*)      lpData;     lpData += sizeof(TR2_PALETTE8);
        }

    READ_ARRAYNUM(bitu32,TR2_CAMERA,lp->dwNumCameras,lp->lpCameras)

	if( m_TRType == TR4_FILE )
		{
		READ_ARRAYNUM(bitu32,TR4_FLYBY_CAMERA,lp->dwNumFlybyCameras,lp->lpFlybyCameras)
		}

    READ_ARRAYNUM(bitu32,TR2_SOUND_SOURCE,lp->dwNumSoundSources,lp->lpSoundSources)

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read NPCs data
    //
    READ_ARRAYNUM_TR(bitu32,TR123_BOX,lp->dwNumBoxes,lp->lpBoxes,TR1_BOX,TR2_BOX)
    READ_ARRAYNUM(bitu32,bitu16,lp->dwNumOverlaps,lp->lpOverlaps)
    READ_ARRAY_TR(lp->dwNumBoxes,TR123_ZONE,lp->lpZones,TR1_ZONE,TR2_ZONE)

    //
    // Read textures data
    //
    READ_ARRAYNUM(bitu32,bitu16,lp->dwNumAnimatedTextures,lp->lpAnimatedTextures)

    if(m_TRType >= TR3_FILE)
        {
		if( m_TRType == TR4_FILE )
			{
			lp->bNumUVRotate = * ((bitu8*) lpData);
			lpData += 4;	// skip 'xTEX'
			READ_ARRAYNUM(bitu32,TR2_OBJECT_TEXTURE,lp->dwNumObjectTextures,lp->lpObjectTextures)
			}
		else
			{
			lp->dwNumObjectTextures = * ((bitu32*) lpData); lpData += sizeof(bitu32);
			lp->lpObjectTextures = new TR2_OBJECT_TEXTURE[ lp->dwNumObjectTextures ];

			for( DWORD dwI = 0; dwI < lp->dwNumObjectTextures; ++dwI )
				{
				memcpy( &lp->lpObjectTextures[ dwI ], lpData, 4 ); lpData += 4;
				memcpy( &lp->lpObjectTextures[ dwI ].Vertices, lpData, 4 * sizeof(TR2_OBJECT_TEXTURE_VERT) ); lpData += 4 * sizeof(TR2_OBJECT_TEXTURE_VERT);

				ZeroMemory( &lp->lpObjectTextures[ dwI ].dwXOffset, 16 );
				lp->lpObjectTextures[ dwI ].Unknown = 0;
				}
			}
        }

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read other data
    //
    READ_ARRAYNUM_TR(bitu32,TR123_ITEM,lp->dwNumItems,lp->lpItems,TR1_ITEM,TR2_ITEM)

	if( m_TRType != TR4_FILE )
		{
		lp->LightMap                   = *(TR2_LIGHTMAP*)       lpData;  lpData += sizeof(TR2_LIGHTMAP);
		}

    if(m_TRType == TR1_FILE  &&  !m_boTUBFile)
        {
        lp->Palette8    = *(TR2_PALETTE8*)      lpData;     lpData += sizeof(TR2_PALETTE8);
        }

	if( m_TRType != TR4_FILE )
		{
        lp->dwNumCinematicFrames		= *(WORD*)      lpData;     lpData += sizeof(WORD);
		READ_ARRAY2(lp->dwNumCinematicFrames,TR2_CINEMATIC_FRAME,lp->lpCinematicFrames,8)
		}
	else
		{
        lp->dwNumCinematicFrames		= *(DWORD*)     lpData;     lpData += sizeof(DWORD);
		READ_ARRAY(lp->dwNumCinematicFrames,TR2_CINEMATIC_FRAME,lp->lpCinematicFrames)
		}

    READ_ARRAYNUM(bitu16,bitu8,lp->wNumDemoData,lp->lpDemoData)

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

    //
    // Read sound data
    //
    if(m_TRType != TR1_FILE)
        {
        lp->SoundMap.tr2 = *(TR2_SOUNDMAP*) lpData;
        lpData += sizeof(TR2_SOUNDMAP);
        }
    else
        {
        lp->SoundMap.tr1 = *(TR1_SOUNDMAP*) lpData;
        lpData += sizeof(TR1_SOUNDMAP);
        }

    READ_ARRAYNUM(bitu32,TR2_SOUND_DETAILS,lp->dwNumSoundDetails,lp->lpSoundDetails)

    if(m_TRType == TR1_FILE)
        {
        READ_ARRAYNUM(bitu32,bitu8,lp->dwNumSamples,lp->lpSamples)
        }

    READ_ARRAYNUM(bitu32,bitu32,lp->dwNumSampleIndices,lp->lpSampleIndices)

	if( m_TRType == TR4_FILE )
		{
		lpData += 6;	// skip 6 unknown bytes
		}

    if (DlgLoad) DlgLoad->vSetLoadProgress(dwBytesRead + lpData - lpbData);

	// Hack for the sky moveable in TR3: replace the lower part of the sky cone by
	// black colored polys
	if( GetTRType() == TR3_FILE )
		{
		BYTE bIdx16 = bFindColor( 0.0f, 0.0f, 0.0f );
		BYTE bIdx8 = ::bFindColor( (BYTE*) m_trFile.tr123File.Palette8.aRGB, 0, 0, 0 );

		for(DWORD dwNumMoveable = 0; dwNumMoveable < m_trFile.tr123File.dwNumMoveables; dwNumMoveable++)
			{
			TR2_MOVEABLE* lpMoveable = &m_trFile.tr123File.lpMoveables[dwNumMoveable];

			if(lpMoveable->dwObjectId == __OBJID_TR3_SKY__)
				{
				DWORD dwNumMesh = GetTRFile().tr123File.lpMeshPointers[lpMoveable->wStartingMesh + 0];

				TR123_MESH* lpm = GetTRFile().tr123File.lplpMeshes[dwNumMesh];

				for( DWORD dwI = 16; dwI < (DWORD)lpm->sNumColoredTriangles; dwI++ )
					{
					lpm->lpColoredTriangles[ dwI ].wTextureIndex = (bIdx16 << 8) + bIdx8;
					}
				}
			}
		}
}


//----------------------------------------------------------------------------
//
//  Function:   vParseTR123File
//
//  Synopsis:   Read a TR1/2/3 level and build internal structures accordingly
//
//  Arguments:  [lpData] -- pointer on the level data
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTRFile::vParseTR123File( BYTE*& lpData, CDlgLevelLoad* DlgLoad )
{
    DWORD   dwI;
	DWORD  dwBytesRead = 0;
    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;
    BYTE*   lpbData = lpData;

    TR2_TEXTILE8**  lplpTextiles8;

    //
    // Read palettes data
    //
    if(m_TRType != TR1_FILE)
        {
        lp->Palette8    = *(TR2_PALETTE8*)      lpData;     lpData += sizeof(TR2_PALETTE8);
        lp->Palette16   = *(TR2_PALETTE16*)     lpData;     lpData += sizeof(TR2_PALETTE16);
        }

    if (DlgLoad) DlgLoad->vSetLoadProgress(lpData - lpbData);

    //
    // Read Textiles data
    //
    lp->dwNumTextiles       = *(bitu32*)            lpData;     lpData += sizeof(bitu32);

    lplpTextiles8 = new TR2_TEXTILE8* [lp->dwNumTextiles];

    for(dwI = 0; dwI < lp->dwNumTextiles; dwI++)
        {
        lplpTextiles8 [dwI] = new TR2_TEXTILE8;
        *lplpTextiles8[dwI] = *(TR2_TEXTILE8*)  lpData;     lpData += sizeof(TR2_TEXTILE8);
        }

    if(m_TRType != TR1_FILE)
        {
        lp->lplpTextiles16  = new TR2_TEXTILE16* [lp->dwNumTextiles];
        for(dwI = 0; dwI < lp->dwNumTextiles; dwI++)
            {
            lp->lplpTextiles16 [dwI]    = new TR2_TEXTILE16;
            *lp->lplpTextiles16[dwI]    = *(TR2_TEXTILE16*) lpData; lpData += sizeof(TR2_TEXTILE16);
            }
        }

    if (DlgLoad) DlgLoad->vSetLoadProgress(lpData - lpbData);

	dwBytesRead = lpData - lpbData;

	vParseGeometry( lpData, dwBytesRead, DlgLoad );

    //
    // Build the 16 bits textiles if TR1
    //
    if(m_TRType == TR1_FILE)
        {
        lp->lplpTextiles16  = new TR2_TEXTILE16* [lp->dwNumTextiles];

        for(DWORD dwTile = 0; dwTile < lp->dwNumTextiles; dwTile++)
            {
            lp->lplpTextiles16 [dwTile]    = new TR2_TEXTILE16;

            for(DWORD dwJ = 0; dwJ < 256; dwJ++)
                {
                for(DWORD dwI = 0; dwI < 256; dwI++)
                    {
                    BYTE bIndex = *((BYTE*)(lplpTextiles8[dwTile]->abTile + dwI + dwJ * 256));

                    BYTE bR = lp->Palette8.aRGB[bIndex].bRed    >> 1;
                    BYTE bG = lp->Palette8.aRGB[bIndex].bGreen  >> 1;
                    BYTE bB = lp->Palette8.aRGB[bIndex].bBlue   >> 1;

                    WORD wColor = (bR << 10) + (bG << 5) + bB;

                    if(bIndex != 0) wColor |= 0x8000;

                    lp->lplpTextiles16[dwTile]->awTile[dwI + dwJ * 256] = wColor;
                    }
                }
            }
        }

    //
    // Delete the 8 bits textiles
    //
    for(DWORD dwTile = 0; dwTile < lp->dwNumTextiles; dwTile++)
        {
        delete lplpTextiles8[dwTile];
        }

    delete [] lplpTextiles8;
}


//----------------------------------------------------------------------------
//
//  Function:   vParseTR4File
//
//  Synopsis:   Read a TR4 level and build internal structures accordingly
//
//  Arguments:  [lpData] -- pointer on the level data
//              [dwDataSize] -- data size of the level
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTRFile::vParseTR4File( BYTE*& lpData, DWORD dwDataSize, CDlgLevelLoad* DlgLoad )
{
    DWORD   dwSize;
    BYTE*   lpDataSave = lpData;
    TR123_FILE_STRUCT* lp = (TR123_FILE_STRUCT*) &m_trFile.tr123File;

    BYTE*   lpabDataUncomp = NULL;
    DWORD   dwHeight = 0;

	// Read the texture chunks
    lp->wNumTextNoBump	= * (bitu16*) lpData;     lpData += sizeof(bitu16);
    lp->wNumTextInhab	= * (bitu16*) lpData;     lpData += sizeof(bitu16);
    lp->wNumTextBump	= * (bitu16*) lpData;     lpData += sizeof(bitu16);

    lp->Texture1.dwTextUncompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    lp->Texture1.dwTextCompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    READ_ARRAY(lp->Texture1.dwTextCompSize,bitu8,lp->Texture1.lpbTextData)

    lp->Texture2.dwTextUncompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    lp->Texture2.dwTextCompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    READ_ARRAY(lp->Texture2.dwTextCompSize,bitu8,lp->Texture2.lpbTextData)

    lp->Texture3.dwTextUncompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    lp->Texture3.dwTextCompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    READ_ARRAY(lp->Texture3.dwTextCompSize,bitu8,lp->Texture3.lpbTextData)

	// Read the geometry and sounds chunks
    TR4_COMPR_TEXTURE	Geometry;

    Geometry.dwTextUncompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    Geometry.dwTextCompSize = * (bitu32*) lpData;  lpData += sizeof(bitu32);
    READ_ARRAY(Geometry.dwTextCompSize,bitu8,Geometry.lpbTextData)

	// Read sounds
	lp->dwNumSounds = * (bitu32*) lpData;	lpData += sizeof( bitu32 );

	lp->lpSounds = new TR4_SOUND [ lp->dwNumSounds ];

	for( DWORD dwSound = 0; dwSound < lp->dwNumSounds; ++dwSound )
		{
		lp->lpSounds[ dwSound ].dwUncompSoundLen	= * (bitu32*) lpData;	lpData += sizeof( bitu32 );
		lp->lpSounds[ dwSound ].dwSoundLen			= * (bitu32*) lpData;	lpData += sizeof( bitu32 );
		lp->lpSounds[ dwSound ].lpbData = new bitu8 [ lp->lpSounds[ dwSound ].dwSoundLen ];

		memcpy( lp->lpSounds[ dwSound ].lpbData, lpData, lp->lpSounds[ dwSound ].dwSoundLen );

		//! DBG
		/*char sz[123];

		wsprintf(sz,"c:\\temp\\snd\\Tnd#%.4d.wav",dwSound);

		DWORD dwNumWritten;

		HANDLE ff = CreateFile(sz,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_ARCHIVE,NULL);

		_ASSERTE( ff != INVALID_HANDLE_VALUE );

		WriteFile( ff, lp->lpSounds[ dwSound ].lpbData, lp->lpSounds[ dwSound ].dwSoundLen, &dwNumWritten, NULL );
		CloseHandle( ff );*/
		//! DBG

		lpData += lp->lpSounds[ dwSound ].dwSoundLen;
		}

	// Uncompress the 16 bits texture and set texture related data
	lp->dwNumTextiles = lp->wNumTextNoBump + lp->wNumTextInhab + lp->wNumTextBump;
	lp->lplpTextiles16 = new TR2_TEXTILE16* [ lp->dwNumTextiles ];

    lpabDataUncomp = NULL;
    dwHeight = 0;

    vUncompressTexture(&lp->Texture2,false,&lpabDataUncomp,&dwHeight);

	for( DWORD dwI = 0; dwI < lp->dwNumTextiles; ++dwI )
	{
		lp->lplpTextiles16[ dwI ] = new TR2_TEXTILE16;

		memcpy( lp->lplpTextiles16[ dwI ]->awTile, lpabDataUncomp + sizeof( TR2_TEXTILE16 ) * dwI, sizeof( TR2_TEXTILE16 ) );
	}

	delete [] lpabDataUncomp;

	delete [] lp->Texture1.lpbTextData;	lp->Texture1.lpbTextData = NULL;	// free some memory -> we don't need it anymore
	delete [] lp->Texture2.lpbTextData;	lp->Texture2.lpbTextData = NULL;	// free some memory -> we don't need it anymore

    if (DlgLoad) DlgLoad->vSetLoadRange( Geometry.dwTextUncompSize );

	// Uncompress the geometry data
    lpabDataUncomp = NULL;
    dwHeight = 0;

    vUncompressTexture(&Geometry,false,&lpabDataUncomp,&dwHeight);

	delete [] Geometry.lpbTextData;

	// Parse the geometry
	vParseGeometry( lpabDataUncomp, 0, DlgLoad );

		//! DBG
/*		char sz[123];

		wsprintf(sz,"c:\\temp\\settomb_w2.bin");

		DWORD dwNumWritten;

		HANDLE ff = CreateFile(sz,GENERIC_WRITE,0,NULL,CREATE_ALWAYS,FILE_ATTRIBUTE_ARCHIVE,NULL);

		_ASSERTE( ff != INVALID_HANDLE_VALUE );

		WriteFile( ff, lpabDataUncomp, Geometry.dwTextUncompSize, &dwNumWritten, NULL );
		CloseHandle( ff );*/
		//! DBG

	delete [] lpabDataUncomp;
}


//----------------------------------------------------------------------------
//
//  Function:   boParseTRLevel
//
//  Synopsis:   Read a TR level and build internal structures accordingly
//
//  Arguments:  [lpData] -- pointer on the level data
//              [dwDataSize] -- data size of the level
//              [lpszFileName] -- name of the file
//
//  Return:     true if the level could be parsed, else false
//
//----------------------------------------------------------------------------
bool    CTRFile::boParseTRLevel(BYTE* lpData,DWORD dwDataSize,TCHAR* lpszFileName, bool boDontConvert, bool boShowProgress)
{
    _tcsupr(lpszFileName);

    DWORD dwVersion = *(bitu32*) lpData; lpData += sizeof(bitu32);

    switch(dwVersion)
        {
        case 0x00000020:    m_TRType        = TR1_FILE; break;
        case 0x0000002d:    m_TRType        = TR2_FILE; break;
        case 0xFF080038:
        case 0xFF180038:    m_TRType        = TR3_FILE; break;
        case 0x00345254:    m_TRType        = TR4_FILE; break;
        default:            m_TRType        = TR_UNKNOWN;
        }

    if(m_TRType == TR_UNKNOWN) return false;

    if(m_TRType == TR1_FILE  &&  (_tcsstr(lpszFileName,_T(".tub")) != NULL))
        {
        m_boTUBFile = true;
        }

	dwDataSize -= sizeof( bitu32 );

    CDlgLevelLoad*   DlgLoad=0;

	if (boShowProgress)
	{
		DlgLoad = new CDlgLevelLoad;
		DlgLoad->vSetLevelName( m_szLevelName );
		DlgLoad->Create(IDD_LEVEL_LOAD);
		DlgLoad->vSetLoadRange(dwDataSize);
	}

    if(m_TRType == TR4_FILE)
        {
        vParseTR4File( lpData, dwDataSize, DlgLoad );
        }
	else
		{
		vParseTR123File( lpData, DlgLoad );
		}

	// Debug
/*	for(DWORD dwObjText = 0; dwObjText < lp->dwNumObjectTextures; dwObjText++)
		{
		TR2_OBJECT_TEXTURE*	lpText = &lp->lpObjectTextures[ dwObjText ];

		DWORD dwTile = lpText->wTile & 0xFF;

		bitu16*	lpData = m_trFile.tr123File.lplpTextiles16[dwTile]->awTile;

		bit16 sMinU, sMinV, sMaxU, sMaxV;

		vComputeObjTextureExtents(lpText,lpText->wTile & 0x8000 ? 3 : 4,false,&sMinU, &sMinV,&sMaxU, &sMaxV);

		for(long j=0; j<sMaxV-sMinV+1;++j)
			{
			memset(lpData+sMinU+(sMinV+j)*256,255,(sMaxU-sMinU+1)*2);
			}
		}

	for(long i=0; i<m_trFile.tr123File.dwNumTextiles;++i)
		{
		bitu16*	lpData = m_trFile.tr123File.lplpTextiles16[i]->awTile;

		char sz[123];
		sprintf(sz,"c:\\temp\\tr4text\\%s%.2d.tga",m_szLevelName,i);
		vMakeTGA16Bits(sz,(BYTE*)lpData,256,256,2,true);
		}
*/

	if (!boDontConvert)
	{
		if( m_TRType != TR4_FILE ) dwReplaceColoredPolys(m_trFile.tr123File.Palette8, m_trFile.tr123File.Palette16);

		//
		// Convert to internal representation of the level
		//
		vConvertFromTRLevel(DlgLoad);
	}

    if (DlgLoad) DlgLoad->DestroyWindow();

	delete DlgLoad;

    return true;
}
