#include "stdafx.h"

#include "Resource.h"
#include "TrViewer.h"
#include "Fragment.h"
#include "DlgLevelLoad.h"
#include "DlgLevelSave.h"
#include "AnimData.h"
#include "AnimKey.h"
#include "AnimTrack.h"
#include "AnimMgr.h"
#include "TRAnimTrack.h"
#include "TGAFile.h"


#ifdef __EXACT_TRANGLE__
DWORD dwGetFrameSize(CTRAnimTrack* lpTrack)
{
    DWORD   dwFrameSize = 0;

    for(DWORD dwKey = 0; dwKey < lpTrack->dwGetNumKeys(); dwKey++)
        {
        CAnimKey* lpKey = lpTrack->lpGetKey(dwKey);

        DWORD   dwCurFrameSize = 0;

        for(DWORD dwData = 0; dwData < lpKey->dwGetNumData(); dwData++)
            {
            const CAnimData& Data = lpKey->GetData(dwData);

            WORD    wTRAngle1, wTRAngle2;

            Data.vGetTRAngle(wTRAngle1, wTRAngle2);

            dwCurFrameSize++;

            if(wTRAngle2 != 0x1234) dwCurFrameSize++;
            }

        if(dwFrameSize < dwCurFrameSize) dwFrameSize = dwCurFrameSize;
        }

    return dwFrameSize + 3 * 3;
}
#endif


//----------------------------------------------------------------------------
//
//  Function:   vConvertToTRLevel
//
//  Synopsis:   Convert internal representation of the level to TR's one
//
//  Arguments:  [DlgSave] -- save dialog (to update progress bars)
//              [boKeepUnusedTextures] -- if true, save the level with the
//                  unused textures
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTRFile::vConvertToTRLevel(CDlgLevelSave* DlgSave, bool boKeepUnusedTextures)
{
    TR123_FILE_STRUCT& trFile = m_trFile.tr123File;

    bool    boTR1 = (m_TRType == TR1_FILE);

    DWORD   dwMaxRange = 5 + trFile.lpListSpriteSeq->size() + trFile.lpListMoveable->size() * 2 + trFile.lpListStaticMesh->size() + 1;

    if (DlgSave) DlgSave->vSetConvert2Range(dwMaxRange);

    DWORD dwProgress = 0;

    //
    // Recalculate OBJECT_TEXTURE structures
    //
    DWORD       dwNumFragments = 0;
    CFragment*  lpLstFragments;

    // Collect fragments for the whole level and sort them
    CFragment::vCollectFragments(this,&lpLstFragments,&dwNumFragments);

    // Add fragments for the unused object textures
    if(boKeepUnusedTextures)
        {
        for(TRListObjText::iterator theIter = trFile.lpListUnusedObjText->begin(); theIter != trFile.lpListUnusedObjText->end(); theIter++)
            {
            TR2_OBJECT_TEXTURE* lpObjText = &(*theIter);
			bool boTri;

			if( GetTRType() == TR4_FILE )
				{
				boTri = (lpObjText->wTile & 0x8000) ? true : false;
				}
			else
				{
				boTri = (lpObjText->Vertices[3].bXPixel == 0)  &&
                        (lpObjText->Vertices[3].bYPixel == 0)  &&
                        (lpObjText->Vertices[3].cXCoordinate == 0)  &&
                        (lpObjText->Vertices[3].cYCoordinate == 0);
				}

            BYTE    bNumVert = boTri ? 3 : 4;
            BYTE*   lpbTextureData = (BYTE*) trFile.lplpTextiles16[ lpObjText->wTile  & 0xFF ]->awTile;

            CFragment::vAddFragment(lpLstFragments, dwNumFragments, lpObjText, bNumVert, OBJTEXT_TYPE_UNUSED, NULL, true, lpbTextureData);
            }
        }

    CFragment::vSortFragments(lpLstFragments,0,dwNumFragments-1);
    CFragment::dwRemoveDuplicated(dwNumFragments,lpLstFragments);

    // Build the new texture
    BYTE*   lpbArea         = NULL;
    DWORD   dwAreaWidth     = 0;
    DWORD   dwAreaHeight    = 0;
    DWORD   dwTextureDepth  = 16;

    DWORD   dwNumObjText = CFragment::dwBuildTextureFromFragments(lpLstFragments,dwNumFragments,&lpbArea,&dwAreaWidth,&dwAreaHeight,true);

	DWORD	dwOldNumTextInhab = trFile.wNumTextInhab;

    trFile.wNumTextInhab = (bitu16)((dwAreaHeight - 1) / 256 + 1);

    // Create the new TR2_OBJECT_TEXTURE structures
    delete [] trFile.lpObjectTextures;

	dwNumObjText += trFile.dwNumRoomObjText;

    trFile.dwNumObjectTextures = dwNumObjText;
    trFile.lpObjectTextures = new TR2_OBJECT_TEXTURE [ dwNumObjText ];

    DWORD dwIndex = 0;

    for(DWORD dwN = 0; dwN < dwNumFragments; dwN++)
        {
        for(LISTOBJTEXT::iterator theIter = lpLstFragments[dwN].m_LstTextCoord.begin(); theIter != lpLstFragments[dwN].m_LstTextCoord.end(); theIter++)
            {
            FRAGMENT_TEXTURE* lpFragText    = &(*theIter);
            TR2_OBJECT_TEXTURE*  lpObjText  = &lpFragText->ObjText;

            if(!lpFragText->boAlreadyDone)
				{
				trFile.lpObjectTextures[dwIndex] = *lpObjText;
				trFile.lpObjectTextures[dwIndex].wTile += trFile.wNumTextNoBump;
				dwIndex++;
				}
            }
        }

	_ASSERTE( dwIndex == dwNumObjText - trFile.dwNumRoomObjText );

    trFile.dwNumObjectTextures = dwNumObjText;

	if( trFile.dwNumRoomObjText != 0  &&  trFile.lpRoomObjText != NULL )
		{
		// re-inject object textures from room / animated textures
		for( DWORD dwObjText = 0; dwObjText < trFile.dwNumRoomObjText; ++dwObjText )
			{
			if( trFile.lpRoomObjText[ dwObjText ].wTile & 0x4000 )
				{
				trFile.lpRoomObjText[ dwObjText ].wTile = ( trFile.lpRoomObjText[ dwObjText ].wTile & 0xbfff ) + trFile.wNumTextInhab;
				}
			}

		memcpy( &trFile.lpObjectTextures[ dwIndex ], trFile.lpRoomObjText, trFile.dwNumRoomObjText * sizeof( TR2_OBJECT_TEXTURE ) );

		for(DWORD dwNumRoom = 0; dwNumRoom < trFile.wNumRooms; dwNumRoom++)
			{
			TR123_ROOM*     lpRoom = &trFile.lpRooms[dwNumRoom];
			TR2_ROOM_DATA*  lpRoomData = &lpRoom->RoomData;

			for(DWORD dwPoly = 0; dwPoly < lpRoomData->wNumRectangles; dwPoly++)
				{
				if( !( lpRoomData->lpRectangles[ dwPoly ].wTextureIndex & 0x4000 ) )
					{
					lpRoomData->lpRectangles[ dwPoly ].wTextureIndex = (bitu16) (lpRoomData->lpRectangles[ dwPoly ].wTextureIndex + dwIndex);
					}
				else
					{
					lpRoomData->lpRectangles[ dwPoly ].wTextureIndex  &= ~0x4000;
					}

				_ASSERTE( (lpRoomData->lpRectangles[ dwPoly ].wTextureIndex & 0x7FFF) < (WORD)dwNumObjText );
				}

			for(dwPoly = 0; dwPoly < lpRoomData->wNumTriangles; dwPoly++)
				{
				if( !( lpRoomData->lpTriangles[ dwPoly ].wTextureIndex & 0x4000 ) )
					{
					lpRoomData->lpTriangles[ dwPoly ].wTextureIndex = (bitu16)(lpRoomData->lpTriangles[ dwPoly ].wTextureIndex + dwIndex);
					}
				else
					{
					lpRoomData->lpTriangles[ dwPoly ].wTextureIndex  &= ~0x4000;
					}

				_ASSERTE( (lpRoomData->lpTriangles[ dwPoly ].wTextureIndex & 0x7FFF) < (WORD)dwNumObjText );
				}
			}

		bitu16* lpsAnimatedTextures     = trFile.lpAnimatedTextures;
		WORD    wNumAnimatedTextures    = *lpsAnimatedTextures++;

		while(wNumAnimatedTextures--)
			{
			bitu16   wNum = *lpsAnimatedTextures++;  wNum++;
			while(wNum--)
				{
				if( !( *lpsAnimatedTextures & 0x4000 ) ) *lpsAnimatedTextures += (bitu16)dwIndex;

				*lpsAnimatedTextures &= ~0x4000;

				_ASSERTE( *lpsAnimatedTextures < dwNumObjText );

				lpsAnimatedTextures++;
				}
			}
		}

	trFile.dwNumRoomObjText = 0;

	delete [] trFile.lpRoomObjText;

	// Process object textures
	for(DWORD dwObjText = 0; dwObjText < trFile.dwNumObjectTextures; dwObjText++)
		{
		TR2_OBJECT_TEXTURE*	lpText = &trFile.lpObjectTextures[ dwObjText ];


		// TR4 only - replace 1 with 0 in cXCoordinate / cYCoordinate of object textures
		if( GetTRType() == TR4_FILE )
			{
			for( BYTE bI = 0; bI < 4; ++bI )
				{
				if( lpText->Vertices[bI].cXCoordinate == 1 ) lpText->Vertices[bI].cXCoordinate = 0;
				if( lpText->Vertices[bI].cYCoordinate == 1 ) lpText->Vertices[bI].cYCoordinate = 0;
				}
			}
		else
			{
			// In TR1/2/3, bit 15 of Tile is always 0
			lpText->wTile &= 0x7FFF;
			}
		}

    // Delete the old tiles
    DWORD dwOldNumTiles = trFile.dwNumTextiles;

    for(dwN = 0; dwN < trFile.dwNumTextiles; dwN++)
        {
		if( ( dwN >= trFile.wNumTextNoBump  &&  dwN < trFile.wNumTextNoBump + dwOldNumTextInhab )  ||  GetTRType() != TR4_FILE )
			delete trFile.lplpTextiles16[dwN];
        }

    // Create the new tiles
	DWORD dwNumRoomText = trFile.wNumTextNoBump + trFile.wNumTextBump;

    trFile.dwNumTextiles = trFile.wNumTextInhab + dwNumRoomText;

    TR2_TEXTILE16** lplpTextiles16 = new TR2_TEXTILE16* [ trFile.dwNumTextiles ];

    for(dwN = 0; dwN < trFile.dwNumTextiles; dwN++)
        {
		if( (WORD)dwN >= trFile.wNumTextNoBump  &&  (WORD)dwN < trFile.wNumTextNoBump + trFile.wNumTextInhab )
			{
			TR2_TEXTILE16* lp16 = new TR2_TEXTILE16;
			memset(lp16,0,sizeof(TR2_TEXTILE16));
			DWORD dwHeight = 256;
			if((dwN - trFile.wNumTextNoBump + 1) * 256 > dwAreaHeight) dwHeight = dwAreaHeight & 0xFF;
			memcpy(lp16,lpbArea + ( dwN - trFile.wNumTextNoBump ) * sizeof(TR2_TEXTILE16), dwHeight * 256 * 2);
			lplpTextiles16[dwN] = lp16;
			}
		else
			{
			lplpTextiles16[ dwN ] = trFile.lplpTextiles16[ dwN >= trFile.wNumTextNoBump ? dwN + dwOldNumTextInhab - trFile.wNumTextInhab : dwN ];
			}
        }

    delete [] trFile.lplpTextiles16;

	trFile.lplpTextiles16 = lplpTextiles16;

    delete [] lpLstFragments;
    delete [] lpbArea;

    if (DlgSave) DlgSave->vSetNumTiles(dwOldNumTiles, trFile.dwNumTextiles);

    dwProgress = 5;

    if (DlgSave) DlgSave->vSetConvert2Progress(dwProgress);

    //
    // Process sprites
    //
    TRListSpriteSeq*    lpListSpriteSeq     = trFile.lpListSpriteSeq;
    DWORD               dwNumSpriteSeq      = lpListSpriteSeq->size();
    DWORD               dwNumSpriteTexture  = 0;

    for(TRListSpriteSeq::iterator theIterator3 = lpListSpriteSeq->begin(); theIterator3 != lpListSpriteSeq->end(); theIterator3++)
        {
        CTrLstSpr* lpLstSpr = *theIterator3;

        dwNumSpriteTexture += lpLstSpr->dwGetNumSprites();
        }

    TR2_SPRITE_SEQUENCE*    lpSpriteSequence    = new TR2_SPRITE_SEQUENCE [ dwNumSpriteSeq ];
    TR2_SPRITE_TEXTURE*     lpSpriteTexture     = new TR2_SPRITE_TEXTURE [ dwNumSpriteTexture ];

    DWORD   dwSpriteSeq             = 0;
    DWORD   dwSpriteTextureOffset   = 0;

    for(theIterator3 = lpListSpriteSeq->begin(); theIterator3 != lpListSpriteSeq->end(); theIterator3++, dwSpriteSeq++)
        {
        CTrLstSpr* lpLstSpr = *theIterator3;

        DWORD   dwNumSprite = lpLstSpr->dwGetNumSprites();

        const TR2_OBJECT_TEXTURE* lpOText = lpLstSpr->lpGetFragText();
        const WORLD_POSITIONNING* lpWorld = lpLstSpr->lpGetWorldPositionning();

        lpSpriteSequence[dwSpriteSeq].lObjectId         = lpLstSpr->dwGetObjectId();
        lpSpriteSequence[dwSpriteSeq].sNegativeLength   = -((bit16)dwNumSprite);
        lpSpriteSequence[dwSpriteSeq].sOffset           = (bit16)dwSpriteTextureOffset;

        for(DWORD dwSprite = 0; dwSprite < dwNumSprite; dwSprite++, dwSpriteTextureOffset++)
            {
            const TR2_OBJECT_TEXTURE* lpObjText = &lpOText[lpWorld[dwSprite].wFragIndex];

            DWORD   dwWidth     = lpObjText->Vertices[1].bXPixel - lpObjText->Vertices[0].bXPixel;
            DWORD   dwHeight    = lpObjText->Vertices[2].bYPixel - lpObjText->Vertices[0].bYPixel;

			if( m_TRType != TR4_FILE )
				{
				lpSpriteTexture[dwSpriteTextureOffset].bX           = lpObjText->Vertices[0].bXPixel;
				lpSpriteTexture[dwSpriteTextureOffset].bY           = lpObjText->Vertices[0].bYPixel;
				lpSpriteTexture[dwSpriteTextureOffset].sBottomSide  = lpWorld[dwSprite].sBottomSide;
				lpSpriteTexture[dwSpriteTextureOffset].sLeftSide    = lpWorld[dwSprite].sLeftSide;
				lpSpriteTexture[dwSpriteTextureOffset].sRightSide   = lpWorld[dwSprite].sRightSide;
				lpSpriteTexture[dwSpriteTextureOffset].sTopSide     = lpWorld[dwSprite].sTopSide;
				lpSpriteTexture[dwSpriteTextureOffset].wHeight      = (bitu16)(dwHeight * 256 + 255);
				lpSpriteTexture[dwSpriteTextureOffset].wWidth       = (bitu16)(dwWidth * 256 + 255);
				lpSpriteTexture[dwSpriteTextureOffset].wTile        = lpObjText->wTile;
				}
			else
				{
				lpSpriteTexture[dwSpriteTextureOffset].bX           = lpWorld[dwSprite].bX;
				lpSpriteTexture[dwSpriteTextureOffset].bY           = lpWorld[dwSprite].bY;
				lpSpriteTexture[dwSpriteTextureOffset].sLeftSide    = lpObjText->Vertices[0].bXPixel;
				lpSpriteTexture[dwSpriteTextureOffset].sRightSide   = (bit16)(lpObjText->Vertices[0].bXPixel + dwWidth + 1);
				lpSpriteTexture[dwSpriteTextureOffset].sTopSide     = lpObjText->Vertices[0].bYPixel;
				lpSpriteTexture[dwSpriteTextureOffset].sBottomSide  = (bit16)(lpObjText->Vertices[0].bYPixel + dwHeight + 1);
				lpSpriteTexture[dwSpriteTextureOffset].wHeight      = (bitu16)(dwHeight * 256);
				lpSpriteTexture[dwSpriteTextureOffset].wWidth       = (bitu16)(dwWidth * 256);
				lpSpriteTexture[dwSpriteTextureOffset].wTile        = lpObjText->wTile + trFile.wNumTextNoBump;
				}
            }

        delete lpLstSpr;

        if (DlgSave) DlgSave->vSetConvert2Progress(++dwProgress);
        }

    delete [] trFile.lpSpriteTextures;
    delete [] trFile.lpSpriteSequences;

    trFile.dwNumSpriteSequences = dwNumSpriteSeq;
    trFile.lpSpriteSequences    = lpSpriteSequence;
    trFile.dwNumSpriteTextures  = dwNumSpriteTexture;
    trFile.lpSpriteTextures     = lpSpriteTexture;

    //
    // Calculate the new number of meshes / mesh pointers and reallocate the arrays
    //
    TRListMoveable*     lpListMoveable                  = trFile.lpListMoveable;
    DWORD               dwNumMeshMoveablesWithoutDummy  = 0;
    DWORD               dwNumMeshMoveables              = 0;
    DWORD               dwNumMoveables                  = lpListMoveable->size();

    for(TRListMoveable::iterator theIterator = lpListMoveable->begin(); theIterator != lpListMoveable->end(); theIterator++)
        {
        CTrMvb* lpMvb = *theIterator;

        for(DWORD dwMesh = 0; dwMesh < lpMvb->dwGetNumMeshes(); dwMesh++)
            {
            TR123_MESH* lpMesh  = const_cast<TR123_MESH*>(lpMvb->lpGetMesh(dwMesh));

            if(!lpMesh->boIsDummy) dwNumMeshMoveablesWithoutDummy++;
            }

        dwNumMeshMoveables += lpMvb->dwGetNumMeshes();
        }

    TRListStaticMesh*   lpListStatMsh                   = trFile.lpListStaticMesh;
    DWORD               dwNumStaticMeshesWithoutDummy   = 0;
    DWORD               dwNumStaticMeshes               = lpListStatMsh->size();
    
    for(TRListStaticMesh::iterator theIterator2 = lpListStatMsh->begin(); theIterator2 != lpListStatMsh->end(); theIterator2++)
        {
        CTrStatMsh* lpStatMsh = *theIterator2;

        TR123_MESH* lpMesh      = const_cast<TR123_MESH*>(lpStatMsh->lpGetMesh(0));

        if(!lpMesh->boIsDummy) dwNumStaticMeshesWithoutDummy++;
        }

    DWORD   dwNumMeshes             = 0;
    DWORD   dwNewNumMeshes          = dwNumMeshMoveablesWithoutDummy + dwNumStaticMeshesWithoutDummy;
    DWORD   dwNumMeshPointers       = 0;
    DWORD   dwNewNumMeshPointers    = dwNumMeshMoveables + dwNumStaticMeshes;

    TR123_MESH**    lplpMeshes      = new TR123_MESH* [ dwNewNumMeshes ];
    bitu32*         lpMeshPointers  = new bitu32 [ dwNewNumMeshPointers ];

	for( DWORD dwMesh = 0; dwMesh < trFile.dwNumMeshes; ++dwMesh )
		{
		vReleaseMeshMemory( trFile.lplpMeshes[ dwMesh ] );

		delete trFile.lplpMeshes[ dwMesh ];
		}

    delete [] trFile.lplpMeshes;
    delete [] trFile.lpMeshPointers;
    
    trFile.dwNumMeshes          = dwNewNumMeshes;
    trFile.lplpMeshes           = lplpMeshes;
    trFile.dwNumMeshPointers    = dwNewNumMeshPointers;
    trFile.lpMeshPointers       = lpMeshPointers;

    //
    // Process moveables
    //
    DWORD   dwNewNumAnimations      = 0;
    DWORD   dwNewNumMeshTrees       = 0;
    DWORD   dwNewNumStateChanges    = 0;
    DWORD   dwNewNumAnimDispatches  = 0;
    DWORD   dwNewNumAnimCommands    = 0;
    DWORD   dwSizeOfKeys            = 0;

    // Calculate the new sizes of the different arrays related to moveables
    for(theIterator = lpListMoveable->begin(); theIterator != lpListMoveable->end(); theIterator++)
        {
        CTrMvb*     lpMvb = *theIterator;
        CAnimMgr*   lpAnimMgr = lpMvb->lpGetAnimMgr();

        dwNewNumAnimations += lpAnimMgr->dwGetNumTracks();
        dwNewNumMeshTrees += 4 * (lpMvb->dwGetNumMeshes() - 1);

        for(DWORD dwTrack = 0; dwTrack < lpAnimMgr->dwGetNumTracks(); dwTrack++)
            {
            CTRAnimTrack* lpTrack = static_cast<CTRAnimTrack*>(lpAnimMgr->lpGetTrack(dwTrack));

            dwNewNumStateChanges += lpTrack->dwGetNumStateChanges();
            dwNewNumAnimDispatches += lpTrack->dwGetNumAnimDispatches();
            dwNewNumAnimCommands += lpTrack->dwGetNumParamsAnimCommands();
#ifndef __EXACT_TRANGLE__
            dwSizeOfKeys += lpTrack->dwGetNumKeys() * (3 * 3 + lpMvb->dwGetNumMeshes() * 2);
#else
            dwSizeOfKeys += lpTrack->dwGetNumKeys() * dwGetFrameSize(lpTrack);
#endif
            if(boTR1) dwSizeOfKeys += lpTrack->dwGetNumKeys();
            }

        if (DlgSave) DlgSave->vSetConvert2Progress(++dwProgress);
        }

    // Allocate the new arrays
    TR2_MOVEABLE*       lpMoveables         = new TR2_MOVEABLE [ dwNumMoveables ];
    TR2_ANIMATION*      lpAnimations        = new TR2_ANIMATION [ dwNewNumAnimations ];
    TR2_MESHTREE*       lpMeshTrees         = new TR2_MESHTREE [ dwNewNumMeshTrees ];
    TR2_STATE_CHANGE*   lpStateChanges      = new TR2_STATE_CHANGE [ dwNewNumStateChanges ];
    TR2_ANIM_DISPATCH*  lpAnimDispatches    = new TR2_ANIM_DISPATCH [ dwNewNumAnimDispatches ];
    TR2_ANIM_COMMAND*   lpAnimCommands      = new TR2_ANIM_COMMAND [ dwNewNumAnimCommands ];
    bitu16*             lpKeys              = new bitu16 [ dwSizeOfKeys ];

    ZeroMemory(lpKeys, dwSizeOfKeys * sizeof(bitu16));

    delete [] trFile.lpMoveables;
    delete [] trFile.lpAnimations;
    delete [] trFile.lpMeshTrees;
    delete [] trFile.lpStateChanges;
    delete [] trFile.lpAnimDispatches;
    delete [] trFile.lpAnimCommands;
    delete [] trFile.lpFrames;

    trFile.dwNumMoveables       = dwNumMoveables;
    trFile.lpMoveables          = lpMoveables;
    trFile.dwNumMeshTrees       = dwNewNumMeshTrees;
    trFile.lpMeshTrees          = lpMeshTrees;
    trFile.dwNumAnimations      = dwNewNumAnimations;
    trFile.lpAnimations         = lpAnimations;
    trFile.dwNumStateChanges    = dwNewNumStateChanges;
    trFile.lpStateChanges       = lpStateChanges;
    trFile.dwNumAnimDispatches  = dwNewNumAnimDispatches;
    trFile.lpAnimDispatches     = lpAnimDispatches;
    trFile.dwNumAnimCommands    = dwNewNumAnimCommands;
    trFile.lpAnimCommands       = lpAnimCommands;
    trFile.dwNumFrames          = dwSizeOfKeys;
    trFile.lpFrames             = lpKeys;

    DWORD   dwMoveable      = 0;
    DWORD   dwMeshTree      = 0;
    DWORD   dwAnimation     = 0;
    DWORD   dwStateChange   = 0;
    DWORD   dwAnimDispatch  = 0;
    DWORD   dwAnimCommand   = 0;
    DWORD   dwKey           = 0;
    DWORD   dwFrame         = 0;

    // Build the new arrays
    for(theIterator = lpListMoveable->begin(); theIterator != lpListMoveable->end(); theIterator++)
        {
        CTrMvb*     lpMvb = *theIterator;
        CAnimMgr*   lpAnimMgr = lpMvb->lpGetAnimMgr();

        // Build the TR2_MOVEABLE structure
        lpMoveables[dwMoveable].dwFrameOffset   = dwKey * 2;
        lpMoveables[dwMoveable].dwMeshTree      = dwMeshTree;
        lpMoveables[dwMoveable].dwObjectId      = lpMvb->dwGetObjectId();
        lpMoveables[dwMoveable].wAnimation      = lpMvb->boIsAnimatedByEngine() ? 0xFFFF : (bitu16)dwAnimation;
        lpMoveables[dwMoveable].wNumMeshes      = (bitu16)lpMvb->dwGetNumMeshes();
        lpMoveables[dwMoveable].wStartingMesh   = (bitu16)dwNumMeshPointers;

        DWORD   dwMoveableStartFrame = dwFrame;

        dwMoveable++;

        // Build the TR2_MESHTREE structure
        memcpy(&lpMeshTrees[dwMeshTree], lpMvb->lpGetMeshTree(), 4 * sizeof(TR2_MESHTREE) * (lpMvb->dwGetNumMeshes() - 1));

        dwMeshTree += 4 * (lpMvb->dwGetNumMeshes() - 1);

        // Add the meshes to the mesh list
        for(DWORD dwMesh = 0; dwMesh < lpMvb->dwGetNumMeshes(); dwMesh++)
            {
            TR123_MESH* lpMesh      = const_cast<TR123_MESH*>(lpMvb->lpGetMesh(dwMesh));
            TR123_MESH* lpMeshDst   = new TR123_MESH;

            vCopyMesh(lpMesh, lpMeshDst);

            if(m_TRType < TR3_FILE) CTRFile::vMeshReverseLighting(lpMeshDst);

            if(lpMeshDst->boIsDummy)
                {
                lpMeshPointers[ dwNumMeshPointers ] = 0;
				vReleaseMeshMemory( lpMeshDst );
				delete lpMeshDst;
                }
            else
                {
                lplpMeshes[ dwNumMeshes++ ]         = lpMeshDst;
                lpMeshPointers[ dwNumMeshPointers ] = dwNumMeshes - 1;
                }

            dwNumMeshPointers++;
            }

        DWORD   dwStartAnimation = dwAnimation;

        // Loop through the tracks for this moveable
        for(DWORD dwTrack = 0; dwTrack < lpAnimMgr->dwGetNumTracks(); dwTrack++)
            {
            CTRAnimTrack* lpTrack = static_cast<CTRAnimTrack*>(lpAnimMgr->lpGetTrack(dwTrack));

            short   sTRUnknown;
            long    lTRSpeed;
            float   fTRAcceleration;
            DWORD   dwNextTrackIndex, dwNextTrackFrame;

            lpTrack->vGetPlayParams(sTRUnknown, lTRSpeed, fTRAcceleration);
            lpAnimMgr->vGetNextAnimInfos(dwNextTrackIndex, dwNextTrackFrame, dwTrack);

            short   sAccelHi = 0;
            WORD    wAccelLo = 0;

            if(fTRAcceleration >= 0.0f)
                {
                sAccelHi = (short) fTRAcceleration;
                wAccelLo = (WORD) ((fTRAcceleration - sAccelHi) * 65536.0f);
                }
            else
                {
                float fAccel = floor(fTRAcceleration);

                sAccelHi = (short) fAccel;
                wAccelLo = (WORD) ((fTRAcceleration - fAccel) * 65536.0f);
                }

            // Build the TR_ANIMATION structure
			bitu16	wRealNumAnimCommands;
			bitu8	abUnknown2[8];

			lpTrack->vGetAddParams(wRealNumAnimCommands, abUnknown2);

			WORD	wNumAnimCommand = (WORD)lpTrack->dwGetNumAnimCommands();

			if( m_TRType == TR4_FILE  &&  wNumAnimCommand == 0 ) wNumAnimCommand = wRealNumAnimCommands;

            lpAnimations[dwAnimation].bFrameRate            = (bitu8)lpTrack->dwGetFrameRate();
#ifndef __EXACT_TRANGLE__
            lpAnimations[dwAnimation].bFrameSize            = (bitu8)(3 * 3 + lpMvb->dwGetNumMeshes() * 2);
#else
            lpAnimations[dwAnimation].bFrameSize            = dwGetFrameSize(lpTrack);
#endif
            if(boTR1) lpAnimations[dwAnimation].bFrameSize++;
            lpAnimations[dwAnimation].dwFrameOffset         = dwKey * 2;
            lpAnimations[dwAnimation].sAccelHi              = sAccelHi;
            lpAnimations[dwAnimation].sSpeed                = (bit16)lTRSpeed;
            lpAnimations[dwAnimation].Unknown               = sTRUnknown;
            lpAnimations[dwAnimation].wAccelLo              = wAccelLo;
			memcpy( &lpAnimations[dwAnimation].Unknown2, abUnknown2, sizeof(abUnknown2) );
            lpAnimations[dwAnimation].wAnimCommand          = (bitu16)dwAnimCommand;
            lpAnimations[dwAnimation].wFrameEnd             = (bitu16)(dwFrame + lpTrack->dwGetNumFrames() - 1 - dwMoveableStartFrame);
            lpAnimations[dwAnimation].wFrameStart           = (bitu16)(dwFrame - dwMoveableStartFrame);
            lpAnimations[dwAnimation].wNextAnimation        = (bitu16)(dwStartAnimation + dwNextTrackIndex);
            lpAnimations[dwAnimation].wNextFrame            = (bitu16)dwNextTrackFrame;
            lpAnimations[dwAnimation].wNumAnimCommands      = (bitu16)wNumAnimCommand;
            lpAnimations[dwAnimation].wNumStateChanges      = (bitu16)lpTrack->dwGetNumStateChanges();
            lpAnimations[dwAnimation].wStateChangeOffset    = (bitu16)dwStateChange;
            lpAnimations[dwAnimation].wStateId              = (bitu16)lpTrack->dwGetStateId();

            dwAnimation++;

            // Build the TR_STATE_CHANGE structures
            for(DWORD dwNumStateChange = 0; dwNumStateChange < lpTrack->dwGetNumStateChanges(); dwNumStateChange++)
                {
                const CTRStateChange& StateChange = lpTrack->GetStateChange(dwNumStateChange);

                lpStateChanges[dwStateChange].wAnimDispatch = (bitu16)dwAnimDispatch;
                lpStateChanges[dwStateChange].wNumAnimDispatches = (bitu16)StateChange.dwGetNumAnimDispatch();
                lpStateChanges[dwStateChange].wStateId = (bitu16)StateChange.dwGetStateId();

                dwStateChange++;

                // Build the TR_ANIM_DISPATCH structures
                for(DWORD dwNumAnimDispatch = 0; dwNumAnimDispatch < StateChange.dwGetNumAnimDispatch(); dwNumAnimDispatch++)
                    {
                    DWORD   dwTag;
                    const   CTRAnimDispatch& AnimDispatch = StateChange.GetAnimDispatch(dwNumAnimDispatch, dwTag, false);

                    DWORD dwStartFrame, dwEndFrame;
                    DWORD dwNextAnimation, dwNextFrame;

                    AnimDispatch.vGetFrameRange(dwStartFrame, dwEndFrame);
                    AnimDispatch.vGetNextAnim(dwNextAnimation, dwNextFrame);

                    lpAnimDispatches[dwAnimDispatch].sHigh = (bit16)(dwFrame + dwEndFrame + 1 - dwMoveableStartFrame);
                    lpAnimDispatches[dwAnimDispatch].sLow = (bit16)(dwFrame + dwStartFrame - dwMoveableStartFrame);
                    lpAnimDispatches[dwAnimDispatch].sNextAnimation = (bit16)(dwStartAnimation + dwNextAnimation);
                    lpAnimDispatches[dwAnimDispatch].sNextFrame = (bit16)dwNextFrame;

                    dwAnimDispatch++;
                    }
                }

            // Build the TR_ANIM_COMMAND structures
            for(DWORD dwNumAnimCommand = 0; dwNumAnimCommand < lpTrack->dwGetNumAnimCommands(); dwNumAnimCommand++)
                {
                const CTRAnimCommand& AnimCommand = lpTrack->GetAnimCommand(dwNumAnimCommand);

                lpAnimCommands[dwAnimCommand++] = AnimCommand.sGetCommand();

                bool boFirstParamIsFrame = CTRAnimCommand::boFirstParamIsFrame( lpAnimCommands[dwAnimCommand-1].sValue );

                for(BYTE bParam = 0; bParam < AnimCommand.wGetNumParams(); bParam++)
                    {
                    lpAnimCommands[dwAnimCommand++] = AnimCommand.sGetParam(bParam);
                    if(bParam == 0  &&  boFirstParamIsFrame)
                        {
                        lpAnimCommands[dwAnimCommand-1].sValue += (bit16)(dwFrame - dwMoveableStartFrame);
                        }
                    }
                }

            // Build the keys
            for(DWORD dwNumKey = 0; dwNumKey < lpTrack->dwGetNumKeys(); dwNumKey++)
                {
                CAnimKey*           lpKey = lpTrack->lpGetKey(dwNumKey);
                const CBoundingBox  BB = lpKey->GetBB();

                DWORD dwCurKey = dwKey;

                lpKeys[dwKey++] =   (bit16) BB.GetBBLo()[0];
                lpKeys[dwKey++] =   (bit16) BB.GetBBHi()[0];
                lpKeys[dwKey++] =  -(bit16) BB.GetBBHi()[1];
                lpKeys[dwKey++] =  -(bit16) BB.GetBBLo()[1];
                lpKeys[dwKey++] =  -(bit16) BB.GetBBHi()[2];
                lpKeys[dwKey++] =  -(bit16) BB.GetBBLo()[2];

                for(DWORD dwNumData = 0; dwNumData < lpKey->dwGetNumData(); dwNumData++)
                    {
                    const CAnimData& AnimData = lpKey->GetData(dwNumData);

                    if(dwNumData == 0)
                        {
                        const CVect3D&  vecTrans = AnimData.vecGetTrans();

                        lpKeys[dwKey++] =  (bit16) vecTrans[0];
                        lpKeys[dwKey++] = -(bit16) vecTrans[1];
                        lpKeys[dwKey++] = -(bit16) vecTrans[2];

                        if(boTR1) lpKeys[dwKey++] = (bitu16)lpKey->dwGetNumData();
                        }

                    WORD wAngle1, wAngle2;

                    AnimData.vGetTRAngle(wAngle1, wAngle2);

                    if(boTR1)
                        {
                        WORD wTemp = wAngle1;
                        wAngle1 = wAngle2;
                        wAngle2 = wTemp;
                        }

                    lpKeys[dwKey++] = wAngle1;

#ifndef __EXACT_TRANGLE__
                    lpKeys[dwKey++] = wAngle2;
#else
                    if(wAngle2 != 0x1234) lpKeys[dwKey++] = wAngle2;
#endif
                    }

                dwKey = dwCurKey + lpAnimations[dwAnimation - 1].bFrameSize;
                }

            if(boTR1) lpAnimations[dwAnimation - 1].bFrameSize = 0;

            dwFrame += lpTrack->dwGetNumFrames();
            }

        if (DlgSave) DlgSave->vSetConvert2Progress(++dwProgress);
        }

    // Adjust the NextFrame field for each animation
    for(dwAnimation = 0; dwAnimation < dwNewNumAnimations; dwAnimation++)
        {
        TR2_ANIMATION*   lpAnimation = &lpAnimations[dwAnimation];

        lpAnimation->wNextFrame += lpAnimations[lpAnimation->wNextAnimation].wFrameStart;
        }

    // Adjust the NextFrame field for each anim_dispatch
    TR2_MOVEABLE*   lpMoveable = lpMoveables;

    for(theIterator = lpListMoveable->begin(); theIterator != lpListMoveable->end(); theIterator++, lpMoveable++)
        {
        CTrMvb*     lpMvb = *theIterator;
        CAnimMgr*   lpAnimMgr = lpMvb->lpGetAnimMgr();

        DWORD   dwStartAnimation = lpMoveable->wAnimation;

        for(DWORD dwAnimation = 0; dwAnimation < lpAnimMgr->dwGetNumTracks(); dwAnimation++)
            {
            TR2_ANIMATION* lpAnim = &lpAnimations[dwStartAnimation + dwAnimation];

            for(DWORD dwStateChange = 0; dwStateChange < lpAnim->wNumStateChanges; dwStateChange++)
                {
                TR2_STATE_CHANGE* lpStateChange = &lpStateChanges[lpAnim->wStateChangeOffset + dwStateChange];

                for(DWORD dwAnimDispatch = 0; dwAnimDispatch < lpStateChange->wNumAnimDispatches; dwAnimDispatch++)
                    {
                    TR2_ANIM_DISPATCH* lpAnimDispatch = &lpAnimDispatches[lpStateChange->wAnimDispatch + dwAnimDispatch];

                    if(lpAnimDispatch->sNextAnimation < (bit16)dwNewNumAnimations)
                        lpAnimDispatch->sNextFrame += lpAnimations[lpAnimDispatch->sNextAnimation].wFrameStart;
                    else
                        lpAnimDispatch->sNextFrame += lpAnimations[lpAnimDispatch->sNextAnimation - dwStartAnimation].wFrameStart;
                    }
                }
            }

        delete lpMvb;
        }

    //
    // Process static meshes
    //
    TR2_STATICMESH* lpStaticMeshes  = new TR2_STATICMESH [ dwNumStaticMeshes ];

    DWORD   dwStaticMesh = 0;

    for(theIterator2 = lpListStatMsh->begin(); theIterator2 != lpListStatMsh->end(); theIterator2++)
        {
        CTrStatMsh* lpStatMsh = *theIterator2;

        TR123_MESH* lpMesh      = const_cast<TR123_MESH*>(lpStatMsh->lpGetMesh(0));
        TR123_MESH* lpMeshDst   = new TR123_MESH;

        vCopyMesh(lpMesh, lpMeshDst);

        if(m_TRType < TR3_FILE) CTRFile::vMeshReverseLighting(lpMeshDst);

        if(lpMeshDst->boIsDummy)
            {
            lpMeshPointers[ dwNumMeshPointers ] = 0;
			vReleaseMeshMemory( lpMeshDst );
			delete lpMeshDst;
            }
        else
            {
            lplpMeshes[ dwNumMeshes++ ]         = lpMeshDst;
            lpMeshPointers[ dwNumMeshPointers ] = dwNumMeshes - 1;
            }

        CBoundingBox BBVis = lpStatMsh->GetBBVisibility();
        CBoundingBox BBCol = lpStatMsh->GetBBCollision();

        lpStaticMeshes[ dwStaticMesh ].dwObjectId           = lpStatMsh->dwGetObjectId();
        lpStaticMeshes[ dwStaticMesh ].wFlags               = lpStatMsh->wGetFlags();
        lpStaticMeshes[ dwStaticMesh ].wMesh                = (bitu16)dwNumMeshPointers;
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[0].sMinX = (bit16) (BBVis.GetBBLo())[0];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[0].sMaxY = (bit16)-(BBVis.GetBBLo())[1];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[0].sMaxZ = (bit16)-(BBVis.GetBBLo())[2];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[0].sMaxX = (bit16) (BBVis.GetBBHi())[0];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[0].sMinY = (bit16)-(BBVis.GetBBHi())[1];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[0].sMinZ = (bit16)-(BBVis.GetBBHi())[2];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[1].sMinX = (bit16) (BBCol.GetBBLo())[0];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[1].sMaxY = (bit16)-(BBCol.GetBBLo())[1];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[1].sMaxZ = (bit16)-(BBCol.GetBBLo())[2];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[1].sMaxX = (bit16) (BBCol.GetBBHi())[0];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[1].sMinY = (bit16)-(BBCol.GetBBHi())[1];
        lpStaticMeshes[ dwStaticMesh ].BoundingBox[1].sMinZ = (bit16)-(BBCol.GetBBHi())[2];

        dwStaticMesh++;
        dwNumMeshPointers++;
        
        delete lpStatMsh;

        if (DlgSave) DlgSave->vSetConvert2Progress(++dwProgress);
        }

	_ASSERTE( dwNumMeshes == trFile.dwNumMeshes );
	_ASSERTE( dwNumMeshPointers == trFile.dwNumMeshPointers );

    delete [] trFile.lpStaticMeshes;

    trFile.dwNumStaticMeshes    = dwNumStaticMeshes;
    trFile.lpStaticMeshes       = lpStaticMeshes;

    //
    // Process items
    //
    TRListItem* lpListItem = trFile.lpListItem;
    DWORD       dwNewNumItems = lpListItem->size();

    TR123_ITEM* lpItems = new TR123_ITEM [ dwNewNumItems ];

    delete [] trFile.lpItems;

    trFile.lpItems = lpItems;
    trFile.dwNumItems = dwNewNumItems;

    DWORD   dwItem  = 0;

    for(TRListItem::iterator theIterator4 = lpListItem->begin(); theIterator4 != lpListItem->end(); theIterator4++)
        {
        TR123_ITEM* lpItem = *theIterator4;

        lpItems[dwItem++] = *lpItem;

		delete lpItem;
        }

    if (DlgSave) DlgSave->vSetConvert2Progress(++dwProgress);

    //
    // Process sounds
    //
    trFile.lpSndMgr->vConvertToTR(this);

    //
    // release memory
    //
    lpListMoveable->clear();

	delete lpListMoveable;

    lpListStatMsh->clear();

	delete lpListStatMsh;

    lpListSpriteSeq->clear();

	delete lpListSpriteSeq;

    trFile.lpListItem->clear();

	delete trFile.lpListItem;

	m_trFile.tr123File.lpListUnusedObjText->clear();

    delete m_trFile.tr123File.lpListUnusedObjText;

	delete trFile.lpSndMgr;

    if (DlgSave) DlgSave->vSetConvert2Progress(dwMaxRange);
}


//----------------------------------------------------------------------------
//
//  Function:   vConvertFromTRLevel
//
//  Synopsis:   Convert TR level to internal representation
//
//  Arguments:  [DlgLoad] -- load dialog (to update progress bars)
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTRFile::vConvertFromTRLevel(CDlgLevelLS* DlgLoad)
{
	DWORD dwMaxRange = m_trFile.tr123File.dwNumMeshes + m_trFile.tr123File.dwNumMoveables + 
                       m_trFile.tr123File.dwNumStaticMeshes + m_trFile.tr123File.dwNumSpriteSequences +
                       m_trFile.tr123File.dwNumItems;

    if (DlgLoad) DlgLoad->vSetConvertRange( dwMaxRange );

    DWORD   dwProgress = 0;

    //
    // Build a list of unused object textures
    //
    BYTE*   lpbObjText = new BYTE [ m_trFile.tr123File.dwNumObjectTextures ];

    ZeroMemory(lpbObjText, m_trFile.tr123File.dwNumObjectTextures * sizeof(BYTE) );

    // Flag object textures used by meshes
    for(DWORD dwMesh = 0; dwMesh < m_trFile.tr123File.dwNumMeshes; dwMesh++)
        {
        TR123_MESH* lpMesh = m_trFile.tr123File.lplpMeshes[ dwMesh ];

        for(DWORD dwPoly = 0; dwPoly < (DWORD)lpMesh->sNumTexturedRectangles; dwPoly++)
            {
            lpbObjText[ lpMesh->lpTexturedRectangles[ dwPoly ].wTextureIndex & 0x7FFF ]++;
            }

        for(dwPoly = 0; dwPoly < (DWORD)lpMesh->sNumTexturedTriangles; dwPoly++)
            {
            lpbObjText[ lpMesh->lpTexturedTriangles[ dwPoly ].wTextureIndex & 0x7FFF ]++;
            }
        }

    // Flag object textures used by rooms
    for(DWORD dwNumRoom = 0; dwNumRoom < m_trFile.tr123File.wNumRooms; dwNumRoom++)
        {
        TR123_ROOM*     lpRoom = &m_trFile.tr123File.lpRooms[dwNumRoom];
        TR2_ROOM_DATA*  lpRoomData = &lpRoom->RoomData;

        for(DWORD dwPoly = 0; dwPoly < lpRoomData->wNumRectangles; dwPoly++)
            {
			if( lpRoomData->lpRectangles[ dwPoly ].wTextureIndex == 0xFFFF) lpRoomData->lpRectangles[ dwPoly ].wTextureIndex = 0;
            lpbObjText[ lpRoomData->lpRectangles[ dwPoly ].wTextureIndex & 0x7FFF ]++;
            }

        for(dwPoly = 0; dwPoly < lpRoomData->wNumTriangles; dwPoly++)
            {
			if( lpRoomData->lpTriangles[ dwPoly ].wTextureIndex == 0xFFFF) lpRoomData->lpTriangles[ dwPoly ].wTextureIndex = 0;
            lpbObjText[ lpRoomData->lpTriangles[ dwPoly ].wTextureIndex & 0x7FFF ]++;
            }
        }

    // Flag object textures used by animated textures
    bitu16* lpsAnimatedTextures     = m_trFile.tr123File.lpAnimatedTextures;
    WORD    wNumAnimatedTextures    = *lpsAnimatedTextures++;

    while(wNumAnimatedTextures--)
        {
        bitu16   wNum = *lpsAnimatedTextures++;  wNum++;
        while(wNum--)
            {
            bitu16  wNumObjText = *lpsAnimatedTextures;

            lpbObjText[ wNumObjText ]++;

            lpsAnimatedTextures++;
            }
        }

    // Build the list of non-flagged object textures
    m_trFile.tr123File.lpListUnusedObjText = new TRListObjText;

    for(DWORD dwObjText = 0; dwObjText < m_trFile.tr123File.dwNumObjectTextures; dwObjText++)
        {
        if( lpbObjText[ dwObjText ] ) continue;

        m_trFile.tr123File.lpListUnusedObjText->push_back( m_trFile.tr123File.lpObjectTextures[ dwObjText ] );
        }

    delete [] lpbObjText;

	//
	// TR4 only
	//
	if( GetTRType() == TR4_FILE )
		{
		// replace 0 with 1 in cXCoordinate / cYCoordinate of object textures
		DWORD dwNumRoomObjText = 0;

		TR2_OBJECT_TEXTURE*	lpRoomText = new TR2_OBJECT_TEXTURE[ m_trFile.tr123File.dwNumObjectTextures ];
		DWORD*	lpObjTextMapping = new DWORD [ m_trFile.tr123File.dwNumObjectTextures ];

		memset( lpObjTextMapping, 255, sizeof( DWORD ) * m_trFile.tr123File.dwNumObjectTextures );

		for(DWORD dwObjText = 0; dwObjText < m_trFile.tr123File.dwNumObjectTextures; dwObjText++)
			{
			TR2_OBJECT_TEXTURE*	lpText = &m_trFile.tr123File.lpObjectTextures[ dwObjText ];

			if( lpText->Unknown & 0x8000 )
				{
				lpObjTextMapping[ dwObjText ] = dwNumRoomObjText;
				lpRoomText[ dwNumRoomObjText++ ] = *lpText;
				}

			for( BYTE bI = 0; bI < 4; ++bI )
				{
				if( lpText->Vertices[bI].cXCoordinate == 0 ) lpText->Vertices[bI].cXCoordinate = 1;
				if( lpText->Vertices[bI].cYCoordinate == 0 ) lpText->Vertices[bI].cYCoordinate = 1;
				}
			}

		// save the object textures of the room / animated textures for later reuse when saving the level
		m_trFile.tr123File.dwNumRoomObjText = dwNumRoomObjText;
		m_trFile.tr123File.lpRoomObjText = lpRoomText;

		for(dwObjText = 0; dwObjText < dwNumRoomObjText; dwObjText++)
			{
			TR2_OBJECT_TEXTURE*	lpText = &lpRoomText[ dwObjText ];

			if( ( lpText->wTile & 0x7FFF ) >= m_trFile.tr123File.wNumTextNoBump + m_trFile.tr123File.wNumTextInhab )
				{
				lpText->wTile = lpText->wTile - m_trFile.tr123File.wNumTextInhab + 0x4000;
				}
			}

		for(DWORD dwNumRoom = 0; dwNumRoom < m_trFile.tr123File.wNumRooms; dwNumRoom++)
			{
			TR123_ROOM*     lpRoom = &m_trFile.tr123File.lpRooms[dwNumRoom];
			TR2_ROOM_DATA*  lpRoomData = &lpRoom->RoomData;

			for(DWORD dwPoly = 0; dwPoly < lpRoomData->wNumRectangles; dwPoly++)
				{
				lpRoomData->lpRectangles[ dwPoly ].wTextureIndex = (bitu16)(( lpRoomData->lpRectangles[ dwPoly ].wTextureIndex & 0x8000 ) + lpObjTextMapping[ lpRoomData->lpRectangles[ dwPoly ].wTextureIndex & 0x7FFF ]);
				}

			for(dwPoly = 0; dwPoly < lpRoomData->wNumTriangles; dwPoly++)
				{
				lpRoomData->lpTriangles[ dwPoly ].wTextureIndex = (bitu16)(( lpRoomData->lpTriangles[ dwPoly ].wTextureIndex & 0x8000 ) + lpObjTextMapping[ lpRoomData->lpTriangles[ dwPoly ].wTextureIndex & 0x7FFF ]);
				}
			}

		bitu16* lpsAnimatedTextures     = m_trFile.tr123File.lpAnimatedTextures;
		WORD    wNumAnimatedTextures    = *lpsAnimatedTextures++;

		while(wNumAnimatedTextures--)
			{
			bitu16   wNum = *lpsAnimatedTextures++;  wNum++;
			while(wNum--)
				{
				*lpsAnimatedTextures = (bitu16)lpObjTextMapping[ *lpsAnimatedTextures ];

				_ASSERTE( *lpsAnimatedTextures < dwNumRoomObjText );

				lpsAnimatedTextures++;
				}
			}

		delete [] lpObjTextMapping;
		}

    //
    // Set the "intensity alpha" flag for meshes
    //
    for(DWORD dwNumMesh = 0; dwNumMesh < m_trFile.tr123File.dwNumMeshes; dwNumMesh++)
        {
        TR123_MESH* lpMesh = m_trFile.tr123File.lplpMeshes[dwNumMesh];

        lpMesh->boIntensityAlpha = CTRFile::boIsMeshIntensityAlpha(lpMesh,m_trFile.tr123File.lpObjectTextures);

        if (DlgLoad) DlgLoad->vSetConvertProgress(++dwProgress);
        }

    const CResMgr&		ResMgr		= ( (CTRViewerApp*) AfxGetApp() )->GetResMgr();
	BYTE				bTRResId	= ResMgr.bGetResId( GetTRType() );

    //
    // Process moveables
    //
    m_trFile.tr123File.lpListMoveable = new TRListMoveable;

    for(DWORD dwNumMoveable = 0; dwNumMoveable < m_trFile.tr123File.dwNumMoveables; dwNumMoveable++)
        {
        TR2_MOVEABLE* lpMoveable = &m_trFile.tr123File.lpMoveables[dwNumMoveable];

		CTrMvb*	lpMvb = new CTrMvb(lpMoveable,this);

		CResMoveable*	lpMoveableInfo = ResMgr.lpGetMoveableInfo( bTRResId, lpszGetShortLevelName(), lpMoveable->dwObjectId );

		lpMvb->vSetResMgr( lpMoveableInfo );

        (*m_trFile.tr123File.lpListMoveable).push_back( lpMvb );

        if (DlgLoad) DlgLoad->vSetConvertProgress(++dwProgress);
        }

    //
    // Process static meshes
    //
    m_trFile.tr123File.lpListStaticMesh = new TRListStaticMesh;

    for(DWORD dwNumStaticMesh = 0; dwNumStaticMesh < m_trFile.tr123File.dwNumStaticMeshes; dwNumStaticMesh++)
        {
        TR2_STATICMESH* lpStatMesh = &m_trFile.tr123File.lpStaticMeshes[dwNumStaticMesh];

        (*m_trFile.tr123File.lpListStaticMesh).push_back(new CTrStatMsh(lpStatMesh,this));

        if (DlgLoad) DlgLoad->vSetConvertProgress(++dwProgress);
        }

    //
    // Process sprite sequences 
    //
    m_trFile.tr123File.lpListSpriteSeq = new TRListSpriteSeq;

    for(DWORD dwNumSprSeq = 0; dwNumSprSeq < m_trFile.tr123File.dwNumSpriteSequences; dwNumSprSeq++)
        {
        TR2_SPRITE_SEQUENCE* lpSprSeq= &m_trFile.tr123File.lpSpriteSequences[dwNumSprSeq];

        (*m_trFile.tr123File.lpListSpriteSeq).push_back(new CTrLstSpr(lpSprSeq,this));

        if (DlgLoad) DlgLoad->vSetConvertProgress(++dwProgress);
        }

    //
    // Process items
    //
    m_trFile.tr123File.lpListItem = new TRListItem;

    for(DWORD dwNumItem = 0; dwNumItem < m_trFile.tr123File.dwNumItems; dwNumItem++)
        {
        TR123_ITEM* lpItem = &m_trFile.tr123File.lpItems[dwNumItem];
        
        TR123_ITEM* lpItem2 = new TR123_ITEM;

        *lpItem2 = *lpItem;

        (*m_trFile.tr123File.lpListItem).push_back(lpItem2);

        if (DlgLoad) DlgLoad->vSetConvertProgress(++dwProgress);
        }

    //
    // Process sounds
    //
    m_trFile.tr123File.lpSndMgr = new CTrSndMgr(this);

    if (DlgLoad) DlgLoad->vSetConvertProgress( dwMaxRange );
}
