// TlboxSpriteMain.cpp : implementation file
//

#include "stdafx.h"

#include "Resource.h"
#include "ChildFrm.h"
#include "HandleBase.h"
#include "HandleSprite.h"
#include "EditBase.h"
#include "EditSprite.h"

#include "ToolboxBase.h"
#include "ToolboxItfBase.h"
#include "ToolboxSheet.h"

#include "TlboxSpriteItf.h"
#include "TlboxSpriteMain.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CTlboxSpriteMain

IMPLEMENT_DYNCREATE(CTlboxSpriteMain, CPropertyPage)

CTlboxSpriteMain::CTlboxSpriteMain( CToolboxBase* lpParent ) : CToolboxItfBase(CTlboxSpriteMain::IDD, lpParent)
{
	//{{AFX_DATA_INIT(CTlboxSpriteMain)
    m_dwAnimSpeed       = 20;
	m_cstrSprSeqName = _T("");
	m_boShowBorder = FALSE;
	m_fListWidth = 256.0f;
	m_cstrObjId = _T("");
	//}}AFX_DATA_INIT
    m_boTypeAnimated    = true;
}

CTlboxSpriteMain::~CTlboxSpriteMain()
{
}

void CTlboxSpriteMain::DoDataExchange(CDataExchange* pDX)
{
	CPropertyPage::DoDataExchange(pDX);
	//{{AFX_DATA_MAP(CTlboxSpriteMain)
	DDX_Control(pDX, IDC_SPR_SPEED, m_SliderSpeed);
	DDX_Text(pDX, IDC_SPR_SPEEDVALUE, m_dwAnimSpeed);
	DDX_Text(pDX, IDC_SPR_NAME, m_cstrSprSeqName);
	DDX_Check(pDX, IDC_SPR_SHOWBORDER, m_boShowBorder);
	DDX_Text(pDX, IDC_SPR_LISTWIDTH, m_fListWidth);
	DDV_MinMaxFloat(pDX, m_fListWidth, 0.f, 1.e+006f);
	DDX_Text(pDX, IDC_SPRITE_OBJID, m_cstrObjId);
	//}}AFX_DATA_MAP
}


BEGIN_MESSAGE_MAP(CTlboxSpriteMain, CPropertyPage)
	//{{AFX_MSG_MAP(CTlboxSpriteMain)
	ON_WM_HSCROLL()
	ON_BN_CLICKED(IDC_SPR_SHOWBORDER, OnSprShowborder)
	ON_BN_CLICKED(IDC_SPR_TYPEANIM, OnSprTypeanim)
	ON_BN_CLICKED(IDC_SPR_TYPELIST, OnSprTypelist)
	ON_EN_KILLFOCUS(IDC_SPR_LISTWIDTH, OnKillfocusSprListwidth)
	ON_EN_KILLFOCUS(IDC_SPRITE_OBJID, OnKillfocusSpriteObjid)
	//}}AFX_MSG_MAP
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CTlboxSpriteMain message handlers

void CTlboxSpriteMain::vInit( bool boNewSprite )
{
	GET_TLBOXHANDLER( CHandleSprite );
	GET_TLBOXEDIT( CEditSprite );

	// Fill the controls of the toolbox
    m_SliderSpeed.SetRange(0,100);

    m_cstrSprSeqName = lpHandler ? lpHandler->lpszGetTitle() : _T("");

    if(m_boTypeAnimated)
        {
        ((CButton*)GetDlgItem(IDC_SPR_TYPEANIM))->SetCheck(1);
        ((CButton*)GetDlgItem(IDC_SPR_TYPELIST))->SetCheck(0);
        }
    else
        {
        ((CButton*)GetDlgItem(IDC_SPR_TYPEANIM))->SetCheck(0);
        ((CButton*)GetDlgItem(IDC_SPR_TYPELIST))->SetCheck(1);
        }

    CTrLstSpr*	lpSprSeq = lpHandler ? lpHandler->GetSpr() : NULL;

    if(lpSprSeq)
        {
        CString s;

        s.Format(_T(" (%ld sprites)"),lpSprSeq->dwGetNumSprites());

        m_cstrSprSeqName += s;

        s.Format(_T("%ld"),lpSprSeq->dwGetObjectId());

        m_cstrObjId = s;
        }

	// Reset some values and redisplay the object view if we deal with a new sprite sequence
    if(boNewSprite)
        {
        m_dwAnimSpeed = 20;
        m_SliderSpeed.SetPos(m_dwAnimSpeed);

        if(lpEdit)
            {
            lpEdit->vSetSprSeqShowBorder(m_boShowBorder ? true : false);
            lpEdit->vSetSprSeqOverallSpeed((float)m_dwAnimSpeed);
            lpEdit->vSetSprSeqTypeAnimated(m_boTypeAnimated);
            lpEdit->vSetSprSeqListWidth(m_fListWidth);
            }
        }

    UpdateData(FALSE);
}


void CTlboxSpriteMain::OnHScroll(UINT nSBCode, UINT nPos, CScrollBar* pScrollBar) 
{
    CString s;

    int iCtrlId = pScrollBar->GetDlgCtrlID();

    switch(nSBCode)
        {
        case TB_THUMBPOSITION:
        case TB_PAGEDOWN:
        case TB_PAGEUP:
        case TB_THUMBTRACK:
            {
            CSliderCtrl* lpSlider = reinterpret_cast<CSliderCtrl*>(pScrollBar);

            int nPos = lpSlider->GetPos();

            switch(iCtrlId)
                {
                case IDC_SPR_SPEED:
                    {
					// Handler the animation speed slider
					GET_TLBOXEDIT(CEditSprite);

                    m_dwAnimSpeed = nPos;

                    lpEdit->vSetSprSeqOverallSpeed((float)m_dwAnimSpeed);

                    s.Format(_T("%ld"),m_dwAnimSpeed);

                    CWnd* lpWnd = GetDlgItem(IDC_SPR_SPEEDVALUE);

                    lpWnd->SetWindowText(s);

                    break;
                    }
                }
            break;
            }
        }

	CPropertyPage::OnHScroll(nSBCode, nPos, pScrollBar);
}


void CTlboxSpriteMain::OnSprShowborder() 
{
	UpdateData(TRUE);
	
	GET_TLBOXEDIT(CEditSprite);

    if(lpEdit) lpEdit->vSetSprSeqShowBorder(m_boShowBorder ? true : false);
}


void CTlboxSpriteMain::OnSprTypeanim() 
{
    UpdateData(TRUE);	

    m_boTypeAnimated = ((CButton*)GetDlgItem(IDC_SPR_TYPEANIM))->GetCheck() == 1;

	GET_TLBOXEDIT(CEditSprite);

    if(lpEdit) lpEdit->vSetSprSeqTypeAnimated(m_boTypeAnimated);
}


void CTlboxSpriteMain::OnSprTypelist() 
{
    UpdateData(TRUE);	
	
    m_boTypeAnimated = ((CButton*)GetDlgItem(IDC_SPR_TYPEANIM))->GetCheck() == 1;

	GET_TLBOXEDIT(CEditSprite);

    if(lpEdit) lpEdit->vSetSprSeqTypeAnimated(m_boTypeAnimated);
}


void CTlboxSpriteMain::OnKillfocusSprListwidth() 
{
    UpdateData(TRUE);
    
	GET_TLBOXEDIT(CEditSprite);

    if(lpEdit) lpEdit->vSetSprSeqListWidth(m_fListWidth);
}


void CTlboxSpriteMain::OnKillfocusSpriteObjid() 
{
	GET_TLBOXHANDLER(CHandleSprite);

    CTrLstSpr* lpSprSeq = lpHandler ? lpHandler->GetSpr() : NULL;

    if(!lpSprSeq) return;
	
    UpdateData(TRUE);

    lpSprSeq->vSetObjectId(_ttol(m_cstrObjId));
}


void	CTlboxSpriteMain::vSetAnimationType( bool boIsAnimated )
{
    m_boTypeAnimated = boIsAnimated;

    if(m_boTypeAnimated)
        {
        ((CButton*)GetDlgItem(IDC_SPR_TYPEANIM))->SetCheck(1);
        ((CButton*)GetDlgItem(IDC_SPR_TYPELIST))->SetCheck(0);
        }
    else
        {
        ((CButton*)GetDlgItem(IDC_SPR_TYPEANIM))->SetCheck(0);
        ((CButton*)GetDlgItem(IDC_SPR_TYPELIST))->SetCheck(1);
        }
	
	GET_TLBOXEDIT(CEditSprite);

    if(lpEdit) lpEdit->vSetSprSeqTypeAnimated(m_boTypeAnimated);
}
