// TlboxMeshSheet.cpp : implementation file
//

#include "stdafx.h"
#include "trviewer.h"

#include "ToolboxBase.h"
#include "ToolboxItfBase.h"
#include "ToolboxSheet.h"

#include "EditBase.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#undef THIS_FILE
static char THIS_FILE[] = __FILE__;
#endif

/////////////////////////////////////////////////////////////////////////////
// CToolboxSheet

IMPLEMENT_DYNAMIC(CToolboxSheet, CPropertySheet)

CToolboxSheet::CToolboxSheet(CWnd* pParentWnd)
	: CPropertySheet(AFX_IDS_APP_TITLE, pParentWnd)
{
	m_lpVisiblePage = NULL;
}


CToolboxSheet::~CToolboxSheet()
{
	for( LstPropPageIter theIter = m_LstPages.begin(); theIter != m_LstPages.end(); ++theIter )
		{
		CToolboxItfBase* lpPage = static_cast< CToolboxItfBase* >( *theIter );

		delete lpPage;
		}

	m_LstPages.clear();
}


BEGIN_MESSAGE_MAP(CToolboxSheet, CPropertySheet)
	//{{AFX_MSG_MAP(CToolboxSheet)
		// NOTE - the ClassWizard will add and remove mapping macros here.
	//}}AFX_MSG_MAP
	ON_MESSAGE(WM_RESIZEPAGE, OnResizePage) 
END_MESSAGE_MAP()

/////////////////////////////////////////////////////////////////////////////
// CToolboxSheet message handlers

void CToolboxSheet::vAddPages( DWORD dwNumPages, CPropertyPage** lplpLstPages )
{
	for( DWORD dwI = 0; dwI < dwNumPages; ++dwI )
		{
		m_LstPages.push_back( lplpLstPages[ dwI ] );

		AddPage( lplpLstPages[ dwI ] );
		}
}


void CToolboxSheet::Resize()
{
	// Find parent
	CWnd* pParent;
	CRect rectParent;

	pParent = GetParent();
	if (pParent == NULL)
		{
		return;
		}

	// Get parents client area
	pParent->GetClientRect(&rectParent);

	// Resize the sheet
	// First find relative change
	CSize sizeRelChange;
	CRect rectWindow;

	GetWindowRect(&rectWindow); 
	ScreenToClient(&rectWindow);
	sizeRelChange.cx = rectWindow.Width() - rectParent.Width();
	sizeRelChange.cy = rectWindow.Height() - rectParent.Height();

	rectWindow.right -= sizeRelChange.cx;
	rectWindow.bottom -= sizeRelChange.cy;
	// Then resize the sheet
	MoveWindow(&rectWindow);

	// Resize the CTabCtrl
	CTabCtrl* pTab = GetTabControl();
	ASSERT(pTab);
	pTab->GetWindowRect(&rectWindow);
	ScreenToClient(&rectWindow);
	rectWindow.right -= sizeRelChange.cx;
	rectWindow.bottom -= sizeRelChange.cy;

	pTab->MoveWindow(&rectWindow);

	// Resize the active page
	CPropertyPage* pPage = GetActivePage();
	ASSERT(pPage);
	// Store page size in m_rectPage
	pPage->GetWindowRect(&m_rectPage);
	ScreenToClient(&m_rectPage);
	m_rectPage.right -= sizeRelChange.cx;
	m_rectPage.bottom -= sizeRelChange.cy;

	pPage->MoveWindow(&m_rectPage);
}


BOOL CToolboxSheet::OnInitDialog() 
{
	BOOL bResult = CPropertySheet::OnInitDialog();
	
	Resize();
	
	return bResult;
}


LONG CToolboxSheet::OnResizePage(UINT, LONG)
	{
	// Resize the page using m_rectPage, which was set in
	// Resize().
	CPropertyPage* pPage = GetActivePage();
	ASSERT(pPage);
	pPage->MoveWindow(&m_rectPage);

	return 0;
	}

BOOL CToolboxSheet::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult) 
{
	NMHDR* pNMHDR = (LPNMHDR) lParam;
	
	// The sheet resizes the page whenever it is activated
	// so we need to resize it to what we want
	if (pNMHDR->code == TCN_SELCHANGE)
		// user-defined message needs to be posted, not sent, because
		// page must be resized after TCN_SELCHANGE has been processed.
		PostMessage(WM_RESIZEPAGE);

	return CPropertySheet::OnNotify(wParam, lParam, pResult);
}


void CToolboxSheet::vInitPages( bool boNewObject )
{
	for( LstPropPageIter theIter = m_LstPages.begin(); theIter != m_LstPages.end(); ++theIter )
		{
		CToolboxItfBase* lpPage = static_cast< CToolboxItfBase* >( *theIter );

		SetActivePage( lpPage );

		if (lpPage->lpGetToolboxBase())
			{
			CEditBase*		pEditBase = lpPage->lpGetToolboxBase()->lpGetEditer();
			CEditGfxBase*	pEditGfx = dynamic_cast<CEditGfxBase*>(pEditBase);

			if (pEditGfx != 0L) pEditGfx->vSetTextToDisplay("");
			}

		lpPage->vInit( boNewObject );
		}
}


void CToolboxSheet::vInitActivatedPage( CToolboxItfBase* lpPage )
{
	if( lpPage )
		{
		m_lpVisiblePage = lpPage;
		lpPage->vInit( false );
		}
}
