#include "stdafx.h"

#include "AnimData.h"
#include "AnimKey.h"
#include "AnimTrack.h"
#include "TRAnimTrack.h"
#include "AnimMgr.h"

#include "ResMgr.h"


//----------------------------------------------------------------------------
//
//  Function:   vInitialize
//
//  Synopsis:   Reset the members of the class
//
//  Arguments:  -
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrMvb::vInitialize(bool boInitAnimations /*= true*/)
{
    if(boInitAnimations)
        {
        m_lpMeshTree	= NULL;
        m_lpAnimMgr		= NULL;
        m_lpSndMgr		= NULL;

		m_dwEdtBodyMvbId	= -1;
		m_dwEdtAssocMvbId	= -1;
		m_boShowRealMeshes	= false;
		m_boShowAssocMvb	= false;
        }

    m_boAnimatedByEngine    = false;
	m_dwStartingMesh		= -1;
	m_lpResMgr				= NULL;
}


//----------------------------------------------------------------------------
//
//  Function:   vRelease
//
//  Synopsis:   Release memory / Reset the members of the class
//
//  Arguments:  [boReleaseParent] -- true to release the parent class
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrMvb::vRelease(bool boReleaseParent /*= true*/)
{
    vRelease2(boReleaseParent);
}


void    CTrMvb::vRelease2(bool boReleaseParent /*= true*/, bool boReleaseMeshes /*= true*/, bool boReleaseAnimations /*= true*/)
{
    if(boReleaseAnimations)
        {
        delete []   m_lpMeshTree;
        delete      m_lpAnimMgr;
        delete      m_lpSndMgr;
        }

	delete m_lpResMgr;

    vInitialize(boReleaseAnimations);

    if(boReleaseParent  &&  boReleaseMeshes) CTrLstMsh::vRelease(true);
}


//----------------------------------------------------------------------------
//
//  Function:   vStealData
//
//  Synopsis:   Transfer data from an object to the current one: after the
//              operation, the source object has lost his data member !
//
//  Arguments:  [obj] -- object to get the data from
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrMvb::vStealData(CTrMvb& obj,bool boStealMeshes /*= true*/, bool boStealAnimations /*= true*/)
{
    if(boStealMeshes)
        {
        CTrLstMsh::vStealData(obj);
        }

    if(boStealAnimations)
        {
        m_lpMeshTree		= obj.m_lpMeshTree;
        m_lpAnimMgr			= obj.m_lpAnimMgr;
        m_lpSndMgr			= obj.m_lpSndMgr;

		m_dwEdtBodyMvbId	= obj.m_dwEdtBodyMvbId;
		m_dwEdtAssocMvbId	= obj.m_dwEdtAssocMvbId;
		m_boShowRealMeshes	= obj.m_boShowRealMeshes;
		m_boShowAssocMvb	= obj.m_boShowAssocMvb;
        }

    m_boAnimatedByEngine    = obj.m_boAnimatedByEngine;
	m_dwStartingMesh		= obj.m_dwStartingMesh;
	m_lpResMgr				= obj.m_lpResMgr;

    obj.vInitialize(boStealAnimations);
}


//----------------------------------------------------------------------------
//
//  Function:   Constructor of class CTrMvb
//
//  Synopsis:   Construct a CTrMvb from a TR moveable
//
//  Arguments:  [lpMoveable] -- moveable to convert
//
//  Return:     -
//
//----------------------------------------------------------------------------
CTrMvb::CTrMvb(TR2_MOVEABLE* lpMoveable, CTRFile* lpTRFile) : CTrLstMsh(VERSION_TRMOVEABLE,TrObj_Type_Moveable,(enumTrVersion)lpTRFile->GetTRType())
{
    vInitialize();

    // Construct CTrMvb
    vConstruct(lpTRFile,lpMoveable);
}


//----------------------------------------------------------------------------
//
//  Function:   Constructor of class CTrMvb
//
//  Synopsis:   Construct a CTrMvb from a block in memory
//
//  Arguments:  [lpMVB] -- memory block
//              [boDeleteData] -- if true, delete the lpMVB pointer at the end
//
//  Return:     -
//
//----------------------------------------------------------------------------
CTrMvb::CTrMvb(BYTE* lpMVB,bool boDeleteData) throw(DWORD) : CTrLstMsh(lpMVB)
{
    vInitialize();

    vSetVersion(VERSION_TRMOVEABLE);
    vSetTRType(TrObj_Type_Moveable);
    vSetTRVersion(Tr4_File);

    if(lpMVB)
        {
        if(GetVersion() != VERSION_TRMOVEABLE) throw((DWORD)__LOADERR_MISMATCHVERSION__);

        BYTE* lpbData = lpMVB + CTrLstMsh::dwGetDataSize();

        vConstruct(lpbData);

        if(boDeleteData) delete [] lpMVB;
        }
    else
        {
        m_lpAnimMgr = new CAnimMgr();
        m_lpSndMgr  = new CTrSndMgr();
        }
}


//----------------------------------------------------------------------------
//
//  Function:   dwGetDataSize
//
//  Synopsis:   Get the size of the CTrMvb when put in a continuous block
//              of memory
//
//  Arguments:  -
//
//  Return:     size
//
//----------------------------------------------------------------------------
DWORD   CTrMvb::dwGetDataSize()
{
    DWORD   dwSize =    CTrLstMsh::dwGetDataSize() +
                        4 * (m_dwNumMeshes - 1) * sizeof(TR2_MESHTREE) +    /* mesh tree */
                        sizeof(DWORD);                                      /* num tracks */

    DWORD   dwNumTracks = m_lpAnimMgr->dwGetNumTracks();

    for(DWORD dwTrack = 0; dwTrack < dwNumTracks; dwTrack++)
        {
        // Params of the track
        CTRAnimTrack* lpTrack = static_cast<CTRAnimTrack*>(m_lpAnimMgr->lpGetTrack(dwTrack));

        dwSize +=   sizeof(WORD) +                                          /* num frames */
                    sizeof(WORD) +                                          /* frame rate */
                    sizeof(WORD) +                                          /* next anim */
                    sizeof(WORD) +                                          /* next frame */
                    sizeof(WORD) +                                          /* state id */
                    sizeof(WORD) +                                          /* unknown */
                    sizeof(BYTE) * 8 +                                      /* unknown2 */
                    sizeof(long) +                                          /* speed */
                    sizeof(float) +                                         /* acceleration */
                    sizeof(DWORD) +                                         /* num keys */
                    sizeof(DWORD) +                                         /* num state changes */
                    sizeof(DWORD) +                                         /* real num anim commands */
                    sizeof(DWORD);                                          /* num anim commands */

        // Animation keys
        DWORD dwNumKeys = lpTrack->dwGetNumKeys();

        dwSize +=   dwNumKeys * (
                    sizeof(float) * 3 * 2 +                                 /* key bounding box */
                    sizeof(float) * 3 +                                     /* translation of the root mesh */
                    sizeof(float) * 4 * m_dwNumMeshes +                     /* quaternions for each mesh */
                    sizeof(WORD) * 2 * m_dwNumMeshes);                      /* TR angles for each mesh */

        // State changes & anim dispatch
        DWORD dwNumStateChanges = lpTrack->dwGetNumStateChanges();

        for(DWORD dwStateChange = 0; dwStateChange < dwNumStateChanges; dwStateChange++)
            {
            const CTRStateChange& StateChange = lpTrack->GetStateChange(dwStateChange);

            dwSize +=   sizeof(WORD) +                                      /* state id */
                        sizeof(WORD);                                       /* num anim dispatch */

            // Anim dispatches
            DWORD dwNumAnimDispatch = StateChange.dwGetNumAnimDispatch();

            dwSize +=   dwNumAnimDispatch * (
                        sizeof(WORD) +                                      /* start frame */
                        sizeof(WORD) +                                      /* end frame */
                        sizeof(WORD) +                                      /* next anim */
                        sizeof(WORD));                                      /* next frame */
            }

        // Animation commands
        DWORD dwNumAnimCmde = lpTrack->dwGetNumAnimCommands();

        for(DWORD dwAnimCmde = 0; dwAnimCmde < dwNumAnimCmde; dwAnimCmde++)
            {
            const CTRAnimCommand& AnimCmde = lpTrack->GetAnimCommand(dwAnimCmde);

            dwSize +=   sizeof(bit16) +                                     /* command */
                        sizeof(WORD) +                                      /* num params */
                        AnimCmde.wGetNumParams() * sizeof(bit16);           /* parameters */
            }
        }

    dwSize += m_lpSndMgr->dwGetDataSize();

    return  dwSize;
}


//----------------------------------------------------------------------------
//
//  Function:   lpbGetData
//
//  Synopsis:   Convert a CTrMvb to a continuous block of memory
//
//  Arguments:  [dwDataSize] -- on return, hold the size of the block
//
//  Return:     pointer to the memory block
//
//  Caution:    you must make a delete [] on the pointer returned when you
//              have finished working with it !
//----------------------------------------------------------------------------
BYTE*   CTrMvb::lpbGetData(DWORD& dwDataSize)
{
    BYTE*   lpbData = CTrLstMsh::lpbGetData(dwDataSize);
    DWORD   dwTrLstMshDataSize = CTrLstMsh::dwGetDataSize();
    BYTE*   lpbDataSave = lpbData;

    lpbData += dwTrLstMshDataSize;

    // Meshes hierarchy
    memcpy(lpbData, m_lpMeshTree, 4 * (m_dwNumMeshes - 1) * sizeof(TR2_MESHTREE));

    lpbData += 4 * (m_dwNumMeshes - 1) * sizeof(TR2_MESHTREE);

    //
    // Animations data
    //
    DWORD   dwNumTracks = m_lpAnimMgr->dwGetNumTracks();

    *(DWORD*)lpbData = dwNumTracks;                         lpbData += sizeof(DWORD);

    for(DWORD dwTrack = 0; dwTrack < dwNumTracks; dwTrack++)
        {
        CTRAnimTrack* lpTrack = static_cast<CTRAnimTrack*>(m_lpAnimMgr->lpGetTrack(dwTrack));

        // Params of the track
        *(WORD*)lpbData = (WORD)lpTrack->dwGetNumFrames();        lpbData += sizeof(WORD);
        *(WORD*)lpbData = (WORD)lpTrack->dwGetFrameRate();        lpbData += sizeof(WORD);
        
        DWORD   dwNextTrackIndex, dwNextTrackFrame;

        m_lpAnimMgr->vGetNextAnimInfos(dwNextTrackIndex,dwNextTrackFrame,dwTrack);

        *(WORD*)lpbData = (WORD)dwNextTrackIndex;                 lpbData += sizeof(WORD);
        *(WORD*)lpbData = (WORD)dwNextTrackFrame;                 lpbData += sizeof(WORD);
        *(WORD*)lpbData = (WORD)lpTrack->dwGetStateId();          lpbData += sizeof(WORD);

        short   sTRUnknown;
        long    lTRSpeed;
        float   fTRAcceleration;
		bitu16	wRealNumAnimCommands;
		bitu8	abUnknown2[8];

        lpTrack->vGetPlayParams(sTRUnknown, lTRSpeed, fTRAcceleration);
        lpTrack->vGetAddParams(wRealNumAnimCommands, abUnknown2);

        *(short*)lpbData    = sTRUnknown;                   lpbData += sizeof(short);
        *(long*)lpbData     = lTRSpeed;                     lpbData += sizeof(long);
        *(float*)lpbData    = fTRAcceleration;              lpbData += sizeof(float);
        memcpy(lpbData, abUnknown2, sizeof(abUnknown2));	lpbData += sizeof(abUnknown2);
		*(DWORD*)lpbData	= wRealNumAnimCommands;			lpbData += sizeof(DWORD);

        // Animation keys
        DWORD dwNumKeys = lpTrack->dwGetNumKeys();

        *(DWORD*)lpbData = dwNumKeys;                       lpbData += sizeof(DWORD);

        for(DWORD dwKey = 0; dwKey < dwNumKeys; dwKey++)
            {
            CAnimKey*   lpKey = lpTrack->lpGetKey(dwKey);

            // Key bounding box
            const CBoundingBox& BB = lpKey->GetBB();

            const CVect3D&  vecBBLo  = BB.GetBBLo();
            const CVect3D&  vecBBHi  = BB.GetBBHi();

            for(DWORD dwJ = 0; dwJ < 3; dwJ++)
                {
                *(float*)lpbData = (float)vecBBLo[dwJ];        lpbData += sizeof(float);
                }

            for(dwJ = 0; dwJ < 3; dwJ++)
                {
                *(float*)lpbData = (float)vecBBHi[dwJ];        lpbData += sizeof(float);
                }

            // Key translation & meshes quaternions
            DWORD dwNumData = lpKey->dwGetNumData();

            for(DWORD dwData = 0; dwData < m_dwNumMeshes; dwData++)
                {
                const CAnimData& Data = lpKey->GetData(dwData);
                
                const CVect3D&      vecTrans = Data.vecGetTrans();
                const CQuaternion&  quatRot  = Data.quatGetQuaternion();

                WORD  wTRAngle1, wTRAngle2;

                Data.vGetTRAngle(wTRAngle1, wTRAngle2);

                // save only the translation for the root mesh
                if(dwData == 0)
                    {
                    for(DWORD dwJ = 0; dwJ < 3; dwJ++)
                        {
                        *(float*)lpbData = (float)vecTrans[dwJ];    lpbData += sizeof(float);
                        }
                    }

                // Save the quaternion
                CVect3D&    vecQuat = const_cast<CQuaternion&>(quatRot).v3GetVector();
                float       fScalar = (float)const_cast<CQuaternion&>(quatRot).fGetScalar();

                for(DWORD dwJ = 0; dwJ < 3; dwJ++)
                    {
                    *(float*)lpbData = (float)vecQuat[dwJ];     lpbData += sizeof(float);
                    }

                *(float*)lpbData = fScalar;                     lpbData += sizeof(float);

                // Save the TR angles
                *(WORD*)lpbData = wTRAngle1;                    lpbData += sizeof(WORD);
                *(WORD*)lpbData = wTRAngle2;                    lpbData += sizeof(WORD);
                }
            }

        // State changes
        DWORD dwNumStateChanges = lpTrack->dwGetNumStateChanges();

        *(DWORD*)lpbData = dwNumStateChanges;               lpbData += sizeof(DWORD);

        for(DWORD dwStateChange = 0; dwStateChange < dwNumStateChanges; dwStateChange++)
            {
            const CTRStateChange& StateChange = lpTrack->GetStateChange(dwStateChange);

            *(WORD*)lpbData = (WORD)StateChange.dwGetStateId();   lpbData += sizeof(WORD);

            // Anim dispatches
            DWORD dwNumAnimDispatch = StateChange.dwGetNumAnimDispatch();

            *(WORD*)lpbData = (WORD)dwNumAnimDispatch;            lpbData += sizeof(WORD);

            for(DWORD dwAnimDispatch = 0; dwAnimDispatch < dwNumAnimDispatch; dwAnimDispatch++)
                {
                DWORD   dwTag;
                const   CTRAnimDispatch& AnimDispatch = StateChange.GetAnimDispatch(dwAnimDispatch, dwTag, false);

                DWORD   dwStartFrame, dwEndFrame, dwNextAnimation, dwNextFrame;

                AnimDispatch.vGetFrameRange(dwStartFrame, dwEndFrame);
                AnimDispatch.vGetNextAnim(dwNextAnimation, dwNextFrame);

                *(WORD*)lpbData = (WORD)dwStartFrame;             lpbData += sizeof(WORD);
                *(WORD*)lpbData = (WORD)dwEndFrame;               lpbData += sizeof(WORD);
                *(WORD*)lpbData = (WORD)dwNextAnimation;          lpbData += sizeof(WORD);
                *(WORD*)lpbData = (WORD)dwNextFrame;              lpbData += sizeof(WORD);
                }
            }

        // Animation commands
        DWORD dwNumAnimCmde = lpTrack->dwGetNumAnimCommands();

        *(DWORD*)lpbData = dwNumAnimCmde;                   lpbData += sizeof(DWORD);

        for(DWORD dwAnimCmde = 0; dwAnimCmde < dwNumAnimCmde; dwAnimCmde++)
            {
            const CTRAnimCommand& AnimCmde = lpTrack->GetAnimCommand(dwAnimCmde);

            WORD wNumParams = AnimCmde.wGetNumParams();

            *(bit16*)lpbData = AnimCmde.sGetCommand().sValue;   lpbData += sizeof(bit16);
            *(WORD*)lpbData  = wNumParams;                      lpbData += sizeof(WORD);

            for(WORD wI = 0; wI < wNumParams; wI++)
                {
                *(bit16*)lpbData = AnimCmde.sGetParam(wI).sValue;   lpbData += sizeof(bit16);
                }
            }
        }

    //
    // Sounds
    //
    DWORD   dwSndSize;
    BYTE*   lpbSndData = m_lpSndMgr->lpbGetData(dwSndSize);

    memcpy(lpbData, lpbSndData, dwSndSize);                     lpbData += dwSndSize;

    delete [] lpbSndData;

    return lpbDataSave;
}


//----------------------------------------------------------------------------
//
//  Function:   boLoad
//
//  Synopsis:   Load a CTrMvb from disk
//
//  Arguments:  [cstrFileName] -- name of the file on disk
//
//  Return:     the object loaded & initialized or NULL if problems
//
//----------------------------------------------------------------------------
/*static*/ CTrMvb*  CTrMvb::objLoad(CString& cstrFileName)
{
    BYTE*   lpbData = CTrObjBase::lpbLoad(cstrFileName);

    CTrMvb* lpMvb = NULL;

try
    {
    lpMvb = new CTrMvb(lpbData,true);
    }
catch(DWORD)
    {
    delete lpMvb;
    lpMvb = NULL;
    }

    return lpMvb;
}


//----------------------------------------------------------------------------
//
//  Function:   vConstruct
//
//  Synopsis:   Construct a CTrMvb from a block in memory
//
//  Arguments:  [lpObjMvb] -- memory block
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrMvb::vConstruct(BYTE* lpObjMvb)
{
    BYTE* lpbData = lpObjMvb;

    // Meshes hierarchy
    m_lpMeshTree = new TR2_MESHTREE [ 4 * (m_dwNumMeshes - 1)];

    memcpy(m_lpMeshTree, lpbData, 4 * (m_dwNumMeshes - 1) * sizeof(TR2_MESHTREE));

    lpbData += 4 * (m_dwNumMeshes - 1) * sizeof(TR2_MESHTREE);

    //
    // Animations data
    //
    m_lpAnimMgr = new CAnimMgr();

    DWORD dwNumAnims = *(DWORD*)lpbData;                lpbData += sizeof(DWORD);

    // Loop through all the animations for the moveable
    for(long dwNumAnim = 0; dwNumAnim < (long)dwNumAnims; dwNumAnim++)
        {
        // Params of the track
        DWORD   dwNumFrames = *(WORD*)lpbData;          lpbData += sizeof(WORD);
        BYTE    bFrameRate = (BYTE)*(WORD*)lpbData;           lpbData += sizeof(WORD);
        DWORD   dwNextTrackIndex = *(WORD*)lpbData;     lpbData += sizeof(WORD);
        DWORD   dwNextTrackFrame = *(WORD*)lpbData;     lpbData += sizeof(WORD);
        DWORD   dwStateId = *(WORD*)lpbData;            lpbData += sizeof(WORD);
        short   sTRUnknown = *(short*)lpbData;          lpbData += sizeof(short);
        long    lTRSpeed = *(long*)lpbData;             lpbData += sizeof(long);
        float   fTRAcceleration = *(float*)lpbData;     lpbData += sizeof(float);
		bitu8	abUnknown2[8]; memcpy(abUnknown2, lpbData, sizeof(abUnknown2));	lpbData += sizeof(abUnknown2);
        DWORD	dwRealNumAnimCommands = *(DWORD*)lpbData;lpbData += sizeof(DWORD);
        DWORD   dwAnimNumKeys = *(DWORD*)lpbData;       lpbData += sizeof(DWORD);

        // Build the track
        CTRAnimTrack* lpTrack = new CTRAnimTrack(dwAnimNumKeys,dwNumFrames,bFrameRate,(FLOAT)30.0);

        lpTrack->vSetStateId(dwStateId);
        lpTrack->vSetPlayParams(sTRUnknown, lTRSpeed, fTRAcceleration);
		lpTrack->vSetAddParams((bitu16)dwRealNumAnimCommands, abUnknown2);

        // Loop through all the keys of the track
        for(DWORD dwKey = 0; dwKey < dwAnimNumKeys; dwKey++)
            {
            float   fBBLoX = *(float*)lpbData;          lpbData += sizeof(float);
            float   fBBLoY = *(float*)lpbData;          lpbData += sizeof(float);
            float   fBBLoZ = *(float*)lpbData;          lpbData += sizeof(float);
            float   fBBHiX = *(float*)lpbData;          lpbData += sizeof(float);
            float   fBBHiY = *(float*)lpbData;          lpbData += sizeof(float);
            float   fBBHiZ = *(float*)lpbData;          lpbData += sizeof(float);

            float   fTransX = *(float*)lpbData;         lpbData += sizeof(float);
            float   fTransY = *(float*)lpbData;         lpbData += sizeof(float);
            float   fTransZ = *(float*)lpbData;         lpbData += sizeof(float);

            // Build the key
            CAnimKey Key(dwKey * bFrameRate,m_dwNumMeshes,CBoundingBox(CVect3D(fBBLoX,fBBLoY,fBBLoZ),CVect3D(fBBHiX,fBBHiY,fBBHiZ)));

            // Loop through all the meshes of the key
            for(DWORD dwMesh = 0; dwMesh < m_dwNumMeshes; dwMesh++)
                {
                float fQuatX    = *(float*)lpbData;     lpbData += sizeof(float);
                float fQuatY    = *(float*)lpbData;     lpbData += sizeof(float);
                float fQuatZ    = *(float*)lpbData;     lpbData += sizeof(float);
                float fQuatS    = *(float*)lpbData;     lpbData += sizeof(float);
                WORD  wTRAngle1 = *(WORD*)lpbData;      lpbData += sizeof(WORD);
                WORD  wTRAngle2 = *(WORD*)lpbData;      lpbData += sizeof(WORD);

                CVect3D vecTrans(fTransX,fTransY,fTransZ);

                CQuaternion quat(fQuatX,fQuatY,fQuatZ,fQuatS);

                Key.vPushData(CAnimData(quat,vecTrans,wTRAngle1,wTRAngle2));

                fTransX = fTransY = fTransZ = (float)0.0;
                }

            // Add the key to the track
            lpTrack->vPushKey(Key);
            }

        // Build the state changes / anim dispatches of the track
        DWORD dwNumStateChanges = *(DWORD*)lpbData;         lpbData += sizeof(DWORD);

        for(DWORD dwStateChange = 0; dwStateChange < dwNumStateChanges; dwStateChange++)
            {
            DWORD   dwStateId = *(WORD*)lpbData;            lpbData += sizeof(WORD);

            CTRStateChange  stateChange(dwStateId);

            DWORD   dwNumAnimDispatch = *(WORD*)lpbData;    lpbData += sizeof(WORD);

            for(DWORD dwAnimDispatch = 0; dwAnimDispatch < dwNumAnimDispatch; dwAnimDispatch++)
                {
                DWORD   dwStartFrame    = *(WORD*)lpbData;  lpbData += sizeof(WORD);
                DWORD   dwEndFrame      = *(WORD*)lpbData;  lpbData += sizeof(WORD);
                DWORD   dwNextAnimation = *(WORD*)lpbData;  lpbData += sizeof(WORD);
                DWORD   dwNextFrame     = *(WORD*)lpbData;  lpbData += sizeof(WORD);

                stateChange.vPushAnimDispatch(CTRAnimDispatch(dwStartFrame, dwEndFrame, dwNextAnimation, dwNextFrame));
                }

            lpTrack->vPushStateChange(stateChange);
            }

        // Build the anim commands of the track
        DWORD dwNumAnimCmde = *(DWORD*)lpbData;             lpbData += sizeof(DWORD);

        for(DWORD dwAnimCommand = 0; dwAnimCommand < dwNumAnimCmde; dwAnimCommand++)
            {
            bit16   sCommand = *(bit16*)lpbData;            lpbData += sizeof(bit16);
            WORD    wNumParams = *(WORD*)lpbData;           lpbData += sizeof(WORD);

            CTRAnimCommand  AnimCommand(TR2_ANIM_COMMAND(sCommand), wNumParams);

            for(WORD wI = 0; wI < wNumParams; wI++)
                {
                bit16   sParam = *(bit16*)lpbData;          lpbData += sizeof(bit16);
                AnimCommand.vSetParam(wI,TR2_ANIM_COMMAND(sParam));
                }

            lpTrack->vPushAnimCommand(AnimCommand);
            }

        // Add the track to the anim manager
        m_lpAnimMgr->vPushTrack(lpTrack);

        // Set the next anim / next frame of the track
        m_lpAnimMgr->vSetNextTrack(dwNumAnim,dwNextTrackIndex,dwNextTrackFrame);
        }

    //
    // Sounds
    //
    m_lpSndMgr = new CTrSndMgr(lpbData);

    lpbData += m_lpSndMgr->dwGetDataSize();
}


//----------------------------------------------------------------------------
//
//  Function:   vConstruct
//
//  Synopsis:   Construct a CTrMvb from a TR moveable
//
//  Arguments:  [lpMoveable] -- moveable to convert
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrMvb::vConstruct(CTRFile* lpTRFile, TR2_MOVEABLE* lpMoveable)
{
    // Collect the meshes
    DWORD           dwNumMeshes = lpMoveable->wNumMeshes;
    TR123_MESH**    lpLstMeshes = new TR123_MESH* [ dwNumMeshes ];
    enumTRType      trType = lpTRFile->GetTRType();

    for(DWORD dwI = 0; dwI < dwNumMeshes; dwI++)
        {
        DWORD dwNumMesh = lpTRFile->GetTRFile().tr123File.lpMeshPointers[lpMoveable->wStartingMesh + dwI];

        lpLstMeshes[dwI] = lpTRFile->GetTRFile().tr123File.lplpMeshes[dwNumMesh];
        }

    CTrLstMsh::vConstruct(lpTRFile,dwNumMeshes,lpLstMeshes);

    delete [] lpLstMeshes;

    // Set the dummy flag
    for(dwI = 0; dwI < dwNumMeshes; dwI++)
        {
        DWORD dwNumMesh = lpTRFile->GetTRFile().tr123File.lpMeshPointers[lpMoveable->wStartingMesh + dwI];

        if(dwNumMesh  ||  (dwNumMesh == 0  &&
            ((trType >= TR3_FILE  &&  lpMoveable->dwObjectId == 1  &&  dwI == 1)  ||
             (trType < TR3_FILE  &&  lpMoveable->dwObjectId == 0)))
           )
            {
            }
        else
            {
			CTRFile::vReleaseMeshMemory( &m_lpLstMeshes[ dwI ] );

            CTRFile::vInitializeMesh(&m_lpLstMeshes[ dwI ],0,0,0,false,true);

            CTRFile::vComputeMeshCollisionSphere(&m_lpLstMeshes[ dwI ]);

            m_lpLstMeshes[dwI].boIsDummy = true;    // dummy mesh
            }
        }

    // Object Id
    vSetObjectId(lpMoveable->dwObjectId);

	m_dwStartingMesh = lpMoveable->wStartingMesh;

    // Meshes hierarchy
    TR2_MESHTREE* lpMeshTree = &lpTRFile->GetTRFile().tr123File.lpMeshTrees[lpMoveable->dwMeshTree];

    vSetMeshTree(lpMeshTree);

    //
    // Animations data
    //
    TR_FILE&    trFile = lpTRFile->GetTRFile();

    m_lpAnimMgr = new CAnimMgr();

    if(lpMoveable->wAnimation == 0xFFFF)
        {
        m_boAnimatedByEngine = true;
        }
    else
        {
        m_boAnimatedByEngine = false;

        long    dwStartingAnim = lpMoveable->wAnimation;
        long    dwNumAnims = lpTRFile->dwGetNumAnimsForMoveable(lpMoveable) + dwStartingAnim;

        // Loop through all the animations for the moveable
        for(long dwNumAnim = dwStartingAnim; dwNumAnim < dwNumAnims; dwNumAnim++)
            {
            TR2_ANIMATION* lpAnim = &trFile.tr123File.lpAnimations[dwNumAnim];

            DWORD dwFrameOffset = lpAnim->dwFrameOffset / 2;
            DWORD dwFrameStep   = lpAnim->bFrameSize;
            DWORD dwNumFrames;

            if(trType == TR1_FILE)
                {
                dwFrameStep = trFile.tr123File.lpFrames[dwFrameOffset + 9] * 2 + 10;
                }

            dwNumFrames = lpAnim->wFrameEnd - lpAnim->wFrameStart + 1;

            DWORD dwAnimNumKeys = (dwNumFrames - 1) / lpAnim->bFrameRate + 1;

            if((dwNumFrames - 1) % lpAnim->bFrameRate) dwAnimNumKeys++;

            // Build the track
            CTRAnimTrack* lpTrack = new CTRAnimTrack(dwAnimNumKeys,dwNumFrames,lpAnim->bFrameRate,(FLOAT)30.0);

            // Loop through all the keys of the track
            for(DWORD dwKey = 0; dwKey < dwAnimNumKeys; dwKey++)
                {
                WORD    wNumAnimatedMeshes;

                bitu16* lpFrame = &trFile.tr123File.lpFrames[dwFrameOffset];

                lpFrame += dwKey * dwFrameStep;

                bit16   sBBLoX  = (bit16) *lpFrame; lpFrame++;
                bit16   sBBHiX  = (bit16) *lpFrame; lpFrame++;
                bit16   sBBLoY  = (bit16) *lpFrame; lpFrame++;
                bit16   sBBHiY  = (bit16) *lpFrame; lpFrame++;
                bit16   sBBLoZ  = (bit16) *lpFrame; lpFrame++;
                bit16   sBBHiZ  = (bit16) *lpFrame; lpFrame++;

                bit16   sTransX = (bit16) *lpFrame; lpFrame++;
                bit16   sTransY = (bit16) *lpFrame; lpFrame++;
                bit16   sTransZ = (bit16) *lpFrame; lpFrame++;

                if(trType == TR1_FILE)
                    {
                    wNumAnimatedMeshes = *lpFrame; lpFrame++;
                    }
                else wNumAnimatedMeshes = lpMoveable->wNumMeshes;

                // Build the key
                CAnimKey Key(dwKey * lpAnim->bFrameRate,lpMoveable->wNumMeshes,CBoundingBox(CVect3D(sBBLoX,(float)-sBBHiY,(float)-sBBHiZ),CVect3D(sBBHiX,(float)-sBBLoY,(float)-sBBLoZ)));

                // Loop through all the meshes of the key
                for(DWORD dwMesh = 0; dwMesh < lpMoveable->wNumMeshes; dwMesh++)
                    {
                    FLOAT fAngleX = 0.0, fAngleY = 0.0, fAngleZ = 0.0;
                    WORD  wTRAngle1 = 0, wTRAngle2 = 0;

                    if(((trType != TR1_FILE)  ||  (trType == TR1_FILE  &&  (dwMesh < wNumAnimatedMeshes))))
                        {
                        CTRFile::vGetRotAngleForFrame(trType,&lpFrame,fAngleX,fAngleY,fAngleZ,wTRAngle1,wTRAngle2);

                        fAngleX *= (FLOAT)(_PI_ / 180.0);
                        fAngleY *= (FLOAT)(_PI_ / 180.0);
                        fAngleZ *= (FLOAT)(_PI_ / 180.0);
                        }

                    CVect3D vecTrans(sTransX,(float)-sTransY,(float)-sTransZ);

                    Key.vPushData(CAnimData(fAngleX,-fAngleY,-fAngleZ,vecTrans,wTRAngle1,wTRAngle2));
                    sTransX = sTransY = sTransZ = 0;
                    }

                if(dwNumAnim != dwNumAnims - 1)
                    {
                    //!ASSERT(lpFrame <= &trFile.tr123File.lpFrames[trFile.tr123File.lpAnimations[dwNumAnim + 1].dwFrameOffset / 2]); 
                    }

                // Add the key to the track
                lpTrack->vPushKey(Key);
                }


            // Set extra parameters to the track
            lpTrack->vSetStateId(lpAnim->wStateId);
            lpTrack->vSetPlayParams(lpAnim->Unknown, lpAnim->sSpeed, lpAnim->sAccelHi + lpAnim->wAccelLo / 65536.0f);
			lpTrack->vSetAddParams(lpAnim->wNumAnimCommands, lpAnim->Unknown2);

            // Build the state changes / anim dispatches of the track
            for(DWORD dwNumStateChanges = 0; dwNumStateChanges < lpAnim->wNumStateChanges; dwNumStateChanges++)
                {
                TR2_STATE_CHANGE*   lpStateChange = &trFile.tr123File.lpStateChanges[lpAnim->wStateChangeOffset + dwNumStateChanges];

                CTRStateChange  stateChange(lpStateChange->wStateId);

                for(DWORD dwNumAnimDispatch = 0; dwNumAnimDispatch < lpStateChange->wNumAnimDispatches; dwNumAnimDispatch++)
                    {
                    TR2_ANIM_DISPATCH*  lpAnimDispatch = &trFile.tr123File.lpAnimDispatches[lpStateChange->wAnimDispatch + dwNumAnimDispatch];
                    TR2_ANIMATION*  lpNextAnim;
                    
                    if(lpAnimDispatch->sNextAnimation < (bit16)trFile.tr123File.dwNumAnimations)
                        lpNextAnim = &trFile.tr123File.lpAnimations[lpAnimDispatch->sNextAnimation];
                    else
                        lpNextAnim = &trFile.tr123File.lpAnimations[lpAnimDispatch->sNextAnimation - lpMoveable->wAnimation];

                    DWORD   dwStartFrame    = (DWORD)lpAnimDispatch->sLow - lpAnim->wFrameStart;
                    DWORD   dwEndFrame      = (DWORD)lpAnimDispatch->sHigh - lpAnim->wFrameStart - 1;
                    DWORD   dwNextAnimation = (DWORD)lpAnimDispatch->sNextAnimation - lpMoveable->wAnimation;
                    DWORD   dwNextFrame     = (DWORD)lpAnimDispatch->sNextFrame - lpNextAnim->wFrameStart;

                    stateChange.vPushAnimDispatch(CTRAnimDispatch(dwStartFrame, dwEndFrame, dwNextAnimation, dwNextFrame));
                    }

                lpTrack->vPushStateChange(stateChange);
                }

            // Build the anim commands of the track
            TR2_ANIM_COMMAND* lpAnimCmde = &trFile.tr123File.lpAnimCommands[lpAnim->wAnimCommand];

			if( lpAnim->wNumAnimCommands < 0x100 /*trFile.tr123File.dwNumAnimCommands*/ )
				{
				for(DWORD dwNumAnimCommands = 0; dwNumAnimCommands < lpAnim->wNumAnimCommands; dwNumAnimCommands++)
					{
					CTRAnimCommand  AnimCommand(lpAnimCmde, lpAnim->wFrameStart);

					lpTrack->vPushAnimCommand(AnimCommand);

					WORD    wNumParams = AnimCommand.wGetNumParams();

					if(wNumParams != -1) lpAnimCmde += wNumParams + 1;
					}
				}
			else
				{
				char sz[123];
				sprintf(sz,"%ld - %ld\n",lpAnim->wNumAnimCommands,trFile.tr123File.dwNumAnimCommands);
				OutputDebugString( sz );
				}

            // Add the track to the anim manager
            m_lpAnimMgr->vPushTrack(lpTrack);

            WORD    wNextAnimation = lpAnim->wNextAnimation;

            // ASSERT(wNextAnimation < trFile.tr123File.dwNumAnimations);  // have seen this condition
                                                                        // broken in title.tr2 & willsden.tr2
                                                                        // from TR3 gold !

            DWORD   dwNextFrame;

            if(wNextAnimation < trFile.tr123File.dwNumAnimations)
                {
                TR2_ANIMATION*  lpNextAnim = &trFile.tr123File.lpAnimations[wNextAnimation];
                dwNextFrame = lpAnim->wNextFrame;

                //if(dwNextFrame < lpNextAnim->wFrameStart)   dwNextFrame = lpNextAnim->wFrameStart;
                //if(dwNextFrame > lpNextAnim->wFrameEnd)     dwNextFrame = lpNextAnim->wFrameStart;

                dwNextFrame -= lpNextAnim->wFrameStart;
                wNextAnimation -= (WORD)dwStartingAnim;
                }
            else
                {
                wNextAnimation = 0;
                dwNextFrame = 0;
                }

            // Set the next anim / next frame of the track
            m_lpAnimMgr->vSetNextTrack(dwNumAnim - dwStartingAnim,wNextAnimation,dwNextFrame);
            }
        }

    //
    // Sounds
    //
    m_lpSndMgr = new CTrSndMgr;

    if(trType != TR1_FILE)
        {
        DWORD   dwNumTracks = m_lpAnimMgr->dwGetNumTracks();

        for(DWORD dwTrack = 0; dwTrack < dwNumTracks; dwTrack++)
            {
            CTRAnimTrack* lpTrack = static_cast<CTRAnimTrack*>(m_lpAnimMgr->lpGetTrack(dwTrack));

            DWORD dwNumAnimCmde = lpTrack->dwGetNumAnimCommands();

            for(DWORD dwAnimCmde = 0; dwAnimCmde < dwNumAnimCmde; dwAnimCmde++)
                {
                const CTRAnimCommand& AnimCmde = lpTrack->GetAnimCommand(dwAnimCmde);

                TR2_ANIM_COMMAND sCommand = AnimCmde.sGetCommand();

                switch(sCommand.sValue)
                    {
                    case ANIMCMD_PLAYSOUNDONFRAME_2:
                        {
                        DWORD   dwInternalIndex = AnimCmde.sGetParam(1).sValue & 0x3fff;

                        ASSERT(dwInternalIndex < sizeof(TR2_SOUNDMAP));

                        short   sSoundDetail = trFile.tr123File.SoundMap.tr2.aData[dwInternalIndex];

                        //ASSERT(sSoundDetail != -1);
                        if(sSoundDetail != -1)
                            {
                            TR2_SOUND_DETAILS*  lpSoundDetail = &trFile.tr123File.lpSoundDetails[sSoundDetail];
                            DWORD               dwFirstSoundIndex = trFile.tr123File.lpSampleIndices[lpSoundDetail->sSample];

                            m_lpSndMgr->boAddSound(dwInternalIndex,lpSoundDetail,dwFirstSoundIndex);
                            }
                        break;
                        }
                    }
                }
            }
        }
}


//----------------------------------------------------------------------------
//
//  Function:   vComputeBoundingBox
//
//  Synopsis:   Compute the bounding box of the moveable
//
//  Arguments:  [BoundingBox] -- on return, holds the bounding box extents
//              [dwNumMesh] -- if -1, compute the BB for all the meshes of
//               the moveable, else compute the BB only for the dwNumMesh mesh
//
//  Return:     -
//
//----------------------------------------------------------------------------
/*virtual*/ void    CTrMvb::vComputeBoundingBox(CBoundingBox& BoundingBox,DWORD dwNumMesh, CMat4x4* lpMat)
{
    #define STACK_DEPTHMAX 10

    float   Stack[3 * STACK_DEPTHMAX];
    DWORD   dwStackPointer = 0;
    float   fTransX = 0, fTransY = 0, fTransZ = 0;

    TR2_MESHTREE*   lpMeshTree = m_lpMeshTree;
    TR123_MESH*     lpMesh = m_lpLstMeshes;
    
    BoundingBox.vSetBB(CVect3D((FLOAT)1e20,(FLOAT)1e20,(FLOAT)1e20),CVect3D((FLOAT)-1e20,(FLOAT)-1e20,(FLOAT)-1e20));

    if((dwNumMesh == -1  ||  dwNumMesh == 0)  &&  lpMesh)
        {
        CTrLstMsh::vComputeBoundingBox(0, BoundingBox, lpMat);
        }

   for(DWORD dwI = 1; dwI < m_dwNumMeshes; dwI++)
        {
        lpMesh = &m_lpLstMeshes[dwI];

        bitu32  dwStack = lpMeshTree->lCoord; lpMeshTree++;
        bit32   lTransX = lpMeshTree->lCoord; lpMeshTree++;
        bit32   lTransY = lpMeshTree->lCoord; lpMeshTree++;
        bit32   lTransZ = lpMeshTree->lCoord; lpMeshTree++;

        if(dwStack & 1)
            {
            if(dwStackPointer == 0)
                {
                //!AfxMessageBox("Moveable bounding box calculation: stack pointer = 0 with pop operation !",MB_OK | MB_ICONSTOP | MB_TOPMOST);
				//! Have seen this condition occure for mvb #418 in TR4 levels
				//! However, it seems that the moveable is rendered correctly... Go figure !
                }
            else
                {
                dwStackPointer -= 3;
                fTransX = Stack[dwStackPointer + 0];
                fTransY = Stack[dwStackPointer + 1];
                fTransZ = Stack[dwStackPointer + 2];
                }
            }

        if(dwStack & 2)
            {
            if(dwStackPointer + 3 > STACK_DEPTHMAX)
                {
                AfxMessageBox("Moveable bounding box calculation: stack overflow !",MB_OK | MB_ICONSTOP | MB_TOPMOST);
                }
            else
                {
                Stack[dwStackPointer + 0] = fTransX;
                Stack[dwStackPointer + 1] = fTransY;
                Stack[dwStackPointer + 2] = fTransZ;
                dwStackPointer += 3;
                }
            }

        fTransX +=  (float)lTransX;
        fTransY += -(float)lTransY;
        fTransZ += -(float)lTransZ;

        if((dwNumMesh == -1  ||  dwNumMesh == dwI)  &&  lpMesh)
            {
            CBoundingBox    BBMesh;

            CTrLstMsh::vComputeBoundingBox(dwI, BBMesh, lpMat);

            if(dwNumMesh == -1) BBMesh.vTranslate(CVect3D(fTransX, fTransY, fTransZ));
            
            BoundingBox.vEnlarge(BBMesh);
            }
        }
}


//----------------------------------------------------------------------------
//
//  Function:   vSetMeshTree
//
//  Synopsis:   Define the hierarchy of the moveable
//
//  Arguments:  [lpMeshTree] -- hierarchy of the moveable
//              [dwNumMeshes] -- number of meshes comprising this moveable.
//              If 0, use the data member m_dwNumMeshes instead
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrMvb::vSetMeshTree(TR2_MESHTREE* lpMeshTree, DWORD dwNumMeshes)
{
    if(m_lpMeshTree) delete [] m_lpMeshTree;

    if(dwNumMeshes == 0) dwNumMeshes = m_dwNumMeshes;

    m_lpMeshTree = new TR2_MESHTREE [ 4 * (dwNumMeshes - 1)];

    memcpy(m_lpMeshTree, lpMeshTree, 4 * (dwNumMeshes - 1) * sizeof(TR2_MESHTREE));
}


//----------------------------------------------------------------------------
//
//  Function:   vReleaseAllAnims
//
//  Synopsis:   Release all the anims for this moveable
//
//  Arguments:  -
//
//  Return:     -
//
//----------------------------------------------------------------------------
void    CTrMvb::vReleaseAllAnims()
{
    delete m_lpAnimMgr;
    delete m_lpSndMgr;

    m_lpAnimMgr = new CAnimMgr();
    m_lpSndMgr  = new CTrSndMgr();
}


//----------------------------------------------------------------------------
//
//  Function:   vSetResMgr
//
//  Synopsis:   Set the CResMoveable class holding editing informations about
//				the moveable
//
//  Arguments:  [lpResMoveable] -- class information
//
//  Return:     -
//
//----------------------------------------------------------------------------
void	CTrMvb::vSetResMgr( CResMoveable* lpResMoveable )
{
	if( m_lpResMgr != NULL ) delete m_lpResMgr;

	m_lpResMgr = lpResMoveable;
}
