/**********************************************************************************************************************/
/*                                                QUATERNION CLASS                                                    */
/* 26/08/00, M : creation of the file                                                                                 */
/**********************************************************************************************************************/

#ifndef   _QUATERNION_INL_
#define   _QUATERNION_INL_

/************************************************ INCLUDE *************************************************************/

#include  "Quaternion.h"
#include  <math.h>                                          // sqrt

/**********************************************************************************************************************/
/*                                                QUATERNION                                                          */
/**********************************************************************************************************************/

/************************************************ constructors ********************************************************/
// initialize quaternion
// 26/08/00, M
/**********************************************************************************************************************/

inline CQuaternion::CQuaternion()                                               // do nothing
  {}

inline CQuaternion::CQuaternion(const CVect3D& v3Vector,const FLOAT fScalar)
  { m_v3Vector = v3Vector; m_fScalar = fScalar; }                               // vector+scalar

inline CQuaternion::CQuaternion(const CVect4D& v4Quat)
  { m_v3Vector[_X_] = v4Quat[_X_]; m_v3Vector[_Y_] = v4Quat[_Y_];               // x/y/z=vector, w=scalar
    m_v3Vector[_Z_] = v4Quat[_Z_]; m_fScalar       = v4Quat[_W_]; }

inline CQuaternion::CQuaternion(const FLOAT fX,const FLOAT fY,const FLOAT fZ,const FLOAT fW)
  { m_v3Vector[_X_] = fX; m_v3Vector[_Y_] = fY; m_v3Vector[_Z_] = fZ; m_fScalar = fW; }

inline CQuaternion::CQuaternion(const CQuaternion& quatQ)                       // copy
  { m_v3Vector = quatQ.m_v3Vector; m_fScalar = quatQ.m_fScalar; }

/************************************************ assignment operators ************************************************/
// assign or modify quaternion
// 26/08/00, M
/**********************************************************************************************************************/

inline CQuaternion& CQuaternion::operator  = (const CQuaternion& quatQ)         // assignment
  { m_v3Vector = quatQ.m_v3Vector; m_fScalar = quatQ.m_fScalar; return *this; }

inline CQuaternion& CQuaternion::operator += (const CQuaternion& quatQ)         // incrementation
  { vAdd(this,quatQ); return *this; }

inline CQuaternion& CQuaternion::operator -= (const CQuaternion& quatQ)         // decrementation
  { vSub(this,quatQ); return *this; }

inline CQuaternion& CQuaternion::operator *= (const CQuaternion& quatQ)         // multiplication
  { vMul(this,quatQ); return *this; }

inline CQuaternion& CQuaternion::operator *= (const FLOAT fK)                   // multiplication by a constant
  { vMul(this,fK);    return *this; }

inline CQuaternion& CQuaternion::operator /= (const FLOAT fK)                   // division by a constant
  { vDiv(this,fK);    return *this; }

/************************************************ friends operators ***************************************************/
// quaternion maths
// 26/08/00, M
/**********************************************************************************************************************/

inline CQuaternion  operator  - (const CQuaternion& quatQ)                      // -Q
  { CQuaternion quatTmp; vNeg(&quatTmp,quatQ);    return(quatTmp); }

inline CQuaternion  operator  * (const FLOAT fK,const CQuaternion& quatQ)       // k*Q
  { CQuaternion quatTmp; vMul(&quatTmp,quatQ,fK); return(quatTmp); }

inline CQuaternion  operator  * (const CQuaternion& quatQ,const FLOAT fK)       // Q*k
  { CQuaternion quatTmp; vMul(&quatTmp,quatQ,fK); return(quatTmp); }

inline CQuaternion  operator  / (const CQuaternion& quatQ,const FLOAT fK)       // Q/k
  { CQuaternion quatTmp; vDiv(&quatTmp,quatQ,fK); return(quatTmp); }

inline CQuaternion  operator  + (const CQuaternion& quatQ1,const CQuaternion& quatQ2)     // Q1+Q2
  { CQuaternion quatTmp; vAdd(&quatTmp,quatQ1,quatQ2); return(quatTmp); }

inline CQuaternion  operator  - (const CQuaternion& quatQ1,const CQuaternion& quatQ2)     // Q1-Q2
  { CQuaternion quatTmp; vSub(&quatTmp,quatQ1,quatQ2); return(quatTmp); }

inline CQuaternion  operator  * (const CQuaternion& quatQ1,const CQuaternion& quatQ2)     // Q1*Q2
  { CQuaternion quatTmp; vMul(&quatTmp,quatQ1,quatQ2); return(quatTmp); }

inline bool         operator == (const CQuaternion& quatQ1,const CQuaternion& quatQ2)     // Q1==Q2
  { return ((quatQ2.m_fScalar == quatQ2.m_fScalar) && (quatQ1.m_v3Vector == quatQ2.m_v3Vector)); }

inline bool         operator != (const CQuaternion& quatQ1,const CQuaternion& quatQ2)     // Q1!=Q2
  { return ((quatQ2.m_fScalar != quatQ2.m_fScalar) || (quatQ1.m_v3Vector != quatQ2.m_v3Vector)); }

/************************************************ special functions ***************************************************/
// quaternion length
// 03/10/00, M
/**********************************************************************************************************************/

inline FLOAT CQuaternion::fLength () const
  { return(FLOAT)sqrt(fLength2()); }

inline FLOAT CQuaternion::fLength2() const
  { return(m_fScalar*m_fScalar + m_v3Vector*m_v3Vector); }

/************************************************ get/set  ************************************************************/
// quaternion access
// 26/08/00, M
/**********************************************************************************************************************/

inline FLOAT CQuaternion::fGetScalar()
  { return m_fScalar; }

inline CVect3D& CQuaternion::v3GetVector()
  { return m_v3Vector; }

inline void CQuaternion::vSetScalar(const FLOAT fScalar)
  { m_fScalar = fScalar; }

inline void CQuaternion::vSetVector(const CVect3D& v3Vector)
  { m_v3Vector = v3Vector; }

inline void CQuaternion::vSetVector(const FLOAT fX,const FLOAT fY,const FLOAT fZ)
  { m_v3Vector.vSet(fX,fY,fZ); }

inline void CQuaternion::vSet(const FLOAT fX,const FLOAT fY,const FLOAT fZ,const FLOAT fW)
  { m_v3Vector.vSet(fX,fY,fZ); m_fScalar = fW; }

inline void CQuaternion::vSet(const CVect4D& v4Quat)
  { m_v3Vector.vSet(v4Quat[_X_],v4Quat[_Y_],v4Quat[_Z_]); m_fScalar = v4Quat[_W_]; }

inline void CQuaternion::vSet(const CQuaternion& quatQ)
  { m_v3Vector.vSet(quatQ.m_v3Vector); m_fScalar = quatQ.m_fScalar; }

/**********************************************************************************************************************/

#endif
